<?

 /*****************************************************************************
 * Filename: include/archive.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 @include_once("Archive/Tar.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");

 class Archive
 {
  function __construct()
  {
   global $_path;

   $this->e=new Errors();
   $this->e->l->read("archive");

   $this->methods=Array("GZ", "BZ2", "TAR", "TGZ", "TAR.GZ", "TAR.BZ2", "RAR", "ZIP");

   $this->clear();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  function clear()
  {
   $this->string="";
   $this->files=Array();
  }


  /*
   * String
   */

  function getString()
  {
   return $this->string;
  }

  function setString($string)
  {
   $this->string=$string;
   return true;
  }


  /*
   * Get files info
   */

  function getFiles()
  {
   return array_keys($this->files);
  }

  function getType($file)
  {
   if(!isset($this->files[$file]))
   {
    $this->e->add("Unable to get type for unknown file: \"".$file."\"", "error");
    return false;
   }

   return $this->files[$file]["type"];
  }

  function getTime($file)
  {
   if(!isset($this->files[$file]))
   {
    $this->e->add("Unable to get time for unknown file: \"".$file."\"", "error");
    return false;
   }

   return $this->files[$file]["time"];
  }

  function getSize($file)
  {
   if(!isset($this->files[$file]))
   {
    $this->e->add("Unable to get size for unknown file: \"".$file."\"", "error");
    return false;
   }

   if($this->files[$file]["type"]=="directory")
   {
    $this->e->add("Unable to get size for directory: \"".$file."\"", "error");
    return false;
   }

   return $this->files[$file]["size"];
  }

  function getContents($file)
  {
   if(!isset($this->files[$file]))
   {
    $this->e->add("Unable to get contents for unknown file: \"".$file."\"", "error");
    return false;
   }

   if($this->files[$file]["type"]=="directory")
   {
    $this->e->add("Unable to get contents for directory: \"".$file."\"", "error");
    return false;
   }

   return $this->files[$file]["contents"];
  }


  /*
   * Set file info
   */

  function _setFile($file, $contents, $time=false)
  {
   if(isset($this->files[$file]))
    $this->e->add("File: \"".$file."\" already added", "warning");

   $this->files[$file]=Array();
   $this->files[$file]["type"]="file";
   $this->files[$file]["time"]=($time!==false?$time:time());
   $this->files[$file]["size"]=strlen($contents);
   $this->files[$file]["contents"]=$contents;

   return true;
  }

  function _setDirectory($dir, $time=false)
  {
   if(substr($dir, -1)!="/")
    $dir.="/";

   if(isset($this->files[$dir]))
    $this->e->add("Directory: \"".$dir."\" already added", "warning");

   $this->files[$dir]=Array();
   $this->files[$dir]["type"]="directory";
   $this->files[$dir]["time"]=($time!==false?$time:time());

   return true;
  }

  function _setFilePath($file)
  {
   if(preg_match_all("!([^/]+/)!", $file, $tmp))
   {
    $dir="";
    foreach($tmp[1] as $tmp2)
    {
     $dir.=$tmp2;
     if(!isset($this->files[$dir]))
      if(!$this->_setDirectory($dir))
       return false;
    }
   }

   return true;
  }

  function addFile($file, $strippath=false)
  {
   if(is_dir($file)&&substr($file, -1)!="/")
    $file.="/";

   if($strippath)
    $archivefile=str_replace($strippath, "", $file);
   else
    $archivefile=$file;

   if(!FS::existsS($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(isset($this->files[$archivefile]))
   {
    $this->e->add("File: \"".$archivefile."\" already in archive", "error");
    return false;
   }

   if(!$this->_setFilePath($archivefile))
    return false;

   if(!is_dir($file)) //file
    if(!$this->_setFile($archivefile, FS::readS($file), FS::getTimeS($file, "modified")))
     return false;

   return true;
  }

  function addDirectory($dir=false, $strippath=false)
  {
   if(!$dir)
    $dir="./";
   else if(substr($dir, -1)!="/")
    $dir.="/";

   if($strippath===true)
    $strippath=$dir;

   if(!FS::existsS($dir))
   {
    $this->e->add("Directory: \"".$dir."\" doesn't exist", "error");
    return false;
   }

   $return=true;
   foreach(FS::filesS($dir, true) as $file)
    if(!$this->addFile($file, $strippath))
     $return=false;

   return $return;
  }

  function rerootFiles($oldroot, $newroot=false)
  {
   if(!$oldroot||$oldroot=="./")
    $oldroot="";
   else if(substr($oldroot, -1)!="/")
    $oldroot.="/";

   if(!$newroot)
    $newroot="";
   else if(substr($newroot, -1)!="/")
    $newroot.="/";

   $remove=Array();
   if(preg_match_all("!([^/]+/)!", $oldroot, $tmp))
   {
    $dir="";
    foreach($tmp[1] as $tmp2)
    {
     $dir.=$tmp2;
     $remove[]=$dir;
     if(!isset($this->files[$dir]))
     {
      $this->e->add("Unable to re-root, root: \"".$dir."\" doesn't exist in archive", "error");
      return false;
     }
    }
   }

   $oldrootregexp="!^".preg_quote($oldroot, "!")."!";

   foreach($this->getFiles() as $file)
   {
    if(!in_array($file, $remove)&&!preg_match($oldrootregexp, $file))
    {
     $this->e->add("Unable to re-root file: \"".$file."\" outside of root: \"".$oldroot."\"", "error");
     return false;
    }
   }

   foreach($remove as $dir)
    unset($this->files[$dir]);

   if($newroot)
   {
    $this->files[$newroot]=Array();
    $this->files[$newroot]["time"]=time();
    $this->files[$newroot]["type"]="directory";
   }

   foreach($this->getFiles() as $file)
   {
    if($file==$newroot)
     continue;

    $newfile=$newroot.preg_replace($oldrootregexp, "", $file);
    $this->files[$newfile]=&$this->files[$file];
    unset($this->files[$file]);
   }

   return true;
  }


  /*
   * Save files
   */

  function saveFile($file, $dir=false, $overwrite=false)
  {
   if(!$dir)
    $dir="./";
   else if(substr($dir, -1)!="/")
    $dir.="/";

   if(!FS::existsS($dir))
   {
    if(!@mkdir($dir, 0755, true))
    {
     $this->e->add("Unable to create destination directory: \"".$dir."\"", "error");
     return false;
    }
   }

   if(!isset($this->files[$file]))
   {
    $this->e->add("File: \"".$file."\" not in archive", "error");
    return false;
   }

   if(!$overwrite&&FS::existsS($dir.$file))
   {
    $this->e->add("File: \"".$dir.$file."\" already exists", "error");
    return false;
   }

   if($this->files[$file]["type"]=="directory")
   {
    if(!FS::existsS($dir.$file)) //don't try to overwrite it
    {
     if(!@mkdir($dir.$file, 0755, true))
     {
      $this->e->add("Unable to create archive directory: \"".$dir.$file."\"", "error");
      return false;
     }
     else
      $this->e->add("Archive directory: \"".$dir.$file."\" created", "note", "error");
    }
    else
     $this->e->add("Not creating archive directory: \"".$dir.$file."\"", "note", "error");
   }
   else
   {
    if(FS::existsS($dir.$file))
     $overwriting=true;
    else
     $overwriting=false;

    if(FS::writeS($dir.$file, $this->files[$file]["contents"], ($overwriting?"overwrite":false))===false)
    {
     $this->e->add("Unable to write archive file: \"".$dir.$file."\"", "error");
     return false;
    }

    if(!FS::setTimeS($dir.$file, $this->files[$file]["time"], "modified"))
    {
     $this->e->add("Unable to change file time for file: \"".$dir.$file."\"", "error");
     return false;
    }

    if(FS::sizeS($dir.$file)!=$this->files[$file]["size"])
    {
     $this->e->add("File size of file: \"".$dir.$file."\" (".filesize($dir.$file).") differs from original (".$this->files[$file]["size"].")", "error");
     return false;
    }

    if($overwriting)
     $this->e->add("File: \"".$dir.$file."\" overwritten", "note", "error");
    else
     $this->e->add("File: \"".$dir.$file."\" saved", "note", "error");
   }

   return true;
  }

  function saveFiles($dir=false, $overwrite=false)
  {
   $return=true;
   foreach($this->getFiles() as $file)
    if(!$this->saveFile($file, $dir, $overwrite))
     $return=false;

   return $return;
  }


  /*
   * Method
   */

  function _detectMethod($file)
  {
   if(preg_match("/\.([a-z0-9]+\.[a-z0-9]+)$/i", $file, $tmp)&&in_array(strtoupper($tmp[1]), $this->methods))
    return strtoupper($tmp[1]);

   if(preg_match("/\.([a-z0-9]+)$/i", $file, $tmp)&&in_array(strtoupper($tmp[1]), $this->methods))
    return strtoupper($tmp[1]);

   $this->e->add("Unable to detect compression method for file: \"".$file."\"", "error");
   return false;
  }

  function _checkSupport($method)
  {
   if(!in_array($method, $this->methods))
   {
    $this->e->add("Unknown compression method: \"".$method."\"", "error");
    return false;
   }

   switch($method)
   {
    case "GZ":
    {
     if(
      !function_exists("gzopen")||
      !function_exists("gzread")||
      !function_exists("gzwrite")||
      !function_exists("gzclose")
     )
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }

     break;
    }
    case "BZ2":
    {
     if(
      !function_exists("bzopen")||
      !function_exists("bzread")||
      !function_exists("bzwrite")||
      !function_exists("bzclose")
     )
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }

     break;
    }
    case "TAR":
    {
     if(!class_exists("Archive_Tar"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      $this->e->add("Tar support needs Pear Archive_Tar: http://pear.php.net/package/Archive_Tar/", "warning");
      return false;
     }

     break;
    }
    case "TGZ":
    case "TAR.GZ":
    {
     if(!$this->_checkSupport("TAR")||!$this->_checkSupport("GZ"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }
     break;
    }
    case "TAR.BZ2":
    {
     if(!$this->_checkSupport("TAR")||!$this->_checkSupport("BZ2"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }
     break;
    }
    case "RAR":
    {
     if(!class_exists("RarArchive"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }
     break;
    }
    case "ZIP":
    {
     if(!class_exists("ZipArchive"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"", "error");
      return false;
     }
     break;
    }
    default:
    {
     $this->e->add("Can't detect PHP compression support for compression method: \"".$method."\"", "error");
     return false;
    }
   }

   return true;
  }


  /*
   * Decopress
   */

  function open($file, $method=false)
  {
   $this->clear();

   if(!FS::existsS($file))
   {
    $this->e->add("File doesn't exist: \"".$file."\"", "error");
    return false;
   }

   if(!$method&&!($method=$this->_detectMethod($file)))
    return false;
   else
    $method=strtoupper($method);

   if(!$this->_checkSupport($method))
    return false;

   switch($method)
   {
    case "GZ":
    {
     if(!($gz=@gzopen($file, "rb")))
     {
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error");
      return false;
     }

     $contents="";
     while($tmp=gzread($gz, 8192))
      $contents.=$tmp;

     gzclose($gz);

     if(!$contents)
     {
      $this->e->add("Unable to get ".$method." contents from file: \"".$file."\"", "error");
      return false;
     }

     if(!$this->setString($contents))
      return false;

     break;
    }
    case "BZ2":
    {
     if(!($bz=@bzopen($file, "r")))
     {
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error");
      return false;
     }

     $contents="";
     while($tmp=bzread($bz, 8192))
      $contents.=$tmp;

     bzclose($bz);

     if(!$contents)
     {
      $this->e->add("Unable to get ".$method." contents from file: \"".$file."\"", "error");
      return false;
     }

     if(!$this->setString($contents))
      return false;

     break;
    }
    case "TAR":
    {
     $archive=new Archive_Tar($file);
     if(!($files=$archive->listContent()))
     {
      $this->e->add("Unable to get ".$method." contents from file: \"".$file."\"", "error");
      return false;
     }

     foreach($files as $entry)
     {
      $archivefile=$entry["filename"];

      if(!$entry["size"])
      {
       if(!$this->_setDirectory($archivefile, $entry["mtime"]))
        return false;
      }
      else
      {
       if(!$this->_setFilePath($archivefile))
        return false;
       if(!$this->_setFile($archivefile, $archive->extractInString($archivefile), $entry["mtime"]))
        return false;
      }
     }

     ksort($this->files);
     break;
    }
    case "TGZ":
    case "TAR.GZ":
    {
     if(!$this->open($file, "GZ"))
      return false;

     $tmpfile=FS::tmpfilenameS("archive");
     if(!FS::writeS($tmpfile, $this->string))
     {
      $this->e->add("Unable to write temporary ".$method." file: \"".$tmpfile."\"", "error");
      return false;
     }

     if(!$this->open($tmpfile, "TAR"))
     {
      $this->e->clear(); //make sure TAR errors are not shown
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error");
      FS::deleteS($tmpfile);
      return false;
     }

     unlink($tmpfile);
     break;
    }
    case "TAR.BZ2":
    {
     if(!$this->open($file, "BZ2"))
      return false;

     $tmpfile=FS::tmpfilenameS("archive");
     if(!FS::writeS($tmpfile, $this->string))
     {
      $this->e->add("Unable to write temporary ".$method." file: \"".$tmpfile."\"", "error");
      return false;
     }

     if(!$this->open($tmpfile, "TAR"))
     {
      $this->e->clear(); //make sure TAR errors are not shown
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error");
      FS::deleteS($tmpfile);
      return false;
     }

     unlink($tmpfile);
     break;
    }
    case "RAR":
    {
     if(!($archive=RarArchive::open($file)))
     {
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error");
      return false;
     }

     $timezone=DT::getTimezoneS();
     DT::setTimezoneS(DT::getTimezoneS(true)); //force system timezone, i.e. assume archive timezone is in system timezone
     foreach($archive->getEntries() as $entry)
     {
      $archivefile=$entry->getName();

      if($entry->isDirectory())
      {
       $archivefile.="/";
       if(!$this->_setDirectory($archivefile, DT::dateConvertS($entry->getFileTime(), "Y-m-d H:i:s")))
       {
        DT::setTimezoneS($timezone); //restore timezone
        return false;
       }
      }
      else
      {
       if(!($stream=$entry->getStream()))
       {
        DT::setTimezoneS($timezone); //restore timezone
        $this->e->add("Unable to get ".$method." contents from file: \"".$file."\"", "error");
        return false;
       }

       $contents="";
       while(!feof($stream))
       {
        if(($buffer=fread($stream, 8192))===false)
        {
        DT::setTimezoneS($timezone); //restore timezone
         $this->e->add("Unable to get ".$method." contents from file: \"".$file."\"", "error");
         fclose($stream);
         return false;
        }

        $contents.=$buffer;
       }

       fclose($stream);

       if(!$this->_setFile($archivefile, $contents, DT::dateConvertS($entry->getFileTime(), "Y-m-d H:i:s")))
       {
        DT::setTimezoneS($timezone); //restore timezone
        return false;
       }
      }
     }

     DT::setTimezoneS($timezone); //restore timezone
     $archive->close();

     ksort($this->files);

     break;
    }
    case "ZIP":
    {
     $archive=new ZipArchive();

     if(($error=$archive->open($file, ZipArchive::CHECKCONS))!==true)
     {
      $this->e->add("Unable to open ".$method." file: \"".$file."\"", "error", $error);
      return false;
     }

     for($entry=0;$entry<$archive->numFiles;$entry++)
     {
      if(!($stats=$archive->statIndex($entry)))
      {
       $this->e->add("Unable to get ".$method." entry stats from file: \"".$file."\"", "error");
       return false;
      }

      $archivefile=$stats["name"];

      if(!$stats["size"])
      {
       if(!$this->_setDirectory($archivefile, $stats["mtime"]))
        return false;
      }
      else
      {
       if(!$this->_setFile($archivefile, $archive->getFromName($archivefile), $stats["mtime"]))
        return false;
      }
     }

     $archive->close();

     ksort($this->files);
     break;
    }
    default:
    {
     $this->e->add("Can't open ".$method." file: \"".$file."\"", "error");
     return false;
    }
   }

   return true;
  }

  function openString($string, $method)
  {
   $method=strtoupper($method);

   if(!$this->_checkSupport($method))
    return false;

   $tmpfile=FS::tmpfilenameS("archive").".".strtolower($method);
   if(!FS::writeS($tmpfile, $string))
   {
    $this->e->add("Unable to write temporary ".$method." file: \"".$tmpfile."\"", "error");
    return false;
   }

   if(!$this->open($tmpfile, $method))
   {
    FS::deleteS($tmpfile);
    return false;
   }

   FS::deleteS($tmpfile);
   return true;
  }


  /*
   * Compress
   */

  function output($method, $headers=false, $filename=false)
  {
   $method=strtoupper($method);

   if(!$this->_checkSupport($method))
    return false;

   switch($method)
   {
    case "GZ":
    {
     if(!$this->string)
     {
      $this->e->add("No string set for archiving", "error");
      return false;
     }

     $tmpfile=FS::tmpfilenameS("archive");
     if(!($gz=gzopen($tmpfile, "wb")))
     {
      $this->e->add("Unable to open temporary ".$method." file: \"".$tmpfile."\"", "error");
      return false;
     }
     gzwrite($gz, $this->string);
     gzclose($gz);

     $output=FS::readS($tmpfile);
     unlink($tmpfile);

     $mime="application/x-gzip";
     break;
    }
    case "BZ2":
    {
     if(!$this->string)
     {
      $this->e->add("No string set for archiving", "error");
      return false;
     }

     $tmpfile=FS::tmpfilenameS("archive");
     if(!($bz=bzopen($tmpfile, "w")))
     {
      $this->e->add("Unable to open temporary ".$method." file: \"".$tmpfile."\"", "error");
      return false;
     }
     bzwrite($bz, $this->string);
     bzclose($bz);

     $output=FS::readS($tmpfile);
     unlink($tmpfile);

     $mime="application/bz2";
     break;
    }
    case "TAR":
    {
     if(!$this->files)
     {
      $this->e->add("No files set for archiving", "error");
      return false;
     }

     $tmpfile=FS::tmpfilenameS("archive");
     $archive=new Archive_Tar($tmpfile);
     foreach($this->getFiles() as $file)
     {
      if($this->files[$file]["type"]=="directory")
      {
       if(!$archive->addString($file, false, $this->files[$file]["time"]))
       {
        $this->e->add("Unable to add directory: \"".$file."\" to ".$method." archive", "error");
        unlink($tmpfile);
        return false;
       }
      }
      else
      {
       if(!$archive->addString($file, $this->files[$file]["contents"], $this->files[$file]["time"]))
       {
        $this->e->add("Unable to add file: \"".$file."\" to ".$method." archive", "error");
        unlink($tmpfile);
        return false;
       }
      }
     }

     $output=FS::readS($tmpfile);
     unlink($tmpfile);

     $mime="application/x-tar";
     break;
    }
    case "TGZ":
    case "TAR.GZ":
    {
     if(!($output=$this->output("TAR")))
      return false;

     $this->setString($output);

     if(!($output=$this->output("GZ")))
      return false;

     $mime="application/x-gzip";
     break;
    }
    case "TAR.BZ2":
    {
     if(!($output=$this->output("TAR")))
      return false;

     $this->setString($output);

     if(!($output=$this->output("BZ2")))
      return false;

     $mime="application/bz2";
     break;
    }
    case "ZIP":
    {
     if(!$this->files)
     {
      $this->e->add("No files set for archiving", "error");
      return false;
     }

     $tmpfile=FS::tmpfilenameS("archive");
     $archive=new ZipArchive();
     if(($error=$archive->open($tmpfile, ZipArchive::CREATE|ZipArchive::EXCL))!==true)
     {
      $this->e->add("Unable to open temporary ".$method." file: \"".$tmpfile."\"", "error", $error);
      return false;
     }

     foreach($this->getFiles() as $file)
     {
      if($this->files[$file]["type"]=="directory")
      {
       if(!$archive->addEmptyDir($file))
       {
        $this->e->add("Unable to add directory: \"".$file."\" to ".$method." archive", "error");
        unlink($tmpfile);
        return false;
       }
      }
      else
      {
       if(!$archive->addFromString($file, $this->files[$file]["contents"]))
       {
        $this->e->add("Unable to add file: \"".$file."\" to ".$method." archive", "error");
        unlink($tmpfile);
        return false;
       }
      }
     }

     $archive->close();

     $output=FS::readS($tmpfile);
     unlink($tmpfile);

     $mime="application/zip";
     break;
    }
    default:
    {
     $this->e->add("Can't create ".$method." output", "error");
     return false;
    }
   }

   if($headers)
   {
    $h=new Headers();
    if($filename)
     $h->file($filename, $mime, strlen($output));
    else
    {
     $h->type($mime);
     $h->size(strlen($output));
    }
   }

   return $output;
  }

  function save($file, $method=false)
  {
   if(!$method&&!($method=$this->_detectMethod($file)))
    return false;
   else
    $method=strtoupper($method);

   if(FS::existsS($file))
   {
    $this->e->add("File: \"".$file."\" already exists", "error");
    return false;
   }

   if(!($contents=$this->output($method)))
    return false;

   if(!FS::writeS($file, $contents))
   {
    $this->e->add("Can't write ".$method." data to: \"".$file."\"", "error");
    return false;
   }

   return true;
  }

 }
?>
