<?

 /*****************************************************************************
 * Filename: include/dt.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");

 class DT
 {
  function __construct()
  {
   $this->e=new Errors();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  static function __callStatic($method, $arguments)
  {
   $object=new self();
   $object->e->logging(false); //disable logging for static calls

   if(substr($method, -1)=="S") //method must have a S suffix
   {
    $method=substr($method, 0, -1); //remove S suffix
    if(method_exists($object, $method)) //check if dynamic method exists
     return call_user_func_array(Array($object, $method), $arguments);
   }

   $bt=debug_backtrace();
   print "<br/>\n<b>Fatal error</b>:  Call to undefined method ".get_class($object)."::".$method." in <b>".$bt[0]["file"]."</b> on line <b>".$bt[0]["line"]."</b><br />\n";
   exit;
  }


  function dateExtract($date, $format)
  {
   if($format=="c") //ISO 8601
    $format="Y-m-d\TH:i:sP";
   else if($format=="r") //RFC 2822
    $format="D, j M Y H:i:s O";

   //array values
   if(is_array($date))
   {
    foreach($date as $k => $v)
     $date[$k]=$this->dateExtract($v, $format);

    return $date;
   }

   //current format support:
   $formatConvert=Array(
    "/d/" => "(\d{2})", //days
    "/j/" => "(\d{1,2})",
    "/D/" => "(\w{3})",
    "/m/" => "(\d{2})", //months
    "/n/" => "(\d{1,2})",
    "/M/" => "(\w{3})",
    "/F/" => "(\w{3,10})",
    "/Y/" => "(\d{4})", //years
    "/y/" => "(\d{2})",
    "/H/" => "(\d{2})", //hours
    "/G/" => "(\d{1,2})",
    "/i/" => "(\d{2})", //minutes
    "/s/" => "(\d{2})", //seconds
    "/W/" => "(\d{1,2})", //weeks
    "/z/" => "(\d{1,3})", //days (per year)
    "/U/" => "(\d+)", //unix timestamp
    "/O/" => "(Z|UTC|GMT|\+?\-?\d{2}\d{2})", //UTC offset
    "/P/" => "(Z|UTC|GMT|\+?\-?\d{2}:\d{2})",
   );

   if(preg_match_all("/\\\\([a-z])/i", $format, $escaped)) //escaped characters in format
   {
    $tmp=preg_replace("/\\\\[a-z]/i", "~", $format); //replace escaped chars with \
    $regexp=preg_replace(array_keys($formatConvert), $formatConvert, preg_quote($tmp));

    if(!preg_match_all("/[a-z]/i", preg_quote($tmp), $keys))
     return false;

    foreach($escaped[1] as $char)
     $regexp=preg_replace("/~/", $char, $regexp, 1);

    unset($tmp);
   }
   else //no escaped characters
   {
    $regexp=preg_replace(array_keys($formatConvert), $formatConvert, preg_quote($format));

    if(!preg_match_all("/[a-z]/i", preg_quote($format), $keys))
     return false;
   }

   if(!preg_match("/".$regexp."/", $date, $values))
    return false;

   //ugly workaround for pregmatch
   $tmp=Array();
   $keys=$keys[0];
   for($c=1;$c<sizeof($values);$c++)
    $tmp[]=$values[$c];
   $values=$tmp;
   unset($tmp);

   if(($key=array_search("U", $keys))!==false) //unix timestamp (seconds sinds 1970, in UTC)
    list($year, $month, $day, $hour, $minute, $second)=explode(" ", date("Y m d H i s", $values[$key])); //overwrite all values
   else //no unix timestamp, parse values
   {
    if(($key=array_search("d", $keys))!==false) //numeric day of the month
     $day=$values[$key];
    else if(($key=array_search("j", $keys))!==false) //numeric day of the month without leading zero
     $day=$values[$key];
    else
     $day=1;

    if(($key=array_search("m", $keys))!==false) //numeric month
     $month=$values[$key];
    else if(($key=array_search("n", $keys))!==false) //numeric month without leading zero
     $month=$values[$key];
    else if(($key=array_search("M", $keys))!==false||($key=array_search("F", $keys))!==false) //month name
    {
     $tmp=date_parse($values[$key]); //convert (short) month name to number
     if($tmp["month"]) //success
      $month=$tmp["month"];
     else //failed
      $month=1;
    }
    else
     $month=1;

    if(($key=array_search("Y", $keys))!==false) //4 digit year
     $year=$values[$key];
    else if(($key=array_search("y", $keys))!==false) //2 digit year
    {
     $year=$values[$key];
     if($year>=50) //we use it from 1950 till 2049
      $year+=1900;
     else
      $year+=2000;
    }
    else
     $year=1970;

    if(($key=array_search("H", $keys))!==false) //24 hours
     $hour=$values[$key];
    else if(($key=array_search("G", $keys))!==false) //24 hours without leading zero
     $hour=$values[$key];
  //implement 12 hour clock (h+g) with am/pm (a/A)
    else
     $hour=0;

    if(($key=array_search("i", $keys))!==false) //2 digit minute
     $minute=$values[$key];
    else
     $minute=0;

    if(($key=array_search("s", $keys))!==false) //2 digit second
     $second=$values[$key];
    else
     $second=0;

    if(($key=array_search("W", $keys))!==false) //numeric week of the year
    {
     $date=(($values[$key]-1)*7); //determine day of the year
     $date=mktime(0, 0, 0, 1, $date, $year); //determine date based on set year

     if(date("W", mktime(0,0,0,12,31,$year))==53) //53 weeks year
     {
      if(($weekday=date("N", $date))>1) //not monday
       $date-=86400*($weekday-1); //round date to previous monday
     }
     else //52 weeks year
     {
      if(($weekday=date("N", $date))>1) //not monday
       $date+=86400*(8-$weekday); //round date to next monday
     }

     if(array_search("d", $keys)===false&&array_search("j", $keys)===false) //day not set
      $day=date("d", $date); //determine day of the month

     if(array_search("m", $keys)===false&&array_search("n", $keys)===false) //month not set
      $month=date("m", $date); //determine month

     $year=date("Y", $date); //overwrite year in case start of the week false in another year
    }

    if(($key=array_search("z", $keys))!==false) //numeric day of the year
    {
     $date=$values[$key];
     $date=mktime(0, 0, 0, 1, $date, $year); //determine date based on set year

     if(array_search("d", $keys)===false&&array_search("j", $keys)===false) //day not set
      $day=date("d", $date); //determine day of the month

     if(array_search("m", $keys)===false&&array_search("n", $keys)===false) //month not set
      $month=date("m", $date); //determine month

     $year=date("Y", $date); //overwrite year in case start of the week false in another year
    }

    if(($key=array_search("P", $keys))!==false||($key=array_search("O", $keys))!==false) //UTC offset
    {
     if($values[$key]=="Z"||$values[$key]=="UTC"||$values[$key]=="GMT") //UTC
      $offset=Array("sign" => "+", "hour" => 0, "minute" => 0); //convert from UTC
     else if(preg_match("/(\+|\-)?(\d{2}):?(\d{2})/", $values[$key], $m)) //offset
      $offset=Array("sign" => ($m[1]=="-"?"-":"+"), "hour" => $m[2], "minute" => $m[3]); //convert from offset
    }
   }

   if(isset($offset)) //UTC offset
   {
    //get UTC time
    if($offset["sign"]=="-") //before UTC
     $time=gmmktime($hour+$offset["hour"], $minute+$offset["minute"], $second, $month, $day, $year);
    else //after UTC
     $time=gmmktime($hour-$offset["hour"], $minute-$offset["minute"], $second, $month, $day, $year);

    if(date("O", $time)!=implode("", $offset)) //offset doesn't match offset for current timezone
     list($year, $month, $day, $hour, $minute, $second)=explode(" ", date("Y m d H i s", $time));  //get local time (all values needed in case of date cross)
   }

   return Array(
    "year" => (int)$year,
    "month" => (int)$month,
    "day" => (int)$day,
    "hour" => (int)$hour,
    "minute" => (int)$minute,
    "second" => (int)$second);
  }

  function dateConvert($date, $inputformat, $outputformat=false)
  {
   //array values
   if(is_array($date))
   {
    foreach($date as $k => $v)
     $date[$k]=$this->dateConvert($v, $inputformat, $outputformat);

    return $date;
   }

   if(!($date=$this->dateExtract($date, $inputformat)))
    return false;

   $output=mktime($date["hour"], $date["minute"], $date["second"], $date["month"], $date["day"], $date["year"]);

   if($outputformat)
    $output=date($outputformat, $output);

   return $output;
  }

  function dateDelta($date, $delta, $interval, $format)
  {
   //array values
   if(is_array($date))
   {
    foreach($date as $k => $v)
     $date[$k]=$this->dateDelta($v, $delta, $interval, $format);

    return $date;
   }

   if(!($date=$this->dateExtract($date, $format)))
    return false;

   switch(strtolower($interval))
   {
    case "second":
    case "seconds":
    {
     $date["second"]+=$delta;
     break;
    }
    case "minute":
    case "minutes":
    {
     $date["minute"]+=$delta;
     break;
    }
    case "hour":
    case "hours":
    {
     $date["hour"]+=$delta;
     break;
    }
    case "day":
    case "days":
    {
     $date["day"]+=$delta;
     break;
    }
    case "week":
    case "weeks":
    {
     $date["day"]+=$delta*7;
     break;
    }
    case "month":
    case "months":
    {
     $date["month"]+=$delta;
     break;
    }
    case "year":
    case "years":
    {
     $date["year"]+=$delta;
     break;
    }
    default:
    {
     $this->e->add("Unknown date delta interval: ".$interval, "error");
     return $date;
    }
   }


   $date=mktime($date["hour"], $date["minute"], $date["second"], $date["month"], $date["day"], $date["year"]);

   return date($format, $date);
  }


  function getTimezone($system=false)
  {
   $timezone=false;
   if($system)
   {
    if(!($timezone=exec("date +%Z"))) //get UNIX timezone
    {
     if(isset($_ENV["TZ"])) //get environment timezone
      $timezone=$_ENV["TZ"];
     else //get current timezone
      $timezone=date_default_timezone_get();
    }
   }
   else //get current timezone
    $timezone=date_default_timezone_get();

   if(!strpos($timezone, "/")) //not a proper timezone name, assume abbreviation
    if(!($timezone=timezone_name_from_abbr($timezone))) //get timezone name
     $timezone=date_default_timezone_get(); //get current

   return $timezone;
  }

  function setTimezone($timezone)
  {
   $timezones=timezone_identifiers_list();

   if(!in_array($timezone, $timezones))
   {
    $this->e->add("Unknown timezone: ".$timezone, "error");
    return false;
   }

   if($timezone!=$this->getTimezone())
    date_default_timezone_set($timezone);

   return true;
  }

  function timezoneConvert($date, $inputtimezone, $outputtimezone, $dateformat)
  {
   if($inputtimezone==$outputtimezone)
    return $date;

   //array values
   if(is_array($date))
   {
    foreach($date as $k => $v)
     $date[$k]=$this->timezoneConvert($v, $inputtimezone, $outputtimezone, $dateformat);

    return $date;
   }

   $timezones=timezone_identifiers_list();
   $current=date_default_timezone_get();

   if(!in_array($inputtimezone, $timezones))
   {
    $this->e->add("Unknown timezone: ".$inputtimezone, "error");
    return $date;
   }

   if(!in_array($outputtimezone, $timezones))
   {
    $this->e->add("Unknown timezone: ".$outputtimezone, "error");
    return $date;
   }

   if($current!=$inputtimezone)
    date_default_timezone_set($inputtimezone);

   $date=$this->dateConvert($date, $dateformat);

   date_default_timezone_set($outputtimezone);

   $date=date($dateformat, $date);

   if($current!=date_default_timezone_get())
    date_default_timezone_set($current);

   return $date;
  }

  function timeAgo($date, $dateformat, $interval=false)
  {
   $now=time();

   $intervals=Array(
    "years" => 31536000,
    "months" => 2419200, //shortest month
    "weeks" => 604800,
    "days" => 86400,
    "hours" => 3600,
    "minutes" => 60,
    "seconds" => 1,
   );

   if(($date=$this->dateConvert($date, $dateformat))===false) //incorrect date
    return false;

   $ago=$now-$date;

   if(!$interval||!isset($intervals[$interval])) //auto detect interval
   {
    foreach($intervals as $interval => $seconds)
     if(abs($ago)>=$seconds)
      break;
   }

   if($interval!="months")
   {
    if($ago>0) //past
     $ago=floor($ago/$intervals[$interval]);
    else if($ago<0) //future
     $ago=ceil($ago/$intervals[$interval]);
    else //now
     $ago=0;
   }
   else //months
   {
    if($ago>0) //past
    {
     $ago=(date("Y", $now)-date("Y", $date))*12;
     $ago+=date("m", $now)-date("m", $date);
    }
    else if($ago<0) //future
    {
     $ago=(date("Y", $now)-date("Y", $date))*12;
     $ago+=date("m", $now)-date("m", $date);

     if(!$ago) // -0 workaround
      $ago="-0";
    }
    else //now
     $ago=0;
   }

   if($ago>1) //more than 1 second ago
    $ago=$ago." ".$interval." ago";
   else if($ago==1) //1 second ago
    $ago=$ago." ".substr($interval, 0, -1)." ago";
   else if($ago==0) //0 seconds ago
   {
    if("$ago"==="-0") //future
    {
     if($interval=="seconds")
      $ago="within a second";
     else if($interval=="minutes")
      $ago="in less than a minute";
     else if($interval=="hours")
      $ago="in less than an hour";
     else if($interval=="days")
      $ago="today";
     else if($interval=="weeks")
      $ago="coming week";
     else if($interval=="months")
      $ago="coming month";
     else if($interval=="years")
      $ago="coming year";
    }
    else //past
    {
     if($interval=="seconds")
      $ago="just now";
     else if($interval=="minutes")
      $ago="less than a minute ago";
     else if($interval=="hours")
      $ago="less than an hour ago";
     else if($interval=="days")
      $ago="today";
     else if($interval=="weeks")
      $ago="this week";
     else if($interval=="months")
      $ago="this month";
     else if($interval=="years")
      $ago="this year";
    }
   }
   else if($ago==-1) //in 1 second
    $ago="in ".abs($ago)." ".substr($interval, 0, -1);
   else //in more than 1 second
    $ago="in ".abs($ago)." ".$interval;

   return $ago;
  }
 }

?>
