<?

 /*****************************************************************************
 * Filename: include/image.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."functions.php");

 define("IMAGETYPE_GD", 666);
 define("IMAGETYPE_TGA", 667);

 class Image
 {
  function __construct()
  {
   $this->e=new Errors();
   $this->e->l->read("image");

   if(!function_exists("gd_info"))
    $this->e->add("Unable to process images, no GD support", "system");

   $this->data=false;
   $this->type=false;
   $this->file=false;
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }


  function load($file)
  {
   if(!file_exists($file))
   {
    $this->e->add("Image file: \"".$file."\" doesn't exist");
    return false;
   }

   $type=$this->_detectType($file);

   if($type==IMAGETYPE_TGA)
   {
    if(!($data=$this->imagecreatefromtga($file)))
     return false;

    return $this->_setImageData($data, IMAGETYPE_GD, $file);
   }

   $data=@file_get_contents($file);
   return $this->_setImageData($data, $type, $file);
  }


  function loadData(&$d, $field)
  {
   if($d->check("type", "localfile", $field))
   {
    $d->setShowFields($field);

    if(!$d->get())
    {
     $this->e->add($d->e);
     return false;
    }

    if(!$d->next())
    {
     $this->e->add("No image data for module: \"".$d->module."\"", "error");
     return false;
    }

    return $this->load($d->field[$field]["localpath"]["directory"].$d->v[$field]);
   }
   else //normal image type
   {
    $d->setShowFields($field, $field."Data", $field."PHPImageType");

    if(!$d->get())
    {
     $this->e->add($d->e);
     return false;
    }

    if(!$d->next()||!$d->v[$field."Data"])
    {
     $this->e->add("No image data for module: \"".$d->module."\"", "error");
     return false;
    }

    return $this->_setImageData($d->v[$field."Data"], $d->v[$field."PHPImageType"], $d->v[$field]);
   }
  }


  function redirectData(&$d, $field)
  {
   global $_path;

   if(!$d->check("type", "localfile", $field))
   {
    $this->e->add("Image redirect is only available for local file field", "error", "Field: \"".$field."\" for module: \"".$d->module."\" isn't a local file field");
    return false;
   }

   $d->setShowFields($field);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$d->next())
   {
    $this->e->add("No image data for module: \"".$d->module."\"", "error");
    return false;
   }

   header("Location: ".$_path["HTML"]["prefix"].$d->field[$field]["localpath"]["directory"].$d->v[$field]);
   return true;
  }


  function save($file=false, $type=false)
  {
   if($this->data===false)
   {
    $this->e->add("Unable to save, image data not loaded");
    return false;
   }

   if(!$file)
   {
    if(!$this->file)
    {
     $this->e->add("No filename given");
     return false;
    }

    $file=$this->file;
   }

   if(!($data=$this->output($file, false, $type)))
    return false;

   if(@file_put_contents($file, $data)===false)
   {
    $this->e->add("Can't save image file: \"".$file."\"");
    return false;
   }

   return true;
  }


  function output($file=false, $headers=true, $type=false)
  {
   if($this->data===false)
   {
    $this->e->add("Unable to output, image data not loaded");
    return false;
   }

   if(!$file)
   {
    if(!$this->file)
    {
     $this->e->add("No filename given");
     return false;
    }

    $file=$this->file;
   }

   if(!$type)
    if(!($type=$this->_detectType($file)))
     return false;

   if($type==$this->type)
    $data=$this->data;
   else
   {
    $data=false;

    if($this->type!=IMAGETYPE_GD)
     $this->data=imagecreatefromstring($this->data);

    ob_start();
    switch($type)
    {
     case IMAGETYPE_JPEG:
     {
      imagejpeg($this->data);
      break;
     }
     case IMAGETYPE_GIF:
     {
      imagegif($this->data);
      break;
     }
     case IMAGETYPE_PNG:
     {
      imagepng($this->data);
      break;
     }
     default:
     {
      break;
     }
    }
    $data=ob_get_contents();
    ob_end_clean();
   }

   if(!$data)
    return false;

   if($headers)
    Headers::fileS((strpos($this->file, "/")!==false?substr($this->file, strrpos($this->file, "/")+1):$this->file), image_type_to_mime_type($type), strlen($data), "attachment");

   return $data;
  }


  function resize($width=0, $height=0)
  {
   if($this->data===false)
   {
    $this->e->add("Unable to resize, image data not loaded");
    return false;
   }

   if($width<0||$width>5000)
   {
    $this->e->add("Refusing to resize width to: ".$width);
    return false;
   }
   if($height<0||$height>5000)
   {
    $this->e->add("Refusing to resize height to: ".$height);
    return false;
   }

   if($this->type!=IMAGETYPE_GD)
   {
    $this->data=imagecreatefromstring($this->data);
   }

   if($width==0&&$height>0)
    $width=$height*(imagesx($this->data)/imagesy($this->data));
   else if($height==0&&$width>0)
    $height=$width*(imagesy($this->data)/imagesx($this->data));
   else if($height==0&&$width==0)
    return false;

   $tmp=imagecreatetruecolor($width, $height);

   if($this->type==IMAGETYPE_GIF||$this->type==IMAGETYPE_PNG)
   {
    imagecolortransparent($tmp, imagecolorallocate($tmp, 0, 0, 0));

    if($this->type==IMAGETYPE_PNG)
     imagealphablending($tmp, false);
    else
     imagealphablending($tmp, true);

    imagesavealpha($tmp, true);
   }
   else
    imagealphablending($tmp, false);

   imagecopyresampled($tmp, $this->data, 0, 0, 0, 0, $width, $height, imagesx($this->data), imagesy($this->data));

   imagedestroy($this->data);
   $this->data=&$tmp;

   $this->type=IMAGETYPE_GD;

   return true;
  }


  function rotate($angle, $red=0, $green=0, $blue=0)
  {
   if($this->data===false)
   {
    $this->e->add("Unable to rotate, image data not loaded");
    return false;
   }

   if($angle!="left"&&$angle!="right"&&$angle!="counterclockwise"&&$angle!="clockwise"&&!preg_match("/^\d+$/", $angle))
   {
    $this->e->add("Refusing to rotate: ".$angle);
    return false;
   }

   if($this->type!=IMAGETYPE_GD)
   {
    $this->data=imagecreatefromstring($this->data);
    $this->type=IMAGETYPE_GD;
   }

   if($angle=="left"||$angle=="counterclockwise")
    $angle=270;
   else if($angle=="right"||$angle=="clockwise")
    $angle=90;

   $color=imagecolorallocate($this->data, $red, $green, $blue);

   $angle=360-$angle; //make rotation clock wise (imagerotate does counter clockwise...)
   $tmp=imagerotate($this->data, $angle, $color);

   imagedestroy($this->data);
   $this->data=&$tmp;

   return true;
  }


  function imagecreatefromtga($file)
  {
   if(!($tga=file_get_contents($file)))
   {
    $this->e->add("Can't open TGA image file: \"".$file."\"");
    return false;
   }

   //read header
   $idlength=hexdec(bin2hex($tga[0]));
   $imagetype=hexdec(bin2hex($tga[2]));

   $colormaptype=hexdec(bin2hex($tga[1]));
   $colormapoffset=hexdec(bin2hex($tga[3].$tga[4]));
   $colormaplength=hexdec(bin2hex($tga[5].$tga[6]));
   $colormapbitsperpixel=hexdec(bin2hex($tga[7]));

//   d($idlength, $imagetype, $colormaptype, $colormapoffset, $colormaplength, $colormapbitsperpixel);

   $xoffset=hexdec(bin2hex($tga[9].$tga[8]));
   $yoffset=hexdec(bin2hex($tga[11].$tga[10]));
   $width=hexdec(bin2hex($tga[13].$tga[12]));
   $height=hexdec(bin2hex($tga[15].$tga[14]));
   $bitsperpixel=hexdec(bin2hex($tga[16]));
   $pixellength=$bitsperpixel/8;
   $imagedescriptor=hexdec(bin2hex($tga[17]));

//   d($xoffset, $yoffset, $width, $height, $bitsperpixel, $imagedescriptor);
   switch($imagetype)
   {
    case 0: //no image data... whatever
    {
     $this->e->add("TGA image file: \"".$file."\" has no image data");
     return false;
    }
    case 2: //uncompressed, true-color image
    {
     $data=imagecreatetruecolor($width, $height);

     $i=18;
     $i+=$idlength; //skip identification information
     $i+=$colormapoffset+$colormaplength; //skip color map ($colormaptype should be 0)

     $pixellengthmin1=$pixellength-1;

     for($y=0;$y<$height;$y++)
     {
      for($x=0;$x<$width;$x++)
      {
       $pixel="";
       for($c=$i+$pixellengthmin1;$c>=$i;$c--) //read pixellength reverse bits
        $pixel.=$tga[$c];

       //xoffset: 0 left to right, >0 right to left
       //yoffset: 0 bottom to top, >0 top to bottom
       imagesetpixel($data, ($xoffset?$width-$x:$x), ($yoffset?$y:$height-$y), hexdec(bin2hex($pixel)));

       $i+=$pixellength;
      }
     }

     break;
    }
    case 3: //uncompressed, black-and-white image
    {
     $data=imagecreatetruecolor($width, $height);

     $i=18;
     $i+=$idlength; //skip identification information
     $i+=$colormapoffset+$colormaplength; //skip color map ($colormaptype should be 0)

     $pixellengthmin1=$pixellength-1;

     for($y=0;$y<$height;$y++)
     {
      for($x=0;$x<$width;$x++)
      {
       $pixel="";
       for($c=$i+$pixellengthmin1;$c>=$i;$c--) //read pixellength reverse bits
        $pixel.=$tga[$c];

       //xoffset: 0 left to right, >0 right to left
       //yoffset: 0 bottom to top, >0 top to bottom
       //pixel: 00 black, ff white
       imagesetpixel($data, ($xoffset?$width-$x:$x), ($yoffset?$y:$height-$y), (bin2hex($pixel)=="00"?0:16777215));

       $i+=$pixellength;
      }
     }

     break;
    }
    default:
    {
     $this->e->add("TGA image file: \"".$file."\" type not supported");
     return false;
    }
   }

   return $data;
  }

  /*
   *
   * Internal functions
   */

  function _setImageData(&$data, $type, $file)
  {
   $this->data=&$data;
   $this->type=$type;
   $this->file=$file;

   return true;
  }

  function _detectType($file)
  {
   if(preg_match("/\.([a-z]+)$/i", $file, $tmp))
    $tmp=strtolower($tmp[1]);
   else
    $tmp=false;

   switch($tmp)
   {
    case "jpg":
    case "jpeg":
    {
     $type=IMAGETYPE_JPEG;
     break;
    }
    case "gif":
    {
     $type=IMAGETYPE_GIF;
     break;
    }
    case "png":
    {
     $type=IMAGETYPE_PNG;
     break;
    }
    case "tga":
    {
     $type=IMAGETYPE_TGA;
     break;
    }
    default:
    {
     $this->e->add("Unable to detect image type from filename: \"".$file."\"");
     return false;
    }
   }

   return $type;
  }

  function __destruct()
  {
   if($this->type==IMAGETYPE_GD)
    imagedestroy($this->data);
  }

 }

?>
