<?

 /*****************************************************************************
 * Filename: include/modules/workflow.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_workflow extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   /*
    * Business rules
    *
    * Workflow (no parent workflow selected)
    * - Name and first state must be unique
    * - Text is first state
    *
    * State
    * - Text must be unique in current workflow
    * - Requires at least 1 previous decision result
    * - May have multiple previous decision results
    * - May not reuse decision results
    * - May have no previous states
    * - Can be:
    *   - Start: defined in workflow
    *   - Preliminairy: has at least one next decision
    *   - End: has no next decisions, workflow ends here
    *
    * Decision
    * - Text must be unique in current workflow
    * - Requires at least 1 previous state
    * - May have multiple previous states
    * - May not reuse states
    * - May have no previous decisions
    * - May only be preliminairy, i.e. must have a next state
    */

   //maximum number of decision results
   $this->maxDecisionResults=4;

   $this->data->title="Workflow";

   $this->data->defaultFields();

   $this->data->addField("workflow", "nr", "all", "reload", false, false, 0, "Existing workflow");
   $this->data->addReference("workflow", "workflow", "nr", "*:0");

   $this->data->addField("name", "text", "all", "title", 1, 100, false, "Name");

   $this->data->addField("type", "enum", "all", "reload;title", false, false, "State", "Type");
   $this->data->enumOptions("type", "State", "Decision");

   //states who's next step is this decision
   $this->data->addField("previousStates", "multilist", "all", "", 0, 65536, false, "Previous state(s)");
   $this->data->listOptions("previousStates");

   //decision results who's next step is this decision/state
   $this->data->addField("previousDecisionResults", "multilist", "all", "", 0, 65536, false, "Previous decision result(s)");
   $this->data->listOptions("previousDecisionResults");


   $this->data->addField("text", "text", "all", "title", 1, 100, false, "Text");
   $this->data->addField("code", "text", "all", "", 0, 25, false, "Code");

   //generate decisions results, at least 1 required
   for($c=1;$c<=$this->maxDecisionResults;$c++)
    $this->data->addField("decisionResult".$c, "text", "all", "", ($c==1?1:0), 255, false, "Decision results ".$c);


   $this->data->addUnique("text", "workflow", "name", "text");
   $this->data->sortOrder("name", "nr");
  }

  function postInit(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->action!="edit"&&$this->data->action!="add")
    return true;

   //only allow workflow (top level) entries
   $this->data->listOptions("workflow", $this->getWorkflows());

   if(!$this->data->getValue("workflow")) //workflow
   {
    //show (start state) name
    $this->data->resetField("name");

    //hide type
    $this->data->setValue("type", "State"); //force start state
    $this->data->field["type"]["properties"].=";hidden";
   }
   else //use existing workflow
   {
    //hide the field
    $this->data->field["name"]["properties"].=";hidden";

    //show type field
    $this->data->resetField("type");
   }

   if(!$this->data->getValue("workflow")) //workflow
   {
    if(!$this->data->getValue("nr")) //add workflow
    {
     //hide decision results
     for($c=1;$c<=$this->maxDecisionResults;$c++)
      $this->data->field["decisionResult".$c]["properties"].=";hidden";

     //hide previous states
     $this->data->field["previousStates"]["properties"].=";hidden";

     //no decision results required
     $this->data->field["decisionResult1"]["min"]=0;

     //hide previous decision results
     $this->data->field["previousDecisionResults"]["properties"].=";hidden";

     return true;
    }
    else if(!($info=$this->getWorkflowInfo($this->data->getValue("nr"), ($this->data->action=="edit"?$this->data->getValue("nr"):false))))
     return false;
   }
   else
   {
    if(!($info=$this->getWorkflowInfo($this->data->getValue("workflow"), ($this->data->action=="edit"?$this->data->getValue("nr"):false))))
     return false;

    //force workflow name
    $this->data->setValue("name", $info["workflow"]);
   }

   if($this->data->getValue("type")=="State")
   {
    //show code field
    $this->data->resetField("code");

    //hide decision results
    for($c=1;$c<=$this->maxDecisionResults;$c++)
     $this->data->field["decisionResult".$c]["properties"].=";hidden";

    //hide previous states
    $this->data->field["previousStates"]["properties"].=";hidden";

    //no decision results required
    $this->data->field["decisionResult1"]["min"]=0;

    //filter out current decision and used results from this workflow decision results
    $decisionResults=Array();
    foreach($info["decisionResults"] as $result)
    {
     $tmp=explode(" -> ", $result);

     if($tmp[0]!=$this->data->getValue("text")) //not current
      if(!in_array($result, $info["usedDecisionResults"])||in_array($result, $info["usedDecisionResultsByCurrent"])) //not used by others
       $decisionResults[]=$result;
    }

    //show previous decision results and fill with filter decision results  list
    $this->data->resetField("previousDecisionResults");
    $this->data->listOptions("previousDecisionResults", $decisionResults);
   }
   else if($this->data->getValue("type")=="Decision")
   {
    //hide code field
    $this->data->field["code"]["properties"].=";hidden";

    //show decision results
    for($c=1;$c<=$this->maxDecisionResults;$c++)
     $this->data->resetField("decisionResult".$c);

    //filter out current state and used states from this workflow states
    $states=Array();
    foreach($info["states"] as $state)
    {
     if($state!=$this->data->getValue("text")) //not current
      if(!in_array($state, $info["usedStates"])||in_array($state, $info["usedStatesByCurrent"])) //not used by others
       $states[]=$state;
    }

    //show previous decision results and fill with filter decision results  list
    $this->data->resetField("previousStates");
    $this->data->listOptions("previousStates", $states);

    //hide previous decision results
    $this->data->field["previousDecisionResults"]["properties"].=";hidden";
   }

   //show warning if not all decision results are used
   $unused=Array();
   foreach($info["decisionResults"] as $decisionResult)
    if(!in_array($decisionResult, $info["usedDecisionResults"])&&(!$this->data->getValue("previousDecisionResults")||!in_array($decisionResult, $this->data->getValue("previousDecisionResults"))))
     $unused[]=$decisionResult;

   foreach($info["usedDecisionResultsByCurrent"] as $decisionResult)
    if(!$this->data->getValue("previousDecisionResults")||!in_array($decisionResult, $this->data->getValue("previousDecisionResults")))
     $unused[]=$decisionResult;

   if($unused)
    $this->data->e->add("Decision result(s): \"".implode("\", \"", $unused)."\" are not yet used", "note");

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function editPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields()
  {
   if(!$this->data->getValue("workflow")) //workflow
   {
   }
   else
   {
    if($this->data->getValue("type")=="State")
    {
     if(!$this->data->getValue("previousDecisionResults"))
     {
      $this->data->e->add("At least 1 previous decision result is required", "error");
      return false;
     }
    }
    else if($this->data->getValue("type")=="Decision")
    {
     if(!$this->data->getValue("previousStates"))
     {
      $this->data->e->add("At least 1 previous state is required", "error");
      return false;
     }

     $results=Array();
     for($c=1;$c<=$this->maxDecisionResults;$c++)
     {
      if(!$this->data->getValue("decisionResult".$c))
       continue;

      if(in_array($this->data->getValue("decisionResult".$c), $results))
      {
       $this->data->e->add("Duplicate decision result: \"".$this->data->getValue("decisionResult".$c)."\"", "error");
       return false;
      }
      else
       $results[]=$this->data->getValue("decisionResult".$c);
     }
    }
   }


   return true;
  }

  function viewPreGet(){return $this->showTextForCode();}
  function searchPreGet(){return $this->showTextForCode();}

  function showTextForCode()
  {
   //also get text field when gathering code
   if($this->data->check("use", "show", "code")&&!$this->data->check("use", "show", "text"))
    $this->data->field["text"]["use"].=";show";

   return true;
  }

  function viewPostGet(){return $this->setEmptyCodes();}
  function searchPostGet(){return $this->setEmptyCodes();}

  function setEmptyCodes()
  {
   //set text as code if code isn't set
   for($c=0;$c<$this->data->vmax;$c++)
    if(!$this->data->getValue("code", $c))
     $this->data->setValue("code", $this->data->getValue("text", $c), $c);

   return true;
  }


  /*
   * Determine available states for workflow reference
   *
   * Module should define:
   * $this->data->addField("workflow", "nr", "all", "", 1, false, $workflow, "Workflow");
   * $this->data->addReference("workflow", "workflow", "nr", "*:1");
   *
   * where $workflow is the name or the database nr of the workflow it should use
   *
   * $d is the Data object that initialized the module.
   * Add action will always use the first state (workflow definition)
   * Edit allows current state and subsequent decision(s). If a decision result is selected it will set its subsequent state
   */

  function determineState(&$d)
  {
   if(!is_object($d)||get_class($d)!="Data")
   {
    $this->data->e->add("Unable to determine workflow state, no Data object", "error");
    return false;
   }

   if(!($workflowReferences=$d->check("reference", "workflow")))
   {
    $this->data->e->add("Unable to determine workflow state, Data object doesn't contain any workflow references", "error");
    return false;
   }

   foreach($workflowReferences as $workflowReference)
   {
    $workflowField=$d->reference[$workflowReference]["from"];

    if(!($state=$d->getValue($workflowField))) //current state not set, skip this field
     continue;

    if(preg_match("/^\d+\s*$/", $state)) //number (with trailing spaces)
     $state=(int)$state; //remove trailing spaces
    else if(!($state=$this->getWorkflowNR($state))) //assume workflow name
     return false;

    //set list options
    $options=$this->getStateOptions($state, true, ($d->action!="add")); //get options unless in add mode
    $d->listOptions($workflowReference, $options);
   }

   return true;
  }

  function getWorkflows()
  {
   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("workflow", 0);

   $d->setShowFields("name");
   $d->sortOrder("name");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   $info=Array("" => "");
   while($d->next())
    $info[$d->v["nr"]]=$d->v["name"];

   return $info;
  }

  function getWorkflowInfo($workflow, $current)
  {
   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("nr", $workflow);
   $d->setOperant("nr", "OR");
   $d->setValue("workflow", $workflow);

   $d->sortOrder("text");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   $info=Array(
    "workflow" => "",
    "states" => Array(),
    "usedStates" => Array(),
    "usedStatesByCurrent" => Array(),
    "decisions" => Array(),
    "decisionResults" => Array(),
    "usedDecisionResults" => Array(),
    "usedDecisionResultsByCurrent" => Array(),
   );

   while($d->next())
   {
    //get workflow info
    if($d->v["nr"]==$workflow)
     $info["workflow"]=$d->v["name"];

    //get state, decision and decision results info
    if($d->v["type"]=="State")
     $info["states"][$d->v["nr"]]=$d->v["text"];
    else if($d->v["type"]=="Decision")
    {
     $info["decisions"][$d->v["nr"]]=$d->v["text"];

     //get unused results
     for($c=1;$c<=$this->maxDecisionResults;$c++)
      if($d->v["decisionResult".$c])
       $info["decisionResults"][$d->v["nr"]."_".$c]=$d->v["text"]." -> ".$d->v["decisionResult".$c];
    }

    //get used decision results
    foreach($d->v["previousStates"] as $state)
     $info["usedStates"][]=$state;

    //get used decision results
    foreach($d->v["previousDecisionResults"] as $result)
     $info["usedDecisionResults"][]=$result;

    //get used states and decision results by current
    if($d->v["nr"]==$current)
    {
     foreach($d->v["previousStates"] as $state)
      $info["usedStatesByCurrent"][]=$state;

     foreach($d->v["previousDecisionResults"] as $result)
      $info["usedDecisionResultsByCurrent"][]=$result;
    }
   }

   return $info;
  }

  function getWorkflowNR($workflow)
  {
   //get workflow nr
   $dw=Data::cache($this->data->module, "view", "all");
   $dw->clearValues();

   $dw->setValue("name", $workflow);
   $dw->setValue("workflow", 0);

   $dw->setShowFields();

   if(!$dw->get())
   {
    $this->data->e->add($dw->e);
    return false;
   }

   if(!$dw->next())
   {
    $this->data->e->add("Unknown workflow: \"".$workflow."\"", "error");
    return false;
   }

   return $dw->v["nr"];
  }

  function getStateText($state){return $this->getStateTextCode($state, "text");}
  function getStateCode($state){return $this->getStateTextCode($state, "code");}
  function getStateTextCode($state, $field)
  {
   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("nr", $state);
   $d->setValue("type", "State");

   $d->setShowFields($field);

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   if($d->next())
    return $d->v[$field];

   $this->data->e->add("Unknown state: \"".$state."\"", "error");
   return false;
  }

  function getStates($workflow)
  {
   if(!preg_match("/^\d+$/", $workflow)) //not a number
    if(!($workflow=$this->getWorkflowNR($workflow))) //assume workflow name
     return false;

   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("nr", $workflow);
   $d->setOperant("nr", "OR");
   $d->setValue("workflow", $workflow);
   $d->setValue("type", "State");

   $d->setShowFields("code");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   $states=Array();
   while($d->next())
    $states[$d->v["nr"]]=$d->v["code"];

   return $states;
  }

  function getStateOptions($state, $includeState=false, $includeOptions=true)
  {
   if(!$includeState&&!$includeOptions)
   {
    $this->data->e->add("No state nor options to gather for state: \"".$state."\"", "error");
    return false;
   }

   $options=Array();

   //get workflow
   $d=Data::cache($this->data->module, "view", "all");
   $d->clearValues();

   $d->setValue("nr", $state, 0);

   $d->setShowFields("workflow");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   if(!$d->next())
   {
    $this->data->e->add("Unable to get workflow for state: \"".$state."\"", "error");
    return false;
   }

   if($d->v["workflow"]==0) //parent entry of workflow
    $workflow=$d->v["nr"]; //entry is workflow
   else //sub entry of workflow
    $workflow=$d->v["workflow"]; //get workflow nr from reference


   //get all workflow states and decision
   $d->setExtra("all");
   $d->clearValues();

   $d->setValue("nr", $workflow);
   $d->setOperant("nr", "OR");
   $d->setValue("workflow", $workflow);

   $showFields=Array("text", "type", "previousStates", "previousDecisionResults");
   for($c=1;$c<=$this->maxDecisionResults;$c++)
    $showFields[]="decisionResult".$c;

   $d->setShowFields($showFields);
   unset($showFields);

   $d->sortOrder("text");

   if(!$d->get())
   {
    $this->data->e->add($d->e);
    return false;
   }

   //get workflow name and current state
   while($d->next())
   {
    if($d->v["type"]=="State")
    {
     if($d->v["nr"]==$state)
     {
      $state=$d->v;
      break;
     }
    }
   }

   if($includeState)
    $options[$state["nr"]]=$state["text"];

   //get decisions after this state, add only has begin state
   if($includeOptions)
   {
    //get decisions after this state
    $d->reset();
    $decisions=Array();
    while($d->next())
    {
     if($d->v["type"]=="Decision")
     {
      if(in_array($state["text"], $d->v["previousStates"]))
       $decisions[]=$d->v;
     }
    }

    //get next state for each decision
    foreach(array_keys($decisions) as $k)
    {
     for($c=1;$c<=$this->maxDecisionResults;$c++)
     {
      if($decisions[$k]["decisionResult".$c]) //only show decision results that are set
      {
       $tmp=$decisions[$k]["text"]." -> ".$decisions[$k]["decisionResult".$c];

       $d->reset();
       while($d->next())
       {
        if($d->v["type"]=="State")
        {
         if(in_array($tmp, $d->v["previousDecisionResults"])) //decision result precondition for state
         {
          //set state nr with decision result name
          if(isset($options[$d->v["nr"]])) //current state is the same as decision result
           $options[$d->v["nr"]." "]=$tmp; //add additional space to ensure uniqueness of entry, current state will be selected
          else
           $options[$d->v["nr"]]=$tmp;
         }
        }
       }

       if(!in_array($tmp, $options)) //decision result not a precondition for any state
        $options[$state["nr"]." "]=$tmp; //set decision result to current state
      }
     }
    }
   }

   return $options;
  }
 }

?>
