<?

 /*****************************************************************************
 * Filename: include/plugins/lister/lister.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["format"]."template.php");

 class Plugin_lister extends Base_Plugin
 {
  function _getData($cachecall=false)
  {
   global $_benchmark, $_config;

   if(!$this->config)
    return false;

   if($_benchmark) $_benchmark->start("Data object");

   if($cachecall)
    $this->originalConfig=$this->config;
   else if(isset($this->originalConfig))
   {
    if($this->originalConfig["key"]==$this->config["key"])
     $this->config=$this->originalConfig;

    unset($this->originalConfig);
   }

   $title=false;
   if($this->config["listtype"]=="Dynamic")
   {
    $dc=Data::cache($this->config["module"]);

    if($_config->get("useModRewrite")&&request("p", 1)) //mod rewrite, second p argument set?
    {
     if(preg_match("/^c(\d+)$/", request("p", 1))) //c(ategory) prefix, c123, lookup category 123
     {
      if(request("p", 2)) //third p argument is id/title in case category is also set
      {
       if(preg_match("/^(\d+)$/", request("p", 2))) //id
        $this->config["listtype"]="Item";
       else
       {
        $this->config["listtype"]="All";
        $title=request("p", 2);
       }
      }
      else
       $this->config["listtype"]="List";
     }
     else if(preg_match("/^(\d+)$/", request("p", 1))) //id
      $this->config["listtype"]="Item";
     else
     {
      $this->config["listtype"]="All";
      $title=request("p", 1);
     }
    }
    else if(request($dc->getID())) //id field set?
     $this->config["listtype"]="Item";
    else
    {
     $list=false;
     foreach($dc->check("properties", "rightsReference") as $field) //list
     {
      if(request($field)) //list
      {
       $this->config["listtype"]="List";
       $list=true;
       break;
      }
     }

     if(!$list)
     {
      if(request("search")) //search
       $this->config["listtype"]="Search";
      else //all
       $this->config["listtype"]="All";
     }
     unset($list);
    }

    unset($dc);
   }

   //setup basic data object
   if($this->config["listtype"]=="Data set")
   {
    $dc=Data::cache("dataset", "view", "item");
    $dc->setValue("nr", $this->config["itemnr"]);
    $dc->setValue("showFields", Array("module", "extra"));

    if($this->config["pagebrowser"]&&request("show_from")) //only show "next page" if page browser is active
     $dc->setValue("dataset_show_from", request("show_from"));
    else
     $dc->setValue("dataset_show_from", 0);

    $dc->setValue("dataset_show_total", $this->config["listitems"]);

    if($dc->get()&&$dc->next()) //failure should at least be registered in dataset module
     $dc=$dc->moduleObject->getDataObject(0);
   }
   else
   {
    if($this->config["listtype"]=="Search")
    {
     $dc=Data::cache($this->config["module"], "search", "full");

     if($dc->validExtra("index"))
      $dc->setExtra("index"); //index search
    }
    else if($this->config["listtype"]=="Tree")
     $dc=Data::cache($this->config["module"], "tree", "down");
    else if($this->config["listtype"]=="All"&&$this->config["filter_field_1"]) //custom filtering for all mode
    {
     $dtmp=Data::cache("lister_pluginConfigs");
     $dtmp->clearValues();

     foreach($this->config as $k => $v)
      if(isset($dtmp->field[$k]))
       $dtmp->setValue($k, $v);

     $dtmp->setValue("extra", "All");
     $dc=$dtmp->moduleObject->getDataObject();
     unset($dtmp);
    }
    else //item, list, all (without filtering)
     $dc=Data::cache($this->config["module"], "view", strtolower($this->config["listtype"]));

    $id=$dc->getID();

    //clear values for all but custom filtering for all mode
    if($this->config["listtype"]!="All"||!$this->config["filter_field_1"])
     $dc->clearValues();


    if($this->config["pagebrowser"]&&request("show_from")) //only show "next page" if page browser is active
     $dc->setValue("show_from", request("show_from"));

    $dc->setValue("show_total", $this->config["listitems"]);

    //customize data object to configuration
    switch($this->config["listtype"])
    {
     case "Item":
     {
      if($this->config["static"])
       $dc->setValue($id, $this->config["itemnr"]);
      else //dynamic
      {
       if($_config->get("useModRewrite")) //mod rewrite
       {
        if(request("p", 2)) //third p argument set?
         $dc->setValue($id, request("p", 2)); //use it
        else if(request("p", 1))
         $dc->setValue($id, request("p", 1)); //use second p argument
       }
       else
        $dc->setRequestValue($id);
      }

      break;
     }
     case "List":
     {
      if($this->config["static"])
      {
       foreach($dc->check("properties", "rightsReference") as $field)
        $dc->setValue($field, $this->config["listcatnr"]);
      }
      else //dynamic
      {
       foreach($dc->check("properties", "rightsReference") as $field)
       {
        if($_config->get("useModRewrite")) //mod rewrite
        {
         $tmp=request("p", 1); //use second p argument

         if(strtolower($tmp[0])=="c") //c(ategory) prefix
          $tmp=substr($tmp, 1); //remove prefix

         $dc->setValue($field, $tmp);

         unset($tmp);

         if($dc->checkField($field))
          $this->plugin->addVar("p[1]", "c".$dc->getValue($field));
        }
        else
        {
         $dc->setRequestValue($field);

         if($dc->checkField($field))
          $this->plugin->addVar($field, $dc->getValue($field));
        }
       }
      }

      break;
     }
     case "Tree":
     {
      $field=$dc->getSelfReference();

      if($this->config["listitems"]) //abused in this case for tree depth
       $dc->setValue("treedepth", $this->config["listitems"]-1);

      if($this->config["static"])
       $dc->setValue($field, $this->config["listcatnr"]);
      else //dynamic
      {
       if($_config->get("useModRewrite")) //mod rewrite
       {
        $tmp=request("p", 1); //use second p argument

        if(strtolower($tmp[0])=="c") //c(ategory) prefix
         $tmp=substr($tmp, 1); //remove prefix

        $dc->setValue($field, $tmp);

        unset($tmp);

        if($dc->checkField($field))
         $this->plugin->addVar("p[1]", "c".$dc->getValue($field));
       }
       else
       {
        $dc->setRequestValue($field);

        if($dc->checkField($field))
         $this->plugin->addVar($field, $dc->getValue($field));
       }
      }

      break;
     }
     case "Search":
     {
      $field="search";
      $dc->setRequestValue($field);
      if($dc->checkField($field))
       $this->plugin->addVar($field, $dc->getValue($field));

      break;
     }
     case "All":
     {
      if($title) //title set via dynamic listtype
       if($field=$dc->check("properties", "title")) //title field found
        $dc->setValue($field[0], $title);

      break;
     }
     default: break;
    }

    if(isset($this->config["groupfield"])&&$this->config["groupfield"])
     $dc->setValue("aggregation_field", $this->config["groupfield"]);
   }

   //set up sorting
   if($this->config["orderbymanual"]&&($orderby=request("sort"))) //manual overwrite based on "sort" variable
    $orderby=explode(",", str_replace(" ", "", $orderby)); //multiple sort fields comma separated, remove spaces from sort fields: usually "+"
   else if($this->config["listtype"]!="Data set") //static sorting (also allowed when manual overwrite is enabled), Data set uses own configured sorting
   {
    //set sorting
    $orderby=Array();
    if($this->config["orderby1"])
    {
     if($this->config["orderbysort1"]=="Asc")
      $orderby[]=$this->config["orderby1"]."+";
     else
      $orderby[]=$this->config["orderby1"]."-";

     if($this->config["orderby2"])
     {
      if($this->config["orderbysort2"]=="Asc")
       $orderby[]=$this->config["orderby2"]."+";
      else
       $orderby[]=$this->config["orderby2"]."-";

      if($this->config["orderby3"])
      {
       if($this->config["orderbysort3"]=="Asc")
        $orderby[]=$this->config["orderby3"]."+";
       else
        $orderby[]=$this->config["orderby3"]."-";
      }
     }
    }
   }

   if(isset($orderby)&&$orderby) //no orderby supplied, use default
    $dc->sortOrder($orderby);

   //no show fields, i.e. cache depth is 0 (actual show fields set in this->output)
   $dc->setShowFields();

   if($_benchmark) $_benchmark->stop("Data object");

   return $dc;
  }


  function _flattenValues($values)
  {
   $result=Array();
   foreach($values as $k => $v)
    if(!is_array($v))
     $result[0][$k]=$v;

   foreach($values as $k => $v)
    if(is_array($v))
     foreach($this->_flattenValues($v) as $res)
      $result[]=$res;

   return $result;
  }

  function cache(&$cacheTime, &$cacheKey)
  {
   global $_benchmark;

   if($_benchmark) $_benchmark->start("Last update");

   $dc=$this->_getData(true);

   if(!$this->_cacheKey($cacheKey, $dc))
    return false;

   if(!$this->_cacheDataUpdated($cacheTime, $dc))
    return false;

   if(!$this->_cacheTemplateUpdated($cacheTime, $dc))
    return false;

   if($_benchmark) $_benchmark->stop("Last update");

   return true;
  }


  function _cacheKey(&$cacheKey, &$dc)
  {
   $cacheKey.=$this->config["nr"]; //added config nr to cache key (prefix set by pager plugin)

   switch($dc->extra)
   {
    case "item":
    {
     $id=$dc->getID();
     $cacheKey.="_item_".$id."_".$dc->getValue($id);
     break;
    }
    case "list":
    {
     foreach($dc->check("properties", "rightsReference") as $field)
      $cacheKey.="_list_".$field."_".$dc->getValue($field);

     break;
    }
    case "all":
    {
     $cacheKey.="_all";

     $tmp="";
     foreach($dc->check("valuetype", "set") as $field)
      if($field!="show_from"&&$field!="show_total")
       for($c=0;$c<$dc->vmax;$c++)
        $tmp.="_".$field."_".$dc->getValue($field, $c);

     if($tmp)
     {
      if(preg_match("/^[a-z0-9_]+$/i", $tmp)) //simple filter, use it as key
       $cacheKey.=$tmp;
      else //complex filter, md5 it
       $cacheKey.="_".md5($tmp);
     }

     unset($tmp);

     break;
    }
    case "down": //tree
    {
     $field=$dc->getSelfReference();
     $cacheKey.="_treeDown_".$field."_".$dc->getValue($field);

     break;
    }
    case "up": //tree
    {
     $field=$dc->getSelfReference();
     $cacheKey.="_treeUp_".$field."_".$dc->getValue($field);

     break;
    }
    case "full":  //search
    {
     $cacheKey.="search";

     $tmp="";
     foreach($dc->check("valuetype", "request") as $field)
      if($field!="show_from"&&$field!="show_total")
       for($c=0;$c<$dc->vmax;$c++)
        $tmp.="_".$field."_".$dc->getValue($field, $c);

     if($tmp)
      $cacheKey.="_".md5($tmp);

     unset($tmp);

     break;
    }
    default:
    {
     $this->e->add("Lister cache data action: \"".$dc->action."->".$dc->extra."\" not supported", "warning", "Cache disabled");
     $cacheKey=false;
     return false;;
    }
   }

   if(isset($this->config["pagebrowser"])&&$this->config["pagebrowser"]&&$this->config["listtype"]!="Item")
    $cacheKey.="_from_".($dc->getValue("show_from")?$dc->getValue("show_from"):0);

   $cacheKey.="_".md5(implode(",", $dc->sortOrder)); //add sorting to cache key
   $cacheKey.=".html";

   return true;
  }

  function _cacheDataUpdated(&$cacheTime, &$dc)
  {
   //get action/extra
   $action=$dc->action;
   $extra=$dc->extra;

   //set view->info mode
   $dc->setAction("database");
   $dc->setExtra("info");

   //get reference depth (based on show fields)
   if(($depth=$dc->getReferenceDepth())===false)
    $depth=0;

   if($dc->get($depth)&&$dc->next()) //try database->info
   {
    foreach($this->_flattenValues($dc->v) as $v)
    {
     $updated=DT::dateConvertS($v["info_updated"], $dc->format[$dc->dataformat]["datetimeformat"]);

     if($updated>$cacheTime)
      $cacheTime=$updated;
    }

    //get database->info from extended modules
    foreach(array_keys($dc->extends) as $module)
    {
     if($dc->extends[$module]->get($depth)&&$dc->extends[$module]->next()) //try database->info
     {
      foreach($this->_flattenValues($dc->extends[$module]->v) as $v)
      {
       $updated=DT::dateConvertS($v["info_updated"], $dc->extends[$module]->format[$dc->extends[$module]->dataformat]["datetimeformat"]);

       if($updated>$cacheTime)
        $cacheTime=$updated;
      }
     }
    }

    $dc->setAction($action);
    $dc->setExtra($extra);
   }
   else //use normal view mode based on actual filter (doesn't detect deleted items!)
   {
    $dc->setAction($action);
    $dc->setExtra($extra);

    $dc->setValue("show_from", 0);
    $dc->setValue("show_total", 1);
    $dc->setShowFields("updated");

    $dc->sortOrder("updated-"); //last updated on top

    if(!$dc->get()||!$dc->next())
    {
     $this->e->add($dc->e);
     return false;
    }

    $updated=DT::dateConvertS($dc->v["updated"], $dc->field["updated"]["format"][$dc->dataformat]["datetimeformat"]);

    if($updated>$cacheTime)
     $cacheTime=$updated;
   }

   return true;
  }

  function _cacheTemplateUpdated(&$cacheTime, &$dc)
  {
   global $_path;

   //templates updated
   if($this->config["listtype"]=="Item")
   {
    if($this->config["templateitem"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templateitem"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templateitem, "modified");

    if($cacheTime<$time)
     $cacheTime=$time;
   }
   else //list, all, search, etc
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatelist, "modified");

    if($cacheTime<$time)
     $cacheTime=$time;

    if(isset($this->config["templatepagebrowser"])&&$this->config["templatepagebrowser"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatepagebrowser"], "modified");
    else if(isset($this->plugin->templatepagebrowser))
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatepagebrowser, "modified");

    if($cacheTime<$time)
     $cacheTime=$time;
   }

   return true;
  }


  function output()
  {
   global $_path, $_benchmark, $_config;

   if($_benchmark) $_benchmark->start("Get content");

   if(!($dc=$this->_getData()))
    return false;

   $id=$dc->getID();

   if($this->config["cmspage"]||$this->config["itempage"])
   {
    $tmp=Data::cache("pager_categories", "view", "item");

    if($this->config["itempage"])
    {
     $tmp->setValue("nr", $this->config["itempagenr"]);
     $tmp->setShowFields("urlname");

     if($tmp->get()&&$tmp->next())
      $itempage=$tmp->v["urlname"];
     else
      $itempage=$this->config["itempagenr"];
    }

    if($this->config["cmspage"])
    {
     $tmp->clearValues();
     $tmp->setValue("nr", $this->config["cmspagenr"]);
     $tmp->setShowFields("urlname");

     if($tmp->get()&&$tmp->next())
      $cmspage=$tmp->v["urlname"];
     else
      $cmspage=$this->config["cmspagenr"];

     $dc->addField("cms_url", "text", "", false, 0); //only use CMS page if configured
    }

    unset($tmp);
   }

   $dc->addField("sort_order", "text", "show", "control", false, false, implode(",", $dc->sortOrder));

   $dc->addField("item_url", "text", "", "control", 0);
   if($_config->get("useModRewrite")) //mod rewrite
   {
    $dc->addField("base_url", "text", "", "control", 0, false, $_path["HTML"]["current"]);
    $dc->addField("root_url", "text", "", "control", 0, false, $_path["root"]);


    $tmp=$_path["HTML"]["current"];
    foreach($this->plugin->vars as $k => $v)
     if($k=="p"||substr($k, 0, 2)=="p[")
      $tmp.=$v."/";

    $dc->addField("page_url", "text", "", "control", 0, false, $tmp);
    unset($tmp);
   }
   else
   {
    $dc->addField("base_url", "text", "", "control", 0, false, "");
    $dc->addField("root_url", "text", "", "control", 0, false, $_path["root"]);
    $dc->addField("page_url", "text", "", "control", 0, false, htmlentities($this->plugin->url(), ENT_COMPAT|ENT_XHTML));
   }

   if($_benchmark) $_benchmark->start("Template");

   //single result use item template (except tree)
   if($this->config["singletemplate"]&&$this->config["listtype"]!="Item"&&$this->config["listtype"]!="Tree") //skip single result check for item
   {
    $tmp=clone $dc;
    if($tmp->get(0, true))
     if($tmp->getValue("show_count")==1)
      $this->config["listtype"]="Item";

    unset($tmp);
   }

   //load template
   if($this->config["listtype"]=="Item")
   {
    if($this->config["templateitem"])
     $templateFile=$_path["project"]["templates"].$this->config["templateitem"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templateitem;
   }
   else //default templates
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelist;
   }

   $template=new Format_Template(true);

   if(!$template->read($templateFile)||!$template->analyze($dc))
   {
    $this->e->add($template->e);
    return false;
   }


   if($template->markerExists("_fields")&&$this->config["showfields"]) //_fields marker in template and predefined fields set
    $dc->setShowFields(array_merge($this->config["showfields"], $dc->check("use", "show"))); //added predefined fields to show fields
   //else //use template defined fields

   if($_benchmark) $_benchmark->stop("Template");


   if($_benchmark) $_benchmark->start("Get data");

   //get data
   if($this->config["pagebrowser"]||$template->showCount())
    $success=$dc->get($template->getLevel(), true);
   else
    $success=$dc->get($template->getLevel());

   if(!$success)
   {
    $this->e->add($dc->e);
    return false;
   }

   //setValues
   for($c=0;$c<$dc->vmax;$c++) //loop trough all entries to calculate its relative locnr
   {
    $dc->setDefaultValue("sort_order", $c);
    $dc->setDefaultValue("page_url", $c);
    $dc->setDefaultValue("base_url", $c);
    $dc->setDefaultValue("root_url", $c);

    if($_config->get("useModRewrite")) //mod rewrite
    {
     if($this->config["cmspage"])
      $dc->setValue("cms_url", $dc->getValue("base_url", $c).$cmspage."/".htmlentities("?m=".$dc->module."&".$id."=".$dc->getValue($id, $c)."&", ENT_COMPAT|ENT_XHTML), $c);

     if($this->config["itempage"])
      $dc->setValue("item_url", $dc->getValue("base_url", $c).$itempage."/".$dc->getValue($id, $c)."/", $c);
     else
      $dc->setValue("item_url", $dc->getValue("page_url", $c).$dc->getValue($id, $c)."/", $c);
    }
    else
    {
     if($this->config["cmspage"])
      $dc->setValue("cms_url", htmlentities("?p=".$cmspage."&m=".$dc->module."&".$id."=".$dc->getValue($id, $c)."&", ENT_COMPAT|ENT_XHTML), $c);

     if($this->config["itempage"])
      $dc->setValue("item_url", htmlentities("?p=".$itempage."&".$id."=".$dc->getValue($id, $c)."&", ENT_COMPAT|ENT_XHTML), $c);
     else
      $dc->setValue("item_url", $dc->getValue("page_url", $c).htmlentities($id."=".$dc->getValue($id, $c)."&", ENT_COMPAT|ENT_XHTML), $c);
    }
   }

   if($_benchmark) $_benchmark->stop("Get data");


   if($_benchmark) $_benchmark->start("Populate template");

   if($dc->e->any("fatal"))
   {
    $this->e->add($dc->e);
    return false;
   }

   if(!$template->populate())
   {
    $this->e->add($template->e);
    return false;
   }

   $output=$template->output();

   if($_benchmark) $_benchmark->stop("Populate template");

   if($this->config["pagebrowser"]&&$dc->extra!="item")
   {
    if($_benchmark) $_benchmark->start("Page browser");

    if($this->config["templatepagebrowser"])
     $templateFile=$_path["project"]["templates"].$this->config["templatepagebrowser"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatepagebrowser;

    $templatepagebrowser=new Format_Template(true);

    if(!$templatepagebrowser->read($templateFile))
    {
     $this->e->add($templatepagebrowser->e);
     return false;
    }

    if($templatepagebrowser->pageBrowser($dc, $dc->getValue("page_url").($_config->get("useModRewrite")?"?":"").(request("sort")?"sort=".$dc->getDefaultValue("sort_order")."&amp;":""))) //add ? for args when using mod rewrite
     $output.=$templatepagebrowser->output();

    if($_benchmark) $_benchmark->stop("Page browser");
   }

   if($_benchmark) $_benchmark->stop("Get content");

   return $output;
  }

 }

?>
