<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/archive.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."archive.php");

 class UT_Archive extends UT_Classes
 {
  function _init()
  {
   global $_path;

   $this->archives=Array(
    "GZ" => $_path["plugins"]."unit_testing/misc/test.gz",
    "BZ2" => $_path["plugins"]."unit_testing/misc/test.bz2",
    "TAR" => $_path["plugins"]."unit_testing/misc/test.tar",
    "TGZ" => $_path["plugins"]."unit_testing/misc/test.tgz",
    "TAR.GZ" => $_path["plugins"]."unit_testing/misc/test.tar.gz",
    "TAR.BZ2" => $_path["plugins"]."unit_testing/misc/test.tar.bz2",
    "RAR" => $_path["plugins"]."unit_testing/misc/test.rar",
    "ZIP" => $_path["plugins"]."unit_testing/misc/test.zip",
   );

   $this->supported=Array(
    "GZ" => (function_exists("gzopen")?true:false),
    "BZ2" => (function_exists("bzopen")?true:false),
    "TAR" => (class_exists("Archive_Tar")?true:false),
    "TGZ" => (class_exists("Archive_Tar")&&function_exists("gzopen")?true:false),
    "TAR.GZ" => (class_exists("Archive_Tar")&&function_exists("gzopen")?true:false),
    "TAR.BZ2" => (class_exists("Archive_Tar")&&function_exists("bzopen")?true:false),
    "RAR" => (class_exists("RarArchive")?true:false),
    "ZIP" => (class_exists("ZipArchive")?true:false),
   );

   $this->contenttypes=Array(
    "GZ" => "application/x-gzip",
    "BZ2" => "application/bz2",
    "TAR" => "application/x-tar",
    "TGZ" => "application/x-gzip",
    "TAR.GZ" => "application/x-gzip",
    "TAR.BZ2" => "application/bz2",
    "ZIP" => "application/zip",
   );

   $this->files=Array(
    "test.txt" => Array(
     "type" => "file",
     "time" => 1438636588,
     "size" => 8,
     "contents" => "testing\n",
    ),
    "test/" => Array(
     "type" => "directory",
     "time" => 1438636607,
    ),
    "test/test.txt" => Array(
     "type" => "file",
     "time" => 1438636607,
     "size" => 13,
     "contents" => "more testing\n",
    ),
   );
  }

  function Constructor()
  {
   global $_path;

   $this->_init();
   $object=new Archive();

   if(!is_object($object)||get_class($object)!="Archive")
   {
    $this->e->add("Constructor doesn't return a Archive object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->methods!==Array("GZ", "BZ2", "TAR", "TGZ", "TAR.GZ", "TAR.BZ2", "RAR", "ZIP"))
   {
    $this->e->add("Object methods doesn't match", "error");
    return UT_FAILED;
   }

   if($object->string!=="")
   {
    $this->e->add("Object string isn't an empty string", "error");
    return UT_FAILED;
   }

   if($object->files!==Array())
   {
    $this->e->add("Object files isn't an empty array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Archive();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Archive object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->methods=Array();
   if($orig->methods==$copy->methods)
   {
    $this->e->add("Archive object copy also modified original \"methods\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Archive object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $ar=new Archive();
   $ar->string="asd";
   $ar->files=Array("asd");

   $ar->clear();

   if($ar->string)
   {
    $this->e->add("Archive clear didn't clear string", "error");
    return UT_FAILED;
   }

   if($ar->files)
   {
    $this->e->add("Archive clear didn't clear files", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * String
   */

  function GetString()
  {
   $ar=new Archive();

   $ar->string="test";
   if($ar->getString()!="test")
   {
    $this->e->add("Archive get string output doesn't match", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetString()
  {
   $ar=new Archive();

   if(!$ar->setString("test"))
   {
    $this->e->add("Archive set shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if($ar->string!="test")
   {
    $this->e->add("Archive set string didn't set string correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * Get files info
   */

  function GetFiles()
  {
   $this->_init();
   $ar=new Archive();

   $ar->files=$this->files;
   if(!$this->_compare(array_keys($this->files), $ar->getFiles()))
   {
    $this->e->add("Archive get files compare content failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetType()
  {
   $this->_init();
   $ar=new Archive();

   $ar->files=$this->files;
   foreach($this->files as $file => $info)
   {
    if($ar->getType($file)!=$info["type"])
    {
     $this->e->add("Archive get type: ".$info["type"]." doesn't match: ".$ar->getType($file), "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->getType("asd"))
   {
    $this->e->add("Archive get type should have failed for unknown file", "error", $file);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetTime()
  {
   $this->_init();
   $ar=new Archive();

   $ar->files=$this->files;
   foreach($this->files as $file => $info)
   {
    if($ar->getTime($file)!=$info["time"])
    {
     $this->e->add("Archive get time: ".$info["time"]." doesn't match: ".$ar->getTime($file), "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->getTime("asd"))
   {
    $this->e->add("Archive get time should have failed for unknown file", "error", $file);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetSize()
  {
   $this->_init();
   $ar=new Archive();

   $ar->files=$this->files;
   foreach($this->files as $file => $info)
   {
    if($info["type"]=="directory")
     continue;

    if($ar->getSize($file)!=$info["size"])
    {
     $this->e->add("Archive get size: ".$info["size"]." doesn't match: ".$ar->getSize($file), "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->getSize("asd"))
   {
    $this->e->add("Archive get size should have failed for unknown file", "error", $file);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetContents()
  {
   $this->_init();
   $ar=new Archive();

   $ar->files=$this->files;
   foreach($this->files as $file => $info)
   {
    if($info["type"]=="directory")
     continue;

    if($ar->getContents($file)!=$info["contents"])
    {
     $this->e->add("Archive get contents: ".$info["contents"]." doesn't match: ".$ar->getContents($file), "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->getContents("asd"))
   {
    $this->e->add("Archive get contents should have failed for unknown file", "error", $file);
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * Set file info
   */

  function SetFile()
  {
   $files=Array();
   $ar=new Archive();

   //force time
   $files["test.txt"]=Array(
    "type" => "file",
    "time" => time(),
    "size" => 7,
    "contents" => "testing",
   );
   if(!$ar->_setFile("test.txt", $files["test.txt"]["contents"], $files["test.txt"]["time"]))
   {
    $this->e->add("Archive set file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive set file compare content failed", "error");
    return UT_FAILED;
   }

   //dynamic time
   $files["test/test2.txt"]=Array(
    "type" => "file",
    "time" => time(),
    "size" => 12,
    "contents" => "more testing",
   );
   if(!$ar->_setFile("test/test2.txt", $files["test/test2.txt"]["contents"]))
   {
    $this->e->add("Archive set file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive set file compare content failed", "error");
    return UT_FAILED;
   }

   //overwrite with custom time
   $files["test.txt"]=Array(
    "type" => "file",
    "time" => 123,
    "size" => 0,
    "contents" => "",
   );
   if(!$ar->_setFile("test.txt", $files["test.txt"]["contents"], $files["test.txt"]["time"]))
   {
    $this->e->add("Archive set file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive set file compare content failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetDirectory()
  {
   $dirs=Array();
   $ar=new Archive();

   //force time
   $dirs["test/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   if(!$ar->_setDirectory("test/", $dirs["test/"]["time"]))
   {
    $this->e->add("Archive set directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set directory compare content failed", "error");
    return UT_FAILED;
   }

   //dynamic time
   $dirs["test2/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   if(!$ar->_setDirectory("test2/"))
   {
    $this->e->add("Archive set directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set directory compare content failed", "error");
    return UT_FAILED;
   }

   //force custom time
   $dirs["test3/"]=Array(
    "type" => "directory",
    "time" => 123,
   );
   if(!$ar->_setDirectory("test3", $dirs["test3/"]["time"]))
   {
    $this->e->add("Archive set directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set directory compare content failed", "error");
    return UT_FAILED;
   }

   //overwrite with custom time
   $dirs["test/"]=Array(
    "type" => "directory",
    "time" => 123,
   );
   if(!$ar->_setDirectory("test/", $dirs["test/"]["time"]))
   {
    $this->e->add("Archive set directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set directory compare content failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetFilePath()
  {
   $dirs=Array();
   $ar=new Archive();

   //no path
   if(!$ar->_setFilePath("test.txt"))
   {
    $this->e->add("Archive set file path shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set file path compare content failed", "error");
    return UT_FAILED;
   }

   //1 level
   $dirs["test/"]=Array(
    "time" => time(),
    "type" => "directory",
   );
   if(!$ar->_setFilePath("test/test.txt"))
   {
    $this->e->add("Archive set file path shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set file path compare content failed", "error");
    return UT_FAILED;
   }

   //2 levels
   $dirs["test/test/"]=Array(
    "time" => time(),
    "type" => "directory",
   );
   if(!$ar->_setFilePath("test/test/test.txt"))
   {
    $this->e->add("Archive set file path shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive set file path compare content failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddFile()
  {
   global $_path;

   $cwd=getcwd();
   chdir($_path["root"]);

   $files=Array();
   $ar=new Archive();

   //no path
   $file="index.php";
   $files[$file]=Array(
    "type" => "file",
    "time" => filemtime($file),
    "size" => filesize($file),
    "contents" => file_get_contents($file),
   );
   if(!$ar->addFile($file))
   {
    $this->e->add("Archive add file shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive add file compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //1 level
   $file="images/index.php";
   $files["images/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $files[$file]=Array(
    "type" => "file",
    "time" => filemtime($file),
    "size" => filesize($file),
    "contents" => file_get_contents($file),
   );
   if(!$ar->addFile($file))
   {
    $this->e->add("Archive add file shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive add file compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //strip path
   $ar->clear();
   $files=Array();

   $file="images/index.php";
   $files["index.php"]=Array(
    "type" => "file",
    "time" => filemtime($file),
    "size" => filesize($file),
    "contents" => file_get_contents($file),
   );
   if(!$ar->addFile($file, "images/"))
   {
    $this->e->add("Archive add file shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($files, $ar->files))
   {
    $this->e->add("Archive add file compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   chdir($cwd);
   return UT_OK;
  }

  function AddDirectory()
  {
   global $_path;

   $cwd=getcwd();
   chdir($_path["root"]);

   $dirs=Array();
   $ar=new Archive();

   //1 level
   $dir="files";
   $dirs["files/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   foreach(FS::filesS($dir) as $file)
    $dirs[$file]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //multiple levels strip path
   $dir="include/plugins/unit_testing/modules/";
   $dirs["unit_testing/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $dirs["unit_testing/modules/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   foreach(FS::filesS($dir) as $file)
    $dirs[str_replace("include/plugins/", "", $file)]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir, "include/plugins/"))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //multiple levels strip full path
   $ar->clear();
   $dirs=Array();

   $dir="include/plugins/unit_testing/modules/";
   foreach(FS::filesS($dir) as $file)
    $dirs[str_replace("include/plugins/unit_testing/modules/", "", $file)]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir, true))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //multiple levels strip path
   $ar->clear();
   $dirs=Array();

   $dir="include/plugins/unit_testing/modules/";
   $dirs["unit_testing/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $dirs["unit_testing/modules/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   foreach(FS::filesS($dir) as $file)
    $dirs[str_replace("include/plugins/", "", $file)]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir, "include/plugins/"))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   chdir($cwd);
   return UT_OK;
  }

  function RerootFiles()
  {
   global $_path;

   $cwd=getcwd();
   chdir($_path["root"]);

   $dirs=Array();
   $ar=new Archive();

   //remove portion
   $dir="include/plugins/unit_testing/modules/";
   $dirs["unit_testing/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $dirs["unit_testing/modules/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   foreach(FS::filesS($dir) as $file)
    $dirs[str_replace("include/plugins/", "", $file)]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$ar->rerootFiles("include/plugins/"))
   {
    $this->e->add("Archive reroot files shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   //replace portion
   $ar->clear();
   $dirs=Array();

   $dir="include/plugins/unit_testing/modules/";
   $dirs["plgs/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $dirs["plgs/unit_testing/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   $dirs["plgs/unit_testing/modules/"]=Array(
    "type" => "directory",
    "time" => time(),
   );
   foreach(FS::filesS($dir) as $file)
    $dirs[str_replace("include/plugins/", "plgs/", $file)]=Array(
     "type" => "file",
     "time" => filemtime($file),
     "size" => filesize($file),
     "contents" => file_get_contents($file),
    );
   if(!$ar->addDirectory($dir))
   {
    $this->e->add("Archive add directory shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$ar->rerootFiles("include/plugins/", "plgs"))
   {
    $this->e->add("Archive reroot files shouldn't have failed", "error");
    $this->e->add($ar->e);
    chdir($cwd);
    return UT_FAILED;
   }

   if(!$this->_compare($dirs, $ar->files))
   {
    $this->e->add("Archive add directory compare content failed", "error");
    chdir($cwd);
    return UT_FAILED;
   }

   chdir($cwd);
   return UT_OK;
  }


  /*
   * Save files
   */

  function SaveFile()
  {
   global $_path;

   $ar=new Archive();
   $ar->files=$this->files;

   //file
   if(FS::existsS("./test.txt"))
   {
    $this->e->add("Archive save file blocked, file already exist", "error");
    return UT_SKIPPED;
   }

   if(!$ar->saveFile("test.txt"))
   {
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS("./test.txt"))
   {
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS("./test.txt", "modified")!=$ar->files["test.txt"]["time"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS("./test.txt")!=$ar->files["test.txt"]["size"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS("./test.txt")!=$ar->files["test.txt"]["contents"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }


   //try to overwrite
   if($ar->saveFile("test.txt"))
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file should have failed", "error");
    return UT_FAILED;
   }


   //ovewrite
   FS::writeS("./test.txt", "junk", "overwrite");

   if(!$ar->saveFile("test.txt", false, true))
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS("./test.txt"))
   {
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS("./test.txt", "modified")!=$ar->files["test.txt"]["time"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS("./test.txt")!=$ar->files["test.txt"]["size"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS("./test.txt")!=$ar->files["test.txt"]["contents"])
   {
    FS::deleteS("./test.txt");
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   FS::deleteS("./test.txt");

   //1 level
   if(FS::existsS($_path["plugins"]."unit_testing/misc/test/"))
   {
    $this->e->add("Archive save file blocked, directory already exist", "error");
    return UT_SKIPPED;
   }

   if(!FS::mkdirS($_path["plugins"]."unit_testing/misc/test/"))
   {
    $this->e->add("Archive save file blocked, can't create directory", "error");
    return UT_FAILED;
   }

   if(!$ar->saveFile("test/test.txt", $_path["plugins"]."unit_testing/misc/"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test/test.txt"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS($_path["plugins"]."unit_testing/misc/test/test.txt", "modified")!=$ar->files["test/test.txt"]["time"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["size"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["contents"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);

   //directory
   if(FS::existsS($_path["plugins"]."unit_testing/misc/test/"))
   {
    $this->e->add("Archive save file blocked, directory already exist", "error");
    return UT_SKIPPED;
   }

   if(!$ar->saveFile("test/", $_path["plugins"]."unit_testing/misc/"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test/"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't create directory", "error");
    return UT_FAILED;
   }

   FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);

   return UT_OK;
  }

  function SaveFiles()
  {
   global $_path;

   $ar=new Archive();
   $ar->files=$this->files;

   //files
   if(FS::existsS($_path["plugins"]."unit_testing/misc/test.txt"))
   {
    $this->e->add("Archive save file blocked, file already exist", "error");
    return UT_SKIPPED;
   }

   if(FS::existsS($_path["plugins"]."unit_testing/misc/test/"))
   {
    $this->e->add("Archive save file blocked, directory already exist", "error");
    return UT_SKIPPED;
   }

   if(!$ar->saveFiles($_path["plugins"]."unit_testing/misc/"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test.txt"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS($_path["plugins"]."unit_testing/misc/test.txt", "modified")!=$ar->files["test.txt"]["time"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($_path["plugins"]."unit_testing/misc/test.txt")!=$ar->files["test.txt"]["size"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS($_path["plugins"]."unit_testing/misc/test.txt")!=$ar->files["test.txt"]["contents"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test/test.txt"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS($_path["plugins"]."unit_testing/misc/test/test.txt", "modified")!=$ar->files["test/test.txt"]["time"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["size"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["contents"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   //try to overwrite
   if($ar->saveFiles($_path["plugins"]."unit_testing/misc/"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file should have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   //ovewrite
   FS::writeS($_path["plugins"]."unit_testing/misc/test/test.txt", "junk", "overwrite");
   FS::writeS($_path["plugins"]."unit_testing/misc/test.txt", "junk", "overwrite");


   if(!$ar->saveFiles($_path["plugins"]."unit_testing/misc/", true))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file shouldn't have failed", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test.txt"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS($_path["plugins"]."unit_testing/misc/test.txt", "modified")!=$ar->files["test.txt"]["time"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($_path["plugins"]."unit_testing/misc/test.txt")!=$ar->files["test.txt"]["size"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS($_path["plugins"]."unit_testing/misc/test.txt")!=$ar->files["test.txt"]["contents"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   if(!FS::existsS($_path["plugins"]."unit_testing/misc/test/test.txt"))
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file didn't save file", "error");
    return UT_FAILED;
   }

   if(FS::getTimeS($_path["plugins"]."unit_testing/misc/test/test.txt", "modified")!=$ar->files["test/test.txt"]["time"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file modified time doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::sizeS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["size"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file size doesn't match", "error");
    return UT_FAILED;
   }

   if(FS::readS($_path["plugins"]."unit_testing/misc/test/test.txt")!=$ar->files["test/test.txt"]["contents"])
   {
    FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
    FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);
    $this->e->add("Archive save file contents doesn't match", "error");
    return UT_FAILED;
   }

   FS::deleteS($_path["plugins"]."unit_testing/misc/test.txt");
   FS::deleteS($_path["plugins"]."unit_testing/misc/test/", true);

   return UT_OK;
  }


  /*
   * Method
   */

  function DetectMethod()
  {
   $this->_init();
   $ar=new Archive();

   foreach($this->archives as $method => $file)
   {
    if($ar->_detectMethod($file)!==$method)
    {
     $this->e->add("Archive detect method failed: ".$ar->_detectMethod($file)." doesn't match: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->_detectMethod("index.php")!==false)
   {
    $this->e->add("Archive detect method should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckSupport()
  {
   $this->_init();
   $ar=new Archive();

   foreach($this->supported as $method => $supported)
   {
    if($ar->_checkSupport($method)!==$supported)
    {
     $this->e->add("Archive check ".$method." support failed: ".($ar->_checkSupport($method)?"true":"false")." doesn't match: ".($supported?"true":"false"), "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }
   }

   if($ar->_checkSupport("asd")!==false)
   {
    $this->e->add("Archive check support should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   * Decopress
   */

  function Open()
  {
   $this->_init();
   $ar=new Archive();

   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();

    if(!$ar->_checkSupport($method))
     continue;

    if(!$ar->open($file))
    {
     $this->e->add("Archive open shouldn't have failed for method: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if($method=="GZ"||$method=="BZ2")
    {
     if($this->files["test.txt"]["contents"]!=$ar->string)
     {
      $this->e->add("Archive open compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else if($method=="ZIP"||$method=="RAR") //ZIP/RAR lib seems to be off one second
    {
     $tmp=$this->files;

     if($method=="ZIP")
     {
      $tmp["test/"]["time"]++;
      $tmp["test/test.txt"]["time"]++;
     }
     else
     {
      $tmp["test/"]["time"]--;
      $tmp["test/test.txt"]["time"]--;
     }

     if(!$this->_compare($tmp, $ar->files))
     {
      $this->e->add("Archive open compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else
    {
     if(!$this->_compare($this->files, $ar->files))
     {
      $this->e->add("Archive open compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
   }

   //wrong method
   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();

    if($ar->open($file, ($method=="ZIP"?"BZ2":"ZIP")))
    {
     $this->e->add("Archive open should have failed for incorrect method: ".$method, "error", $file);
     return UT_FAILED;
    }
   }

   //wrong file
   if($ar->open("index.php")!==false)
   {
    $this->e->add("Archive open should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function OpenString()
  {
   $this->_init();
   $ar=new Archive();

   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();

    if(!$ar->_checkSupport($method))
     continue;

    if(!$ar->openString(file_get_contents($file), $method))
    {
     $this->e->add("Archive open string shouldn't have failed for method: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if($method=="GZ"||$method=="BZ2")
    {
     if($this->files["test.txt"]["contents"]!=$ar->string)
     {
      $this->e->add("Archive open string compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else if($method=="ZIP"||$method=="RAR") //ZIP/RAR lib seems to be off one second
    {
     $tmp=$this->files;

     if($method=="ZIP")
     {
      $tmp["test/"]["time"]++;
      $tmp["test/test.txt"]["time"]++;
     }
     else
     {
      $tmp["test/"]["time"]--;
      $tmp["test/test.txt"]["time"]--;
     }

     if(!$this->_compare($tmp, $ar->files))
     {
      $this->e->add("Archive open string compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else
    {
     if(!$this->_compare($this->files, $ar->files))
     {
      $this->e->add("Archive open string compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
   }


   //wrong method
   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();

    if($ar->openString(file_get_contents($file), ($method=="ZIP"?"BZ2":"ZIP")))
    {
     $this->e->add("Archive open string should have failed for incorrect method: ".$method, "error", $file);
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  /*
   * Compress
   */

  function Output()
  {
   $this->_init();
   $ar=new Archive();

   //supported output
   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();
    $ar->files=$this->files;
    $ar->string=$this->files["test.txt"]["contents"];

    if(!$ar->_checkSupport($method)||$method=="RAR") //RAR has no write support
     continue;

    if(!($output=$ar->output($method)))
    {
     $this->e->add("Archive output shouldn't have failed for method: ".$method, "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if(!$ar->openString($output, $method))
    {
     $this->e->add("Archive output open string shouldn't have failed for method: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if($method=="GZ"||$method=="BZ2")
    {
     if($this->files["test.txt"]["contents"]!=$ar->string)
     {
      $this->e->add("Archive output compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else if($method=="ZIP") //ZIP output doesn't support time
    {
     $tmp=$this->files;

     $tmp["test.txt"]["time"]=$ar->files["test.txt"]["time"];
     $tmp["test/"]["time"]=$ar->files["test/"]["time"];
     $tmp["test/test.txt"]["time"]=$ar->files["test/test.txt"]["time"];

     if(!$this->_compare($tmp, $ar->files))
     {
      $this->e->add("Archive output compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else
    {
     if(!$this->_compare($this->files, $ar->files))
     {
      $this->e->add("Archive output compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
   }


   //output headers
   $this->_storeHeaders();
   $ar->files=$this->files;
   $ar->string=$this->files["test.txt"]["contents"];
   foreach($this->contenttypes as $method => $header)
   {
    $ar->e->clear();

    if(!$ar->_checkSupport($method))
     continue;

    if(!($output=$ar->output($method, true)))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output shouldn't have failed for method: ".$method, "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    $headers=headers_list();
    if(!in_array("Content-Type: ".$header, $headers))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output Content-Type header not found or incorrect for method: ".$method, "error");
     return UT_FAILED;
    }

    if(!in_array("Content-Length: ".strlen($output), $headers))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output Content-Length header not found or incorrect for method: ".$method, "error");
     return UT_FAILED;
    }
   }

   //output file headers
   foreach($this->contenttypes as $method => $header)
   {
    $ar->e->clear();

    if(!$ar->_checkSupport($method))
     continue;

    if(!($output=$ar->output($method, true, "test.".strtolower($method))))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output shouldn't have failed for method: ".$method, "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    $headers=headers_list();
    if(!in_array("Content-Type: ".$header, $headers))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output Content-Type header not found or incorrect for method: ".$method, "error");
     return UT_FAILED;
    }

    if(!in_array("Content-Length: ".strlen($output), $headers))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output Content-Length header not found or incorrect for method: ".$method, "error");
     return UT_FAILED;
    }

    if(!in_array("Content-Disposition: attachment; filename=\"test.".strtolower($method)."\"", $headers))
    {
     $this->_restoreHeaders();
     $this->e->add("Archive output Content-Disposition header not found or incorrect for method: ".$method, "error");
     return UT_FAILED;
    }
   }

   $this->_restoreHeaders();

   //not supported output
   $ar->e->clear();
   $ar->files=$this->files;
   if($ar->output("RAR"))
   {
    $this->e->add("Archive output should have failed for method: RAR", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Save()
  {
   global $_path;

   $this->_init();
   $ar=new Archive();

   //supported save
   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();
    $ar->files=$this->files;
    $ar->string=$this->files["test.txt"]["contents"];

    if(!$ar->_checkSupport($method)||$method=="RAR") //RAR has no write support
     continue;

    $file=$_path["plugins"]."unit_testing/misc/TESTING.".strtolower($method);
    if(!($save=$ar->save($file, $method)))
    {
     FS::deleteS($file);
     $this->e->add("Archive save shouldn't have failed for method: ".$method, "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if(!$ar->open($file, $method))
    {
     FS::deleteS($file);
     $this->e->add("Archive save open shouldn't have failed for method: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if($method=="GZ"||$method=="BZ2")
    {
     if($this->files["test.txt"]["contents"]!=$ar->string)
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else if($method=="ZIP") //ZIP save doesn't support time
    {
     $tmp=$this->files;

     $tmp["test.txt"]["time"]=$ar->files["test.txt"]["time"];
     $tmp["test/"]["time"]=$ar->files["test/"]["time"];
     $tmp["test/test.txt"]["time"]=$ar->files["test/test.txt"]["time"];

     if(!$this->_compare($tmp, $ar->files))
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else
    {
     if(!$this->_compare($this->files, $ar->files))
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }

    FS::deleteS($file);
   }

   //supported save without method
   foreach($this->archives as $method => $file)
   {
    $ar->e->clear();
    $ar->files=$this->files;
    $ar->string=$this->files["test.txt"]["contents"];

    if(!$ar->_checkSupport($method)||$method=="RAR") //RAR has no write support
     continue;

    $file=$_path["plugins"]."unit_testing/misc/TESTING.".strtolower($method);
    if(!($save=$ar->save($file)))
    {
     FS::deleteS($file);
     $this->e->add("Archive save shouldn't have failed for method: ".$method, "error");
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if(!$ar->open($file))
    {
     FS::deleteS($file);
     $this->e->add("Archive save open shouldn't have failed for method: ".$method, "error", $file);
     $this->e->add($ar->e);
     return UT_FAILED;
    }

    if($method=="GZ"||$method=="BZ2")
    {
     if($this->files["test.txt"]["contents"]!=$ar->string)
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else if($method=="ZIP") //ZIP save doesn't support time
    {
     $tmp=$this->files;

     $tmp["test.txt"]["time"]=$ar->files["test.txt"]["time"];
     $tmp["test/"]["time"]=$ar->files["test/"]["time"];
     $tmp["test/test.txt"]["time"]=$ar->files["test/test.txt"]["time"];

     if(!$this->_compare($tmp, $ar->files))
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }
    else
    {
     if(!$this->_compare($this->files, $ar->files))
     {
      FS::deleteS($file);
      $this->e->add("Archive save compare content failed for method: ".$method, "error", $file);
      return UT_FAILED;
     }
    }

    FS::deleteS($file);
   }

   //not supported save
   $ar->e->clear();
   $ar->files=$this->files;
   $file=$_path["plugins"]."unit_testing/misc/TESTING.RAR";
   if($ar->save($file, "RAR"))
   {
    FS::deleteS($file);
    $this->e->add("Archive save should have failed for method: RAR", "error");
    $this->e->add($ar->e);
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
