<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/core.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut.php");
 require_once($_path["include"]."diff.php");

 class UT_Core extends UT
 {
  function _storeHeaders()
  {
   //store original headers
   $this->headers=headers_list();
  }

  function _restoreHeaders()
  {
   //clear current headers (if remove function is available)
   if(function_exists("header_remove"))
    header_remove();

   //restore original headers
   foreach($this->headers as $header)
    header($header, true);

   header("Content-type: text/html"); //ensure content-type header isn't overwritten
  }

  function _compare($expected, $values, $keys=Array(), $object_ids=Array())
  {
   if(gettype($expected)!=gettype($values))
   {
    if($keys)
     $this->e->add("Key: ".implode(" / ", $keys)." expected type: ".gettype($expected)." doesn't match: ".gettype($values));
    else
     $this->e->add("Expected type: ".gettype($expected)." doesn't match: ".gettype($values));

    return UT_FAILED;
   }

   if(is_array($expected)) //run throught array
   {
    $ret=UT_OK;
    foreach(array_keys($expected) as $key) //check if all expected keys exist
    {
     if(!isset($values[$key])&&$expected[$key]!==NULL)
     {
      $this->e->add("Expected key: ".implode(" / ", $keys)." / ".$key." not found");
      $ret=UT_FAILED;
     }
     else
     {
      //check if key values match
      if($this->_compare($expected[$key], $values[$key], array_merge($keys, Array($key)))===UT_FAILED) //compare values / sub arrays/objects
       $ret=UT_FAILED;
     }
    }

    foreach(array_keys($values) as $key) //check if non expected keys exist
    {
     if(!isset($expected[$key])&&$values[$key]!==NULL)
     {
      $this->e->add("Key: ".implode(" / ", $keys)." / ".$key." not expected");
      $ret=UT_FAILED;
     }
    }

    return $ret;
   }
   else if(is_object($expected)) //run throught object
   {
    $ret=UT_OK;

    if(in_array(spl_object_hash($expected), $object_ids)) //object already validated (can happen when objects have back reference to parent object)
     return $ret; //done checking

    $object_ids[]=spl_object_hash($expected); //store object id

    foreach(array_keys(get_object_vars($expected)) as $key) //check if all expected keys exist
    {
     if(!isset($values->$key)&&$expected->$key!==NULL)
     {
      $this->e->add("Expected key: ".implode(" / ", $keys)." / ".$key." not found");
      $ret=UT_FAILED;
     }
     else
     {
      //check if key values match
      if($this->_compare($expected->$key, $values->$key, array_merge($keys, Array($key)), $object_ids)==UT_FAILED) //compare values / sub arrays/objects
       $ret=UT_FAILED;
     }
    }

    foreach(array_keys(get_object_vars($values)) as $key) //check if non expected keys exist
    {
     if(!isset($expected->$key)&&$values->$key!==NULL)
     {
      $this->e->add("Key: ".implode(" / ", $keys)." / ".$key." not expected");
      $ret=UT_FAILED;
     }
    }

    return $ret;
   }
   else //value
   {
    if($expected!==$values)
    {
     if(gettype($expected)=="string")
     {
      Diff::strings($values, $expected);
      if($keys)
       $this->e->add("Key: ".implode(" / ", $keys)." diff", "warning");
      else
       $this->e->add("Diff:", "warning");

      $expected=explode("\n", $expected);
      $values=explode("\n", $values);
      foreach(array_keys($expected) as $key)
      {
       if($expected[$key][0]=="=") //match
        $this->e->add("#".$key." matches line: ".substr($expected[$key], 1), "warning", false, false);
       if($expected[$key][0]=="-") //line extra
        $this->e->add("#".$key." didn't expect line: ".substr($values[$key], 1), "warning", false, false);
       else if($expected[$key][0]=="+") //line missing
        $this->e->add("#".$key." expect line: ".substr($expected[$key], 1), "warning", false, false);
       else if($expected[$key][0]=="!") //mismatch
       {
        $this->e->add("#".$key." expected line : ".substr($expected[$key], 1), "warning", false, false);
        $this->e->add("#".$key." doesn't match: ".substr($values[$key], 1), "warning", false, false);
       }
      }
     }
     else
     {
      if($keys)
       $this->e->add("Key: ".implode(" / ", $keys)." expected: ".$expected." doesn't match: ".$values);
      else
       $this->e->add("Expected: ".$expected." doesn't match: ".$values);
     }

     return UT_FAILED;
    }
   }

   return UT_OK;
  }
 }
 class UT_Classes extends UT_Core
 {
 }
 class UT_Modules extends UT_Core
 {
 }
 class UT_Plugins extends UT
 {
 }
?>
