<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/mutilate.data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/mutilate.php");
 require_once($_path["include"]."data.php");

 class UT_Mutilate_Data extends UT_Mutilate
 {
  function _addData($values)
  {
   $d=Data::cache("unit_testing_mutilate", "add", "item", Array("db" => "session"));

   $d->field["datetime"]["format"]["html"]["datetimeformat"]="Y-m-d H:i:s";
   $d->field["date"]["format"]["html"]["dateformat"]="Y-m-d";
   $d->field["time"]["format"]["html"]["timeformat"]="H:i:s";
   $d->format["html"]["datetimeformat"]="Y-m-d H:i:s";
   $d->format["html"]["dateformat"]="Y-m-d";
   $d->format["html"]["timeformat"]="H:i:s";

   unset($d->reference["catnr"], $d->field["catnr"]);
   $d->clearValues();

   foreach($values as $key => $value)
    $d->setValue($key, $value);

   if(!isset($values["name"]))
    $d->setValue("name", "test test");

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return false;
   }

   return true;
  }

  function _cleanData()
  {
   global $_session;

   $_session->set("sessionStorage.data.unit_testing_mutilate"); //ensure no values in DB

   return true;
  }


  function AddMutilation()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   if(!$d->addMutilation("name", "strip"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$d->addMutilation("name", "strip", 32))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->addMutilation("asdasdasd", "strip"))
   {
    $this->e->add("Shouldn't be able to add mutilation for unknown field");
    return UT_FAILED;
   }

   if($d->addMutilation("name", "asdasdasd"))
   {
    $this->e->add("Shouldn't be able to add unknown mutilation");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function RemoveMutilation()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   if(!$d->addMutilation("name", "strip"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$d->removeMutilation("name", "strip"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->removeMutilation("asdasdasd", "strip"))
   {
    $this->e->add("Shouldn't be able to remove mutilation for unknown field");
    return UT_FAILED;
   }

   if(!$d->removeMutilation("name", "asdasdasd"))
   {
    $this->e->add("Remove unknown mutilation should be ignored");
    return UT_FAILED;
   }

   if(!$d->removeMutilation("name", "strip"))
   {
    $this->e->add("Remove non-added mutilation should be ignored");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ApplyMutilations()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   if(!$d->addMutilation("name", "strip"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$d->applyMutilations("name", "Abcde fghij"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->removeMutilation("name", "strip");


   if($d->applyMutilations("asdasdasd", "Abcde fghij"))
   {
    $this->e->add("Shouldn't be able to apply mutilations for unknown field");
    return UT_FAILED;
   }

   if($d->applyMutilations("name", "Abcde fghij")!="Abcde fghij")
   {
    $this->e->add("Apply no mutilation should be ignored");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HumanReadableNumber()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("owner" => 123)))
    return UT_FAILED;

   if(!$d->addMutilation("owner", "humanReadableNumber"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="123".$d->format["html"]["decimalseparator"]."00")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"123.00\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableNumber");


   if(!$d->addMutilation("owner", "humanReadableNumber", 0))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="123")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"123\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableNumber");


   $this->_cleanData();
   if(!$this->_addData(Array("owner" => 1234)))
    return UT_FAILED;

   if(!$d->addMutilation("owner", "humanReadableNumber"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="1".$d->format["html"]["decimalseparator"]."23K")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"1.23K\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableNumber");


   if(!$d->addMutilation("owner", "humanReadableNumber", 0))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="1K")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"1K\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableNumber");


   if($d->addMutilation("name", "humanReadableNumber", 0))
   {
    $this->e->add("Shouldn't be able to add humanReadableNumber mutilation to name field");
    return UT_FAILED;
   }

//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array(123, 1234, Array(1234567)))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "humanReadableNumber"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="123".$d->format["html"]["decimalseparator"]."00")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"123.00\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="1".$d->format["html"]["decimalseparator"]."23K")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"1.23K\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="1".$d->format["html"]["decimalseparator"]."23M")
   {
    $this->e->add("Mutilation humanReadableNumber should be \"1.23M\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HumanReadableFileSize()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("owner" => 123)))
    return UT_FAILED;

   if(!$d->addMutilation("owner", "humanReadableFileSize"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="123".$d->format["html"]["decimalseparator"]."00B")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"123.00B\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableFileSize");


   if(!$d->addMutilation("owner", "humanReadableFileSize", 0))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="123B")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"123B\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableFileSize");


   $this->_cleanData();
   if(!$this->_addData(Array("owner" => 1234)))
    return UT_FAILED;

   if(!$d->addMutilation("owner", "humanReadableFileSize"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="1".$d->format["html"]["decimalseparator"]."23KB")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"1.23KB\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableFileSize");


   if(!$d->addMutilation("owner", "humanReadableFileSize", 0))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="1KB")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"1KB\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "humanReadableFileSize");


   if($d->addMutilation("name", "humanReadableFileSize", 0))
   {
    $this->e->add("Shouldn't be able to add humanReadableFileSize mutilation to name field");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array(123, 1234, Array(1234567)))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "humanReadableFileSize"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="123".$d->format["html"]["decimalseparator"]."00B")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"123.00B\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="1".$d->format["html"]["decimalseparator"]."23KB")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"1.23KB\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="1".$d->format["html"]["decimalseparator"]."23MB")
   {
    $this->e->add("Mutilation humanReadableFileSize should be \"1.23MB\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function LeadingZeros()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("owner" => 123)))
    return UT_FAILED;

   if(!$d->addMutilation("owner", "leadingZeros", 5))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="00123")
   {
    $this->e->add("Mutilation leadingZeros should be \"00123\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "leadingZeros");


   if(!$d->addMutilation("owner", "leadingZeros", 0))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["owner"]!="123")
   {
    $this->e->add("Mutilation leadingZeros should be \"123\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("owner", "leadingZeros");


   if($d->addMutilation("owner", "leadingZeros"))
   {
    $this->e->add("Should not be able to add leadingZeros mutilation without arguments", "error");
    return UT_FAILED;
   }


   if($d->addMutilation("name", "humanReadableFileSize", 0))
   {
    $this->e->add("Shouldn't be able to add humanReadableFileSize mutilation to name field");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array(123, 1234, Array(1234567)))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "leadingZeros", 5))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="00123")
   {
    $this->e->add("Mutilation leadingZeros should be \"00123\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="01234")
   {
    $this->e->add("Mutilation leadingZeros should be \"01234\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="1234567")
   {
    $this->e->add("Mutilation leadingZeros should be \"1234567\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Strip()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("name" => "Abcde fghij")))
    return UT_FAILED;

   if(!$d->addMutilation("name", "strip", 5))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="Ab...")
   {
    $this->e->add("Mutilation strip should be \"Abcde...\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "strip");


   if(!$d->addMutilation("name", "strip", 12))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="Abcde fghij")
   {
    $this->e->add("Mutilation strip should be \"Abcde fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "strip");


   if($d->addMutilation("owner", "strip"))
   {
    $this->e->add("Shouldn't be able to add strip mutilation to owner field");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde", "abcde fghij", Array("abcde fghij klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "strip", 8))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde")
   {
    $this->e->add("Mutilation strip should be \"abcde\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde...")
   {
    $this->e->add("Mutilation strip should be \"abcde...\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcde...")
   {
    $this->e->add("Mutilation strip should be \"abcde...\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Replace()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("name" => "Abcde fghij")))
    return UT_FAILED;

   if(!$d->addMutilation("name", "replace", "/Abcde/", "EDCBA"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="EDCBA fghij")
   {
    $this->e->add("Mutilation replace should be \"EDCBA fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "replace");


   if(!$d->addMutilation("name", "replace", "/Abcde/"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!=" fghij")
   {
    $this->e->add("Mutilation replace should be \" fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "replace");


   if($d->addMutilation("name", "replace", "asdasaf"))
   {
    $this->e->add("Should not be able to add replace mutilation with incorrect regexp", "error");
    return UT_FAILED;
   }

   if($d->addMutilation("name", "replace"))
   {
    $this->e->add("Should not be able to add replace mutilation without arguments", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde", "abcde fghij", Array("abcde fghij klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "replace", "/abcde/", ""))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="")
   {
    $this->e->add("Mutilation replace should be \"\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!=" fghij")
   {
    $this->e->add("Mutilation replace should be \"fghij\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!=" fghij klmno")
   {
    $this->e->add("Mutilation replace should be \"fghij klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ChangeCase()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("name" => "Abcde fghij")))
    return UT_FAILED;

   if(!$d->addMutilation("name", "case", "lower"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="abcde fghij")
   {
    $this->e->add("Mutilation changeCase to lower should be \"abcde fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "case");


   if(!$d->addMutilation("name", "case", "upper"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="ABCDE FGHIJ")
   {
    $this->e->add("Mutilation changeCase to upper should be \"ABCDE FGHIJ\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "case");


   if(!$d->addMutilation("name", "case", "first"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $this->_cleanData();
   if(!$this->_addData(Array("name" => "ABCDE FGHIJ")))
    return UT_FAILED;

   $d->clearValues();
   $d->get();
   $d->next();


   if($d->v["name"]!="Abcde fghij")
   {
    $this->e->add("Mutilation changeCase first letter should be \"Abcde fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "case");


   if(!$d->addMutilation("name", "case", "words"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["name"]!="Abcde Fghij")
   {
    $this->e->add("Mutilation changeCase first letter of each word should be \"Abcde Fghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("name", "case");


   if($d->addMutilation("name", "case", "fail"))
   {
    $this->e->add("Mutilation changeCase fail argument should fail", "error");
    return UT_FAILED;
   }

   if($d->addMutilation("name", "case"))
   {
    $this->e->add("Should not be able to add case mutilation without arguments", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde", "abcde fghij", Array("abcde fghij klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "case", "words"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="Abcde")
   {
    $this->e->add("Mutilation case should be \"Abcde\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="Abcde Fghij")
   {
    $this->e->add("Mutilation case should be \"Abcde Fghij\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="Abcde Fghij Klmno")
   {
    $this->e->add("Mutilation case should be \"Abcde Fghij Klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvert()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $d->field["time"]["format"]["html"]["timeformat"]="H:i:s";
   $d->format["html"]["timeformat"]="H:i:s";

   $this->_cleanData();
   if(!$this->_addData(Array()))
    return UT_FAILED;


   $format="Y m d";
   if(!$d->addMutilation("date", "dateConvert", $format))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["date"]!=date($format))
   {
    $this->e->add("Mutilation date convert should be \"".date($format)."\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("date", "dateConvert");


   $format="h i s";
   if(!$d->addMutilation("time", "dateConvert", $format))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["time"]!=date($format))
   {
    $this->e->add("Mutilation date convert should be \"".date($format)."\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("time", "dateConvert");


   $format="Y m d h i s";
   if(!$d->addMutilation("datetime", "dateConvert", $format))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["datetime"]!=date($format))
   {
    $this->e->add("Mutilation date convert should be \"".date($format)."\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("datetime", "dateConvert");


   if($d->addMutilation("datetime", "dateConvert"))
   {
    $this->e->add("Should not be able to add dateConvert mutilation without arguments", "error");
    return UT_FAILED;
   }


   if($d->addMutilation("name", "dateConvert", $format))
   {
    $this->e->add("Should not be able to add dateConvert mutilation to name field", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function TimezoneConvert()
  {
   $format="Y-m-d H:i:s";
   $sourceTimezone=date_default_timezone_get();

   if($sourceTimezone=="Europe/Amsterdam")
    $targetTimezone="America/New_York";
   else
    $targetTimezone="Europe/Amsterdam";

   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));
   $this->_cleanData();
   if(!$this->_addData(Array()))
    return UT_FAILED;

   $d->field["datetime"]["format"]["html"]["datetimeformat"]=$format;

   if(!$d->addMutilation("datetime", "timezoneConvert", $targetTimezone))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["datetime"]!=DT::timezoneConvertS(date($format), $sourceTimezone, $targetTimezone, $format))
   {
    $this->e->add("Mutilation timezone convert should be \"".DT::timezoneConvertS(date($format), $sourceTimezone, $targetTimezone, $format)."\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("datetime", "timezoneConvert");


   if(!$d->addMutilation("datetime", "timezoneConvert", $targetTimezone))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $format="Y m d H i s";
   if(!$d->addMutilation("datetime", "dateConvert", $format))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["datetime"]!=DT::timezoneConvertS(date($format), $sourceTimezone, $targetTimezone, $format))
   {
    $this->e->add("Mutilation timezone convert should be \"".DT::timezoneConvertS(date($format), $sourceTimezone, $targetTimezone, $format)."\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("datetime", "dateConvert");
   $d->removeMutilation("datetime", "timezoneConvert");


   if(!$d->addMutilation("datetime", "dateConvert", $format))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if($d->addMutilation("datetime", "timezoneConvert", $targetTimezone))
   {
    $this->e->add("Should not be able to add timezoneConvert mutilation after dateConvert", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("datetime", "dateConvert");


   if($d->addMutilation("datetime", "timezoneConvert"))
   {
    $this->e->add("Should not be able to add timezoneConvert mutilation to without arguments", "error");
    return UT_FAILED;
   }


   if($d->addMutilation("name", "timezoneConvert", $targetTimezone))
   {
    $this->e->add("Should not be able to add timezoneConvert mutilation to name field", "error");
    return UT_FAILED;
   }

   if($d->addMutilation("date", "timezoneConvert", $targetTimezone))
   {
    $this->e->add("Should not be able to add timezoneConvert mutilation to date field", "error");
    return UT_FAILED;
   }

   if($d->addMutilation("time", "timezoneConvert", $targetTimezone))
   {
    $this->e->add("Should not be able to add timezoneConvert mutilation to time field", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function StripTags()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("html" => "Abcde<br/><a href=\"\">fghij</a>")))
    return UT_FAILED;

   if(!$d->addMutilation("html", "stripTags"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcdefghij")
   {
    $this->e->add("Mutilation stripTags should be \"Abcdefghij\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "stripTags");


   if($d->addMutilation("nr", "stripTags"))
   {
    $this->e->add("Should not be able to add stripTags mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde<br/>", "abcde <b>fghij</b>", Array("abcde<br/>fghij<br/>klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "stripTags"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde")
   {
    $this->e->add("Mutilation stripTags should be \"abcde\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde fghij")
   {
    $this->e->add("Mutilation stripTags should be \"abcde fghij\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcdefghijklmno")
   {
    $this->e->add("Mutilation stripTags should be \"abcdefghijklmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HTMLEncode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("html" => "Abcde<br/><a href=\"\">fghij</a>")))
    return UT_FAILED;

   if(!$d->addMutilation("html", "encode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;")
   {
    $this->e->add("Mutilation encode should be \"Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "encode");


   if(!$d->addMutilation("html", "encode", "html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;")
   {
    $this->e->add("Mutilation encode should be \"Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "encode");


   if(!$d->addMutilation("html", "encode", "url;html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde%3Cbr%2F%3E%3Ca+href=%22%22%3Efghij%3C%2Fa%3E")
   {
    $this->e->add("Mutilation encode should be \"Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "encode");


   if($d->addMutilation("nr", "encode"))
   {
    $this->e->add("Should not be able to add encode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde<br/>", "abcde <b>fghij</b>", Array("abcde<br/>fghij<br/>klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "encode", "html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde&lt;br/&gt;")
   {
    $this->e->add("Mutilation encode should be \"abcde&lt;br/&gt;\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde &lt;b&gt;fghij&lt;/b&gt;")
   {
    $this->e->add("Mutilation encode should be \"abcde &lt;b&gt;fghij&lt;/b&gt;\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcde&lt;br/&gt;fghij&lt;br/&gt;klmno")
   {
    $this->e->add("Mutilation encode should be \"abcde&lt;br/&gt;fghij&lt;br/&gt;klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HTMLDecode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("html" => "Abcde&lt;br/&gt;&lt;a href=&quot;&quot;&gt;fghij&lt;/a&gt;")))
    return UT_FAILED;

   if(!$d->addMutilation("html", "decode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde<br/><a href=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/><a href=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "decode");


   if(!$d->addMutilation("html", "decode", "html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde<br/><a href=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/><a href=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "decode");


   $this->_cleanData();
   if(!$this->_addData(Array("html" => "Abcde%3Cbr%2F%3E%3Ca+href=%22%22%3Efghij%3C%2Fa%3E")))
    return UT_FAILED;

   if(!$d->addMutilation("html", "decode", "html;url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["html"]!="Abcde<br/><a href=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/><a href=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("html", "decode");


   if($d->addMutilation("nr", "decode"))
   {
    $this->e->add("Should not be able to add decode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde&lt;br/&gt;", "abcde &lt;b&gt;fghij&lt;/b&gt;", Array("abcde&lt;br/&gt;fghij&lt;br/&gt;klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "decode", "html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde<br/>")
   {
    $this->e->add("Mutilation decode should be \"abcde<br/>\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde <b>fghij</b>")
   {
    $this->e->add("Mutilation decode should be \"abcde <b>fghij</b>\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcde<br/>fghij<br/>klmno")
   {
    $this->e->add("Mutilation decode should be \"abcde<br/>fghij<br/>klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function MarkupEncode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session", "format" => "text"));

   $this->_cleanData();
   if(!$this->_addData(Array("text" => "Abcde<br/><a href=\"fghij\" title=\"fghij\">fghij</a>")))
    return UT_FAILED;

   if(!$d->addMutilation("text", "encode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde
[link=fghij]fghij[link]")
   {
    $this->e->add("Mutilation encode should be \"Abcde
[link=fghij]fghij[link]\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "encode");


   if(!$d->addMutilation("text", "encode", "markup"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde
[link=fghij]fghij[link]")
   {
    $this->e->add("Mutilation encode should be \"Abcde
[link=fghij]fghij[link]\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "encode");


   if(!$d->addMutilation("text", "encode", "markup;url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde%0A%5Blink=fghij%5Dfghij%5Blink%5D")
   {
    $this->e->add("Mutilation encode should be \"Abcde%0A%5Blink=fghij%5Dfghij%5Blink%5D\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "encode");


   if($d->addMutilation("nr", "encode"))
   {
    $this->e->add("Should not be able to add encode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde<br/>", "abcde <a href=\"fghij\" title=\"fghij\">fghij</a>", Array("abcde<br/>fghij<br/>klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "encode", "markup"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde
")
   {
    $this->e->add("Mutilation encode should be \"abcde
\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde [link=fghij]fghij[link]")
   {
    $this->e->add("Mutilation encode should be \"abcde [link=fghij]fghij[link]\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcde
fghij
klmno")
   {
    $this->e->add("Mutilation encode should be \"abcde
fghij
klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function MarkupDecode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session", "format" => "text"));

   $this->_cleanData();
   if(!$this->_addData(Array("text" => "Abcde
[link=fghij title=\"\"]fghij[link]")))
    return UT_FAILED;

   if(!$d->addMutilation("text", "decode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "decode");


   if(!$d->addMutilation("text", "decode", "markup"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "decode");

   $this->_cleanData();
   if(!$this->_addData(Array("text" => "Abcde%0A%5Blink=fghij+title%3D%22%22%5Dfghij%5Blink%5D")))
    return UT_FAILED;

   if(!$d->addMutilation("text", "decode", "markup;url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["text"]!="Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"Abcde<br/>
<a href=\"fghij\" title=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("text", "decode");

   if($d->addMutilation("nr", "decode"))
   {
    $this->e->add("Should not be able to add decode mutilation to nr field", "error");
    return UT_FAILED;
   }

//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("abcde
", "abcde [link=fghij title=\"\"]fghij[link]", Array("abcde
fghij
klmno")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "decode", "markup"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="abcde<br/>
")
   {
    $this->e->add("Mutilation decode should be \"abcde<br/>
\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="abcde <a href=\"fghij\" title=\"\">fghij</a>")
   {
    $this->e->add("Mutilation decode should be \"abcde <a href=\"fghij\" title=\"\">fghij</a>\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="abcde<br/>
fghij<br/>
klmno")
   {
    $this->e->add("Mutilation decode should be \"abcde<br/>
fghij<br/>
klmno\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function URLEncode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("url" => "http://domain.com/?arg1=_asd&arg2=@asd")))
    return UT_FAILED;

   if(!$d->addMutilation("url", "encode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&arg2=%40asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=%40asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "encode");


   if(!$d->addMutilation("url", "encode", "url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&arg2=%40asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=%40asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "encode");


   if(!$d->addMutilation("url", "encode", "url;html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&amp;arg2=%40asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=%40asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "encode");


   if($d->addMutilation("nr", "encode"))
   {
    $this->e->add("Should not be able to add encode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("http://domain.com/?arg1=_asd&arg2=@asd", "http://domain.com/", Array("http://domain.com/?arg1=_asd&arg2=@asd")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "encode", "url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="http://domain.com/?arg1=_asd&arg2=%40asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=%40asd\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="http://domain.com/")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="http://domain.com/?arg1=_asd&arg2=%40asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=%40asd\"", "error");
    return ut_failed;
   }


   return UT_OK;
  }


  function URLDecode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("url" => "http://domain.com/?arg1=_asd&arg2=%40asd")))
    return UT_FAILED;

   if(!$d->addMutilation("url", "decode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&arg2=@asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=@asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "decode");


   if(!$d->addMutilation("url", "decode", "url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&arg2=@asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=@asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "decode");


   $this->_cleanData();
   if(!$this->_addData(Array("url" => "http://domain.com/?arg1=_asd&amp;arg2=%40asd")))
    return UT_FAILED;

   if(!$d->addMutilation("url", "decode", "html;url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["url"]!="http://domain.com/?arg1=_asd&arg2=@asd")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/?arg1=_asd&arg2=@asd\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("url", "decode");


   if($d->addMutilation("nr", "decode"))
   {
    $this->e->add("Should not be able to add decode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("http://domain.com/?arg1=_asd&arg2=%40asd", "http://domain.com/", Array("http://domain.com/?arg1=_asd&arg2=%40asd")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "decode", "url"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="http://domain.com/?arg1=_asd&arg2=@asd")
   {
    $this->e->add("Mutilation decode should be \"http://domain.com/?arg1=_asd&arg2=@asd\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="http://domain.com/")
   {
    $this->e->add("Mutilation decode should be \"http://domain.com/\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="http://domain.com/?arg1=_asd&arg2=@asd")
   {
    $this->e->add("Mutilation decode should be \"http://domain.com/?arg1=_asd&arg2=@asd\"", "error");
    return ut_failed;
   }


   return UT_OK;
  }



  function Base64Encode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("base64" => "http://domain.com/")))
    return UT_FAILED;

   if(!$d->addMutilation("base64", "encode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="aHR0cDovL2RvbWFpbi5jb20v")
   {
    $this->e->add("Mutilation encode should be \"aHR0cDovL2RvbWFpbi5jb20v\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "encode");


   if(!$d->addMutilation("base64", "encode", "base64"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="aHR0cDovL2RvbWFpbi5jb20v")
   {
    $this->e->add("Mutilation encode should be \"aHR0cDovL2RvbWFpbi5jb20v\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "encode");


   $this->_cleanData();
   if(!$this->_addData(Array("base64" => "some text<br/>
with lineending")))
    return UT_FAILED;

   if(!$d->addMutilation("base64", "encode", "base64;html"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="c29tZSB0ZXh0Jmx0O2JyLyZndDsKd2l0aCBsaW5lZW5kaW5n")
   {
    $this->e->add("Mutilation encode should be \"c29tZSB0ZXh0Jmx0O2JyLyZndDsKd2l0aCBsaW5lZW5kaW5n\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "encode");


   if($d->addMutilation("nr", "encode"))
   {
    $this->e->add("Should not be able to add encode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("http://domain.com/", "some text<br/>
with lineending", Array("fpgmp9)")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "encode", "base64"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="aHR0cDovL2RvbWFpbi5jb20v")
   {
    $this->e->add("Mutilation encode should be \"aHR0cDovL2RvbWFpbi5jb20v\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="c29tZSB0ZXh0PGJyLz4Kd2l0aCBsaW5lZW5kaW5n")
   {
    $this->e->add("Mutilation encode should be \"c29tZSB0ZXh0PGJyLz4Kd2l0aCBsaW5lZW5kaW5n\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="ZnBnbcZwOSk=")
   {
    $this->e->add("Mutilation encode should be \"ZnBnbcZwOSk=\"", "error");
    return ut_failed;
   }


   return UT_OK;
  }


  function Base64Decode()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("base64" => "aHR0cDovL2RvbWFpbi5jb20v")))
    return UT_FAILED;

   if(!$d->addMutilation("base64", "decode"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="http://domain.com/")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "decode");


   if(!$d->addMutilation("base64", "decode", "base64"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="http://domain.com/")
   {
    $this->e->add("Mutilation encode should be \"http://domain.com/\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "decode");


   $this->_cleanData();
   if(!$this->_addData(Array("base64" => "c29tZSB0ZXh0Jmx0O2JyLyZndDsKd2l0aCBsaW5lZW5kaW5n")))
    return UT_FAILED;

   if(!$d->addMutilation("base64", "decode", "html;base64"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["base64"]!="some text<br/>
with lineending")
   {
    $this->e->add("Mutilation encode should be \"some text<br/>
with lineending\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("base64", "decode");


   if($d->addMutilation("nr", "decode"))
   {
    $this->e->add("Should not be able to add decode mutilation to nr field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("aHR0cDovL2RvbWFpbi5jb20v", "c29tZSB0ZXh0PGJyLz4Kd2l0aCBsaW5lZW5kaW5n", Array("ZnBnbcZwOSk=")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "decode", "base64"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="http://domain.com/")
   {
    $this->e->add("Mutilation decode should be \"http://domain.com/\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="some text<br/>
with lineending")
   {
    $this->e->add("Mutilation decode should be \"some text<br/>
with lineending\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="fpgmp9)")
   {
    $this->e->add("Mutilation decode should be \"fpgmp9)\"", "error");
    return ut_failed;
   }


   return UT_OK;
  }



  function ScrambleEmail()
  {
   $d=Data::cache("unit_testing_mutilate", "view", "all", Array("db" => "session"));

   $this->_cleanData();
   if(!$this->_addData(Array("email" => "test@domain.com")))
    return UT_FAILED;

   if(!$d->addMutilation("email", "scrambleEmail"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->get();
   $d->next();

   if($d->v["email"]!="test(at)domain[dot]com")
   {
    $this->e->add("Mutilation replace should be \"test(at)domain[dot]com\"", "error");
    return UT_FAILED;
   }

   $d->removeMutilation("email", "scrambleEmail");


   if($d->addMutilation("name", "scrambleEmail"))
   {
    $this->e->add("Should not be able to add scrambleEmail mutilation to name field", "error");
    return UT_FAILED;
   }


//array
   $this->_cleanData();
   if(!$this->_addData(Array("array" => Array("test@domain.com", "t.e@s.t.com", Array("test@domain.com")))))
    return UT_FAILED;

   if(!$d->addMutilation("array", "scrambleEmail"))
   {
    $this->e->add($d->e);
    return UT_FAILED;
   }

   $d->e->clear();
   $d->get();
   $d->next();

   if($d->v["array"][0]!="test(at)domain[dot]com")
   {
    $this->e->add("Mutilation scrambleEmail should be \"test(at)domain[dot]com\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][1]!="t[dot]e(at)s[dot]t[dot]com")
   {
    $this->e->add("Mutilation scrambleEmail should be \"t[dot]e(at)s[dot]t[dot]com\"", "error");
    return UT_FAILED;
   }
   if($d->v["array"][2][0]!="test(at)domain[dot]com")
   {
    $this->e->add("Mutilation scrambleEmail should be \"test(at)domain[dot]com\"", "error");
    return ut_failed;
   }

   return UT_OK;
  }

 }

?>
