<?

 /*****************************************************************************
 * Filename: include/session.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 $_session=new Session();

 class Session
 {
  function __construct()
  {
   global $_jseskey;

   if(!session_id())
    session_start();

   $this->global="_global";

   //setup session key
   if($_jseskey)
    $this->setKey($_jseskey);
   else if(isset($_SESSION["jseskey"]))
    $this->setKey($_SESSION["jseskey"]);
   else
    $this->setKey("default");
  }


  /*
   * Main global/local session keys
   */

  function setGlobal()
  {
   //setup global session
   if(!isset($_SESSION["jses"][$this->global]))
    $_SESSION["jses"][$this->global]=Array();

   $this->values=&$_SESSION["jses"][$this->global];
   return true;
  }

  function setCurrent()
  {
   $this->values=&$_SESSION["jses"][$this->key];
   return true;
  }


  function setKey($key)
  {
   //save key
   $_SESSION["jseskey"]=$this->key=$key;

   //setup current session
   if(!isset($_SESSION["jses"][$this->key]))
    $_SESSION["jses"][$this->key]=Array();

   $this->values=&$_SESSION["jses"][$this->key];
   return true;
  }

  function getKey()
  {
   return $this->key;
  }

  function getKeys()
  {
   $keys=Array();
   foreach(array_keys($_SESSION["jses"]) as $key)
    if($key!=$this->global)
     $keys[]=$key;

   return $keys;
  }


  /*
   * Set/Get session variables
   */

  function set($key, $value=NULL)
  {
   //get keys
   $keys=explode(".", $key);
   $l=sizeof($keys);

   //lookup keys
   $tmp=&$this->values;
   for($c=0;$c<$l;$c++)
   {
    if(is_array($tmp))
    {
     if(!isset($tmp[$keys[$c]]))
      $tmp[$keys[$c]]=Array();

     $tmp=&$tmp[$keys[$c]];
    }
    else if(is_object($tmp))
    {
     $tmp2=$keys[$c];

     if(!isset($tmp->$tmp2))
      $tmp->$tmp2=Array();

     $tmp=&$tmp->$tmp2;
    }
    else //non array/object
     return false;
   }

   $tmp=$value;
   return true;
  }

  function get($key){return $this->getReference($key);}
  function &getReference($key)
  {
   //get keys
   $keys=explode(".", $key);
   $l=sizeof($keys);

   //lookup keys
   $empty=NULL;
   $tmp=&$this->values;
   for($c=0;$c<$l;$c++)
   {
    if(is_array($tmp))
    {
     if(!isset($tmp[$keys[$c]]))
      return $empty;

     $tmp=&$tmp[$keys[$c]];
    }
    else if(is_object($tmp))
    {
     $tmp2=$keys[$c];

     if(!isset($tmp->$tmp2))
      return $empty;

     $tmp=&$tmp->$tmp2;
    }
    else //non array/object
     return $empty;
   }

   return $tmp;
  }


  /*
   * JAS backend/frontend projects
   */

  function backendKey()
  {
   return "jas";
  }

  function inBackend()
  {
   if($this->key==$this->backendKey())
    return true;

   return false;
  }

  function annex()
  {
   //incorporate default session into JAS
   if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&!isset($_SESSION["jses"][$_SESSION["jses"]["default"]["project"]]))
   {
    $_SESSION["jses"][$_SESSION["jses"]["default"]["project"]]=&$_SESSION["jses"]["default"];

    if($_SESSION["jses"]["default"]["project"]!="jas") //normal project
     $this->set("backendProject", $_SESSION["jses"]["default"]["project"]); //activate in backend
   }

   return true;
  }

  function frontend()
  {
   return $this->switchBackend(false);
  }

  function backend()
  {
   return $this->switchBackend(true);
  }

  function switchBackend($backend=NULL) //autodetect
  {
   global $_path, $_session, $_config;

   if($this->inBackend()) //currently in backend
   {
    if($backend===true) //switch to backend
     return true; //done

    if(!$this->get("backendProject")) //no project selected
     return true;

    $key=$this->get("backendProject"); //switch to frontend
   }
   else //currently in frontend
   {
    if($backend===false) //switch to frontend
     return true; //done

    $key=$this->backendKey(); //switch to backend
   }

   $this->setKey($key);

   if(!$this->inBackend()) //switched to frontend
    if(!$this->get("project")) //project not yet set in session
     $this->set("project", $key);

   unset($_path["project"]); //clear project path definition
   require($_path["root"]."include.php"); //init basics

   return true;
  }

  function setBackendProject($project)
  {
   if(!$this->inBackend()) //not in backend
    return false;

   if(!$project)
    $this->set("backendProject");
   else if(in_array($project, JAS::projects()))
    $this->set("backendProject", $project);
   else
    return false;

   return true;
  }

 }
?>
