<?

 /*****************************************************************************
 * Filename: include/storage/session.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."errors.php");

 class System_Storage_session extends Storage
 {
  function __construct(&$d)
  {
   global $_session;

   parent::__construct($d, "session");
   $this->session="sessionStorage";

   if(!($db=$this->getConfig()))
    return;

   if(!$db["host"])
   {
    $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
    return;
   }

   $var=$db["host"];

   if(isset($db["name"])&&$db["name"]=="global") //store in global _SESSION variable
   {
    $_session->setGlobal(); //global session

    if($_session->get($this->session.".".$var.".".$this->data->module)===NULL)
     $_session->set($this->session.".".$var.".".$this->data->module, Array());

    $this->storage=&$_session->getReference($this->session.".".$var.".".$this->data->module);

    $_session->setCurrent(); //back to current session
   }
   else //store in _session
   {
    if($_session->get($this->session.".".$var.".".$this->data->module)===NULL)
     $_session->set($this->session.".".$var.".".$this->data->module, Array());

    $this->storage=&$_session->getReference($this->session.".".$var.".".$this->data->module);
   }

   $this->clear();
  }


  function table()
  {
   return true;
  }

  function checkRights($action)
  {
   return true;
  }

  function insertID()
  {
   $d=&$this->data; //D for short

   if(!($id=$d->getID()))
   {
    $this->e->add("Refusing to get insertID, no ID field", "sql", false, ($this->data->module!="log")); //don't log
    return false;
   }

   if(($d->dataformat=="sql"||$d->extends)) //allow force id insertion
   {
    if($insertID=$d->sqlValue($id))
    {
     if(!$this->unique($id))
     {
      $this->e->add("Refusing to get insertID, ID \"".$insertID."\" already exists", "sql", false, ($this->data->module!="log")); //don't log
      return false;
     }

     return $insertID;
    }
   }

   $insertID=0;

   for($c=0;$c<sizeof($this->storage);$c++)
    if(isset($this->storage[$c][$id])&&$this->storage[$c][$id]>$insertID)
     $insertID=$this->storage[$c][$id];

   $insertID++;

   return $insertID;
  }

  function get($gather="data")
  {
   $d=&$this->data; //D for short

   if($d->check("valuetype", "set", "aggregation_field"))
   {
    $this->e->add($this->type." storage doesn't support: \"aggregation\"", "system");
    return false;
   }

   if(!$this->_filterDataset($this->storage, $gather))
    return false;

   if($gather=="count"||$gather=="both")
   {
    if(!$this->_setCount())
     return false;

    if($gather=="count")
     return true;
   }

   if(!$this->_sort())
    return false;

   if(!$this->_filterShowTotal())
    return false;

   return true;
  }

  function add()
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("add"))) //nothing to add... ?
    return true;

   if(!($insertID=$this->insertID()))
    return false;

   $id=$d->getID();
   $d->setValue($id, $insertID, $d->vc);

   $c=sizeof($this->storage);
   $this->storage[$c]=Array();
   $this->storage[$c][$id]=$insertID;

   foreach($values as $k => $v)
    $this->storage[$c][$k]=$v;

   return true;
  }

  function edit()
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("edit"))) //nothing to update
    return true;

   $id=$d->getID();
   $idValue=$d->sqlValue($id);

   $l=sizeof($this->storage);
   for($c=0;$c<$l;$c++)
   {
    if($this->storage[$c][$id]==$idValue)
    {
     foreach($values as $k => $v)
      $this->storage[$c][$k]=$v;

     break;
    }
   }

   return true;
  }

  function delete()
  {
   $d=&$this->data; //D for short

   $id=$d->getID();
   $idValue=$d->sqlValue($id);

   $l=sizeof($this->storage);
   for($c=0;$c<$l;$c++)
   {
    if($this->storage[$c][$id]==$idValue)
    {
     for($x=$c+1;$x<$l;$x++)
      $this->storage[$x-1]=$this->storage[$x];

     unset($this->storage[$l-1]);

     break;
    }
   }

   return true;
  }

 }
?>
