<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/configuration.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."data.php");

 class Backend_Configuration extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->reload=false;
   $this->subsection=false;

   $this->nr=false;
   $this->module=false;
   $this->modules=Array();
   $this->multipleModules=Array("configModules", "configFields", "configRepositories", "configSites");

   $this->fileCache=false;
   $this->memoryCacheData=false;
  }

  function main($section=false, $subsection=false, $nr=false)
  {
   global $_session, $_config, $_path;

   //section
   if(!parent::main($section))
    return false;

   //set backend theme
   if(($theme=request("setTheme"))!==NULL) //theme change
   {
    //clear theme cookie
    setCookie("backendTheme", false, 1, $_path["HTML"]["root"], $_SERVER["SERVER_NAME"]);
    unset($_COOKIE["backendTheme"]);

    if(preg_match("/^\w+$/", $theme)&&$theme!="default")
    {
     if($this->backend->fs->exists($_path["project"]["style"].$theme.".css"))
     {
      //set (valid) theme cookie
      setCookie("backendTheme", $theme, time()+60*60*24*365*5, $_path["HTML"]["root"], $_SERVER["SERVER_NAME"]);
      $_COOKIE["backendTheme"]=$theme;
     }
    }
   }

   $_session->frontend();

   //reload config
   if(request("reloadConfig")) //actual reloading already done in backend
    $this->e->add("Configuration reloaded", "note");

   if(strpos($this->section, "config")===0) //view section
   {
    if(!in_array($this->section, JAS::modules())) //invalid module
    {
     $this->e->add("Module: \"".$this->section."\" not found", "error");
     $this->section=false;
    }
    else
    {
     $this->module=$this->section;

     $d=Data::cache($this->module, "view", "all", Array("skipCheckTables" => true));

     if(!$d->tableExists())
      Headers::redirectS($this->url("database/".$this->module."/"));

     if($subsection=="add"||$subsection=="edit"||$subsection=="delete") //cms section
     {
      $this->subsection=$subsection;
      $this->section="cms";
     }
     else
      $this->section="view";

     if(!in_array($this->module, $this->multipleModules)) //single entry config module
     {
      //get nr
      $d->setShowFields("nr");
      $d->clearValues();

      if($d->get()&&$d->next())
       $this->nr=$d->v("id");

      if($this->subsection=="add"&&$this->nr)
       $this->subsection="edit";
      else if($this->subsection=="edit"&&!$this->nr)
       $this->subsection="add";
     }
     else //multi module
     {
      if($nr)
       $this->nr=$nr;
      else if($this->section=="view") //no item to view
       $this->section=false; //view all entries for multi module
     }

     if($this->section=="cms")
     {
      $this->cms=new CMS(Array(
       "handleSubmit" => false,
       "showErrors" => false,
       "showTitles" => false,
      ));

      if(!$this->backend->rewrite) //vars
       foreach($this->backend->pageVars as $k => $page)
        $this->cms->addVar("p[".$k."]", $page);

      //set page as CMS in history
      $hs=new History();
      if(!$hs->cms())
       $this->e->add($hs->e);

      //init data
      $d=Data::cache($this->module, $this->subsection, "item");

      if($this->nr)
       $d->setValue($d->getID(), $this->nr);

      $this->cms->setDataForm($d, $this->form);

      //submit cms
      if($this->cms->formHandleSubmit()) //submit successful
      {
       $_config->reload();
       $this->e->add("Configuration reloaded", "note");

       if(!request("cmsContinue")) //no more cms action?
        $this->reload=$this->cms->handleReloadPage();
      }

      $this->e->add($d->e);
     }
    }
   }

   if(!$this->section||$this->section=="view")
   {
    $this->backend->search("modules", "config"); //inline search config modules

    foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
     if(strpos($module, "config")===0) //only modules with config prefix
      $this->modules[$module]=$v;

    foreach(array_keys($this->modules) as $module)
    {
     $this->modules[$module]["multiple"]=(in_array($module, $this->multipleModules));

     if($this->modules[$module]["enabled"])
     {
      $this->modules[$module]["data"]=Data::cache($module, "view", "all");
      $this->modules[$module]["data"]->setShowFields(array_merge(Array($this->modules[$module]["data"]->getID(), "updated"), $this->modules[$module]["data"]->check("properties", "title")));
      $this->modules[$module]["data"]->addMutilation("updated", "timeAgo");
      $this->modules[$module]["data"]->clearValues();

      $this->modules[$module]["data"]->get();
     }
    }

    if($_config->userRoot())
    {
     //clear file cache
     if($this->backend->fs->exists($_path["project"]["cache"]))
      $this->fileCache=$this->backend->fs->files($_path["project"]["cache"]);

     if(request("clearFileCache")&&$this->fileCache)
     {
      foreach($this->fileCache as $file)
       $this->backend->fs->delete($file);

      $this->e->add("File cache cleared", "note");
      $this->fileCache=false;
     }
    }
   }

   $_session->backend();
   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config, $_path;

   $_session->frontend();

   if($this->section=="view")
   {
    $d=Data::cache($this->module, "view", "item");
    $this->language->read("module_".$this->module);
    $d->setValue($d->getID(), $this->nr);

    if(!($r=$this->viewItem($d, Array("show" => "all", "get" => 1))))
     return false;

    //set title after viewItem got data
    $this->backend->setTitle($this->icon("config_dim"),
     $this->link($this->language->t("Configuration"), Array("configuration/")),
     $this->link(strip_tags($d->v("title")), $this->url()));

    $r="<div class=\"twothird\">\n".$r."</div>\n";

    //actions
    $v=$this->modules[$d->module];

    if(($v["multiple"]||!$v["data"]->vmax)&&$d->r("add"))
     $this->menuItem("actions", $this->language->t("Add"), $this->url("configuration/".$d->module."/add/"), $this->language->t("Add: ".$d->title), "add");
    if($d->r("edit"))
     $this->menuItem("actions", $this->language->t("Edit"), $this->url("configuration/".$d->module."/edit/".($v["multiple"]?$d->v("id")."/":false)), $this->language->t("Edit: ".$d->v("title")), "edit");
    if($d->r("delete"))
     $this->menuItem("actions", $this->language->t("Delete"), $this->url("configuration/".$d->module."/delete/".($v["multiple"]?$d->v("id")."/":false)), $this->language->t("Delete: ".$d->v("title")), "delete");
   }
   else //default section
   {
    $this->backend->setTitle($this->icon("config_dim"),
     $this->link($this->language->t("Configuration"), Array("configuration/")));

    foreach($this->modules as $module => $v)
    {
     if(!$v["enabled"])
      continue;

     if($this->module&&$module!=$this->module)
      continue;

     if($search=request("search"))
      if(!stristr($module, $search)&&!stristr($v["title"], $search))
       continue;

     $this->tableRow("config", true);
     if($module=="config")
     {
      $this->tableColumn("config", $v["title"]);
      $this->tableColumn("config", $this->language->t("Updated"));
     }
     else
      $this->tableColumn("config", $v["title"], false, false, Array("colspan" => 2));


     if($v["multiple"]||!$v["data"]->vmax)
      $this->tableColumn("config", $this->link("", Array("configuration/".$module."/add/"), $this->language->t("Add").": ".$v["title"], "add", Array((!$v["data"]->r("add")?"disabled":false))), false, false, Array("class" => "cms", ($v["data"]->validAction("add")?false:"disabled")));
     else
      $this->tableColumn("config", "");

     while($v["data"]->next())
     {
      $this->tableRow("config");

      if($v["multiple"])
       $this->tableColumn("config", $v["data"]->v("title"), Array("configuration/".$module."/view/".$v["data"]->v("id")."/"), $this->language->t("Show: ".$v["data"]->v("title")));
      else
       $this->tableColumn("config", $v["data"]->v("title"), Array("configuration/".$module."/"), $this->language->t("Show: ".$v["data"]->v("title")));

      $this->tableColumn("config", $v["data"]->v["updated"], false, false, Array("class" => "updated"));

      $this->menuItem("cms_".$v["data"]->module."_".$v["data"]->v("id"), $this->language->t("Edit"), ($v["data"]->r("edit")?$this->url("configuration/".$v["data"]->module."/edit/".($v["multiple"]?$v["data"]->v("id")."/":false)):false), $this->language->t("Edit: ".$v["data"]->v("title")), "edit".($v["data"]->r("edit")?"":"_disabled"));
      $this->menuItem("cms_".$v["data"]->module."_".$v["data"]->v("id"), $this->language->t("Delete"), ($v["data"]->r("delete")?$this->url("configuration/".$v["data"]->module."/delete/".($v["multiple"]?$v["data"]->v("id")."/":false)):false), $this->language->t("Delete: ".$v["data"]->v("title")), "delete".($v["data"]->r("delete")?"":"_disabled"));

      $this->tableColumn("config", $this->icon("cms").$this->menu("cms_".$v["data"]->module."_".$v["data"]->v("id")), false, false, Array("class" => "cms popup right"));
     }
    }

    $r="<div class=\"twothird\">\n";
    $r.=$this->table("config");
    $r.="</div>\n";
   }


   //actions
   $this->menuItem("actions", $this->language->t("Reload config"), $this->url(false, Array("reloadConfig" => "true")), $this->language->t("Reload configuration"), "reload");
   $this->menuItem("actions", $this->language->t("Clear file cache"), $this->url(false, Array("clearFileCache" => "true")), $this->language->t("Clear file cache"), "cache", ($this->fileCache?false:Array("disabled")));
   $this->menuItem("actions", $this->language->t("Language info"), $this->url("language/"), $this->language->t("View language information"), "language");

   if(request("backendTheme", "COOKIE")!="dark") //dark theme
    $this->menuItem("actions", $this->language->t("Dark theme"), $this->url(false, Array("setTheme" => "dark")), $this->language->t("Use dark theme"), "theme");
   else //default theme
    $this->menuItem("actions", $this->language->t("Default theme"), $this->url(false, Array("setTheme" => "")), $this->language->t("Use default theme"), "theme");

   //modules list
   $this->tableRow("modules", true);
   $this->tableColumn("modules", "", false, false, Array("colspan" => 2));

   foreach($this->modules as $module => $v)
   {
    if(!$v["enabled"])
    {
     $this->tableRow("modules", false, Array("disabled"));
     $this->tableColumn("modules", $v["title"]);
     $this->tableColumn("modules", $this->link("", Array("database/".$module."/"), $this->language->t("Create table: ".$module), "database", Array((!$_config->userRoot()?"disabled":false))), false, false, Array("class" => "cms"));
    }
    else
    {
     $this->tableRow("modules");

     if(!$v["data"]->vmax)
     {
      $this->tableColumn("modules", $v["title"]);
      $this->tableColumn("modules", $this->link("", Array("configuration/".$module."/add/"), $this->language->t("Add").": ".$v["title"], "add", Array((!$v["data"]->r("add")?"disabled":false))), false, false, Array("class" => "cms"));
     }
     else
     {
      $this->tableColumn("modules", $v["title"], Array("configuration/".$module."/"), $this->language->t("Show: ".$v["title"]));

      if(!$v["multiple"])
       $this->tableColumn("modules", $this->link("", Array("configuration/".$module."/edit/"), $this->language->t("Edit: ".$v["title"]), "edit", Array((!$v["data"]->r("edit")?"disabled":false))), false, false, Array("class" => "cms"));
      else //multi
       $this->tableColumn("modules", $this->link("", Array("configuration/".$module."/add/"), $this->language->t("Add").": ".$v["title"], "add", Array((!$v["data"]->r("add")?"disabled":false))), false, false, Array("class" => "cms"));
     }
    }
   }

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("config_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
   $r.=$this->table("modules");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_cms()
  {
   global $_session;

   if($this->reload)
   {
    $this->e->add($this->cms->e);

    //set title after viewItem got data
    $this->backend->setTitle($this->icon("config_dim"),
     $this->link($this->language->t("Configuration"), Array("configuration/")),
     $this->language->t("Updated"));

    return $this->reload;
   }

   $_session->frontend();
   $r=$this->cms->form();
   $this->e->add($this->cms->e);
   $_session->backend();

   //set title after viewItem got data
   $this->backend->setTitle($this->icon("config_dim"),
    $this->link($this->language->t("Configuration"), Array("configuration/")),
    $this->link($this->cms->data->l->t($this->cms->data->title), $this->url("configuration/".$this->cms->data->module."/")));

   return $r;
  }

 }
?>
