<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/database.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");

 class Backend_Database extends Backend_Modules
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);
  }

  function main($module=false, $section=false)
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->search("modules"); //inline search all modules

   //search
   if($module)
   {
    foreach(Backend_Modules::modules(true) as $k => $v) //filter modules list
    {
     if($module==$k)
     {
      $this->modules[$module]=$v;
      break;
     }
    }
   }
   else if($search=request("search"))
   {
    foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
     if(stristr($module, $search)||stristr($v["title"], $search))
      $this->modules[$module]=$v;

    if(sizeof($this->modules)==1) //only one module
     Headers::redirectS($this->url("database/".key($this->modules)."/"));
   }
   else
    $this->modules=Backend_Modules::modules(true);


   if(!$_config->userRoot())
    $this->e->add("Database updates only permitted for root user", "warning");

   if(!($tables=request("tables")))
    $tables=Array();
   else
   {
    //set page as CMS in history
    $hs=new History();
    if(!$hs->cms())
     $this->e->add($hs->e);
   }

   $modules=array_keys($this->modules);
   for($c=0;$c<sizeof($modules);$c++)
   {
    $module=$modules[$c];
    $this->modules[$module]["updated"]=false;

    if(!$_config->userRoot()||$module=="empty"||strpos($module, "pluginInfo")!==false) //skip for non root user or empty and pluginInfo modules
    {
     $this->modules[$module]["query"]="";
     $this->modules[$module]["action"]=false;
     continue;
    }

    if(isset($this->modules[$module]["data"]))
    {
     unset($d);
     $d=&$this->modules[$module]["data"];
    }
    else //normal module
     $d=Data::cache($module, "database", "create", Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true));

    foreach(array_keys($d->reference) as $ref)
    {
     if($d->reference[$ref]["type"]=="*:*")
     {
      unset($dr);
      $dr=$d->getReferenceData($ref);
      $dr->setAction("database");
      $dr->setExtra("create");

      //add *:* to parse list
      $modules[]=$dr->module;

      //add *:* module info
      $this->modules[$dr->module]=$this->modules[$module];
      $this->modules[$dr->module]["title"]=$dr->title;
      $this->modules[$dr->module]["enabled"]=$dr->tableExists();
      $this->modules[$dr->module]["data"]=&$dr;
     }
    }

    if($this->modules[$module]["enabled"])
     $d->setExtra("alter");

    if(request("s")&&in_array($module, $tables)) //create/update table
    {
     if($d->submit())
     {
      $this->modules[$module]["updated"]=true;
      $_session->set("tableExists.".$_session->get("project").".".$d->db.".".$d->module); //clear table cache

      if($d->extra=="create"&&$d->tableExists()) //correctly created
      {
       $this->modules[$module]["enabled"]=true;
       $d->setExtra("alter");
      }
     }

     $this->e->add($d->e);
     $d->e->clear();
    }

    if(!$d->get())
    {
     $this->modules[$module]["query"]="";
     $this->modules[$module]["action"]=$d->e->show();

     $_session->set("tableExists.".$_session->get("project").".".$d->db.".".$d->module); //clear table cache
    }
    else
    {
     //filter errors
     $this->modules[$module]["query"]="";
     $this->modules[$module]["action"]=Array();
     for($x=0;$x<$d->e->getErrors();$x++)
     {
      if(strpos($d->e->getError($x), "No changes")===0) //no changes
       continue; //skip

      else if(strpos($d->e->getError($x), "Query")===0) //query
       $this->modules[$module]["query"]=preg_replace("/Query for table:.+?:/", "", $d->e->getError($x));

      else
       $this->modules[$module]["action"][]=$d->e->getError($x);
     }
    }
   }

   ksort($this->modules); //resort for new *:* modules

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("database_dim"),
    $this->link($this->language->t("Database"), Array("database/")));

   //default modules
   $default=Array("categories", "users", "config", "userRights", "groupRights", "usersAdditionalGroups");
   if($_config->get("logging"))
    $default[]="log";

   $this->tableRow("modules", true);
   $this->tableColumn("modules", "", false, false, Array("class" => "checkall"));
   $this->tableColumn("modules", $this->language->t("Module"), false, false, Array("class" => "module"));
   $this->tableColumn("modules", $this->language->t("Action"), false, false, Array("class" => "action"));

   //modules list
   $enabled=0;
   foreach($this->modules as $module => $v)
   {
    if(!$v["enabled"])
     $this->tableRow("modules", false, Array("class" => "disabled"));
    else
    {
     $this->tableRow("modules", false, ($v["updated"]?Array("selected"):false));
     $enabled++;
    }

    if($v["query"])
     $this->tableColumn("modules", $this->tooltip($this->form->checkbox((!$v["enabled"]&&in_array($module, $default)?true:false), false, "tables[]", $module), $v["query"]));
    else
     $this->tableColumn("modules", "");

    $this->tableColumn("modules", $v["title"], false, false, Array("class" => "title"));

    if(!$v["action"])
     $this->tableColumn("modules", "");
    else if(is_array($v["action"]))
     $this->tableColumn("modules", "<ul class=\"image\"><li>".implode("</li><li>", $v["action"])."</li></ul>\n");
    else
     $this->tableColumn("modules", $v["action"]);
   }

   $this->tableRow("modules", false, Array("class" => "total"));
   $this->tableColumn("modules", "");

   if(!$this->modules)
    $this->tableColumn("modules", $this->language->t("No modules"), false, false, Array("colspan" => 2));
   else
   {
    $this->tableColumn("modules", $this->language->t("Total"));
    $this->tableColumn("modules", $enabled." / ".sizeof($this->modules), false, false, Array("class" => "nr"));
   }


   $r="<div class=\"twothird\">\n";
   $r.=$this->table("modules", "database");
   $r.="</div>\n";


   $r.="<div class=\"third\">\n";

   //database submit
   $r.="<h3>".$this->icon("database_dim", "small")." ".$this->language->t("Database")."</h3>\n";
   $r.=$this->field($this->language->t("Tables"), $this->form->input("submit", "s", $this->language->t("Update"), false, false, ($_config->userRoot()?false:"disabled")));

   //actions
   if(sizeof($this->modules)==1&&$enabled) //only one enabled module
   {
    $d=Data::cache(array_keys($this->modules)[0]);
    $this->actionsMenu($d);
   }
   else
    $this->actionsMenu();

   //database statistics
   if($_config->userRoot())
   {
    $d=Data::cache("empty", "database", "info");
    foreach(Backend_Modules::modules(true) as $module => $v)
    {
     if(!$v["enabled"])
      continue;

     $d->module=$module;

     if($d->get())
      $stats[$module]=$d->getValue("info_rows");
    }

    arsort($stats);

    foreach($stats as $module => $value)
     $this->statsEntry("database", $this->link(Mutilate::stripS(ucfirst($module), 11), Array("database/".$module."/"), $this->language->t("Show ".$module." table")), $value);
   }


   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   if($_config->userRoot())
   {
    $r.="<h3>".$this->icon("database_dim", "small")." ".$this->language->t("Database statistics")."</h3>\n";
    $r.=$this->stats("database");
   }

   $r.="</div>\n";

   //make full page part of the form
   $r=$this->form($r);

   $_session->backend();

   return $r;
  }

 }
?>
