<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/modules.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/data.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class Backend_Modules extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->module=false;
   $this->modules=Array();

   $this->memoryCacheData=false;
  }

  function main($module=false, $section=false)
  {
   global $_session, $_config;

   //section
   if(!parent::main($section))
    return false;

   if($this->section=="quick")
    return true;

   $_session->frontend();

   //check module
   if($module&&$module!="_")
   {
    if(!in_array($module, JAS::modules())) //invalid module
     $this->e->add("Module: \"".$module."\" not found", "error");
    else if($this->section=="database")
    {
     foreach($this->modules(true) as $k => $v) //filter modules list
     {
      if($module==$k)
      {
       $this->modules[$module]=$v;
       break;
      }
     }
    }
    else //valid module
    {
     //modify quick categories list url
     if($quick=$this->backend->component("backend/categories/quick")) //categories list found
      $quick->path=false; //current page

     $this->module=$module;
     $this->section="list";

     $d=Data::cache($this->module);

     //set inline search options
     if($titles=$d->check("properties", "title"))
      $this->backend->search("data", $this->module, "title", $titles[0]);

     //ordering
     if($d->action=="order")
     {
      if(!request("catnr")&&($category=$d->check("reference", "categories"))) //order with category
       $d->field[$category[0]]["properties"].=";ignore"; //add fake ignore category ordering property

      //set page as CMS in history
      $hs=new History();
      if(!$hs->cms())
       $this->e->add($hs->e);

      $d->submit();
      $this->e->add($d->e);
     }

     //search redirect
     if(request("search"))
     {
      $d->setAction("search");
      if($d->fulltextIndex) //full text index exists
       $d->setExtra("index");
      else //normal search
       $d->setExtra("full");

      $d->setShowFields();
      $d->setValue("show_total", 2);
      $d->setRequestValue("search");

      if($d->get()&&$d->vmax==1) //only 1 entry
       Headers::redirectS($this->url("data/".$d->module."/".$d->getValue($d->getID())."/"));
     }
    }
   }
   else //no module set
   {
    $this->backend->search("modules"); //inline search all modules

    //search
    if($search=request("search"))
    {
     foreach($this->modules(true) as $module => $v) //filter modules list
      if(stristr($module, $search)||stristr($v["title"], $search))
       $this->modules[$module]=$v;

     if(!$this->section) //default section
     {
      if(sizeof($this->modules)==1) //only one module
      {
       if(current($this->modules)["enabled"]) //module is enabled
        Headers::redirectS($this->url("modules/".key($this->modules)."/"));
       else //table doesn't exist
        Headers::redirectS($this->url("database/".key($this->modules)."/"));
      }
     }
    }
    else
     $this->modules=$this->modules(true);
   }

   if(!$this->section)//default section
   {
    //clear module cache
    if($_config->get("cache.memory")&&$_config->get("cache.memoryCacheData"))
    {
     $this->memoryCacheData=true;
     if($_config->userRoot()&&request("clearMemoryCacheData"))
     {
      $cache=new Cache("memory");
      foreach($cache->search("DATA::CACHE(".$_session->get("project").",") as $key)
       $cache->remove($key);

      $this->e->add("Data objects memory cache cleared", "note");
     }
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("module_dim"),
    $this->link($this->language->t("Modules"), Array("modules/")));

   $this->tableRow("modules", true);
   $this->tableColumn("modules", $this->language->t("Module"), false, false, Array("class" => "module"));
   $this->tableColumn("modules", $this->language->t("Type"), false, false, Array("class" => "type"));

   //modules list
   $enabled=0;
   foreach($this->modules as $module => $v)
   {
    if(!$v["enabled"])
    {
     $this->tableRow("modules", false, Array("class" => "disabled"));
     $this->tableColumn("modules", $v["title"], $this->url("database/".$module."/"), $this->language->t("Create table: ".$v["title"]));
    }
    else
    {
     $enabled++;
     $this->tableRow("modules");
     $this->tableColumn("modules", $v["title"], $this->url("modules/".$module."/"), $v["title"]);
    }

    if($v["type"]=="plugin")
     $this->tableColumn("modules", $v["pluginfo"]["title"], $this->url("plugins/".$v["plugin"]."/"), $v["plugin"]." v".$v["pluginfo"]["version"]." ".$v["pluginfo"]["status"].($v["pluginfo"]["description"]?" - ".$v["pluginfo"]["description"]:""));
    else
     $this->tableColumn("modules", $this->language->t(ucfirst($v["type"])));
   }

   $this->tableRow("modules", false, Array("class" => "total"));

   if(!$this->modules)
    $this->tableColumn("modules", $this->language->t("No modules"), false, false, Array("colspan" => 2));
   else
   {
    $this->tableColumn("modules", $this->language->t("Total"));
    $this->tableColumn("modules", $enabled." / ".sizeof($this->modules), false, false, Array("class" => "nr"));
   }


   $r="<div class=\"twothird\">\n";
   $r.=$this->table("modules");
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   if($this->memoryCacheData)
    $this->menuItem("actions", $this->language->t("Clear cache"), $this->url(false, Array("clearMemoryCacheData" => "true")), $this->language->t("Clear data objects cache"), "cache", Array("class" => "value", ($_config->userRoot()?false:"disabled")));

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_list()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");

   $this->backend->setTitle($this->icon("module_dim"),
    $this->link($this->language->t("Modules"), Array("modules/")),
    $this->link($d->l->t($d->title), Array("modules/".$d->module."/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
   {
    $d->setExtra("list");
    $d->setValue($field, $catnr);
   }

   $r="<div class=\"twothird\">\n";
   $r.=$this->viewList($d, Array("get" => 0));
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   //table info
   $d->clearValues();
   $d->setAction("database");
   $d->setExtra("info");

   $d->addMutilation("info_updated", "timeAgo");
   if(isset($d->field["info_created"]))
    $d->addMutilation("info_created", "timeAgo");

   $r.="<div class=\"third\">\n";
   if(!$d->get())
    $this->e->add($d->e);
   else if($d->next())
   {
    $this->tableRow("info", true);
    $this->tableColumn("info", "", false, false, Array("colspan" => 2));

    foreach($d->check("use", "info") as $field)
    {
     $this->tableRow("info");
     $this->tableColumn("info", $d->title($field, true), false, false, Array("class" => "key"));
     $this->tableColumn("info", $d->v[$field], false, false, Array("class" => "value ".$d->field[$field]["type"]." ".$field));
    }

    $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
    $r.=$this->table("info");
   }

   //actions
   $this->actionsMenu($d);

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_quick()
  {
   global $_session;

   $this->menuItem("modules", $this->language->t("Modules"), $this->url("modules/"), $this->language->t("Modules"), "module");

   //list modules
   $_session->frontend();

   if(request("p", 0)=="modules")
    $current=request("p", 1);
   else
    $current=false;

   foreach($this->modules() as $module => $v)
    if($v["enabled"])
     $this->menuItem("modules", $v["title"], $this->url("modules/".$module."/"), $v["title"], false, ($module==$current?Array("selected"):false));

   $_session->backend();

   $r=$this->menu("modules");

   return $r;
  }


  /*
   * Support
   */

  static function modules($plugins=false)
  {
   global $_path, $_session, $_Backend_Module_List, $backend;

   $project=$_session->get("project");

   if($plugins)
    $project.="_all"; //separate suffix for all modules (including plugins)

   if(isset($_Backend_Module_List[$project]))
    return $_Backend_Module_List[$project];

   if($plugins)
    $plugins=Backend_Plugins::plugins();

   //read module status
   $d=Data::cache("empty");
   $language=new Language();
   $_Backend_Module_List[$project]=Array();
   foreach(JAS::modules(false, ($plugins?false:"_none")) as $module) //no plugin modules
   {
    $language->sets=Array();
    $language->read("module_".$module);

    $values=Array();
    if(preg_match(REGEXP_DATA_PLUGIN_MODULE, $module, $m)) //plugin module
    {
     $values["type"]="plugin";
     $values["plugin"]=$m[1];

     //find module info
     if(FS::existsS($_path["project"]["plugins"].$m[1]."/modules/".$m[2].".php"))
      $values["file"]=$_path["project"]["plugins"].$m[1]."/modules/".$m[2].".php";
     else if(FS::existsS($_path["plugins"].$m[1]."/modules/".$m[2].".php"))
      $values["file"]=$_path["plugins"].$m[1]."/modules/".$m[2].".php";
     else
     {
      $backend->e->add("No module info for module: \"".$module."\"", "plugin");
      continue;
     }
    }
    else //normal module
    {
     //find module info
     if(FS::existsS($_path["project"]["modules"].$module.".php"))
     {
      $values["type"]="project";
      $values["file"]=$_path["project"]["modules"].$module.".php";
     }
     else if(FS::existsS($_path["modules"].$module.".php"))
     {
      $values["type"]="system";
      $values["file"]=$_path["modules"].$module.".php";
     }
     else
     {
      $backend->e->add("No module info for module: \"".$module."\"", "module");
      continue;
     }
    }

    //get title+deleted field from module file directly
    if($content=FS::readS($values["file"]))
    {
     if($values["type"]!="system"&&preg_match("/extends System_Module_([^\s]+)/", $content, $m)) //extended system module
      $content.=FS::readS($_path["modules"].$m[1].".php"); //also read system module file content

     if(preg_match("/this->data->title=[^\"]*\"([^\"]+)\"/", $content, $m))
      $values["title"]=$m[1];

     if(preg_match_all("/this->extend\(\"([^\"]+)\"\)/", $content, $m))
     {
      $values["extend"]=Array();

      foreach($m[1] as $extend)
       $values["extend"][]=$extend;
     }

     if(preg_match("/this->data->defaultFields\(\)/", $content)) //catnr+owner
     {
      $values["category"]=true;
      $values["owner"]=true;
     }
     else //custom catnr+owner fields
     {
      if(preg_match("/this->data->defaultFields\(.+rights/", $content))
       $values["category"]=true;
      else if(preg_match("/this->data->addFields\(.+rightsReference/", $content))
       $values["category"]=true;

      if(preg_match("/this->data->defaultFields\(.+owner/", $content))
       $values["owner"]=true;
      else if(preg_match("/this->data->addFields\(.+owner/", $content))
       $values["owner"]=true;
     }

     if(preg_match("/this->data->addField\(.+deleted/", $content))
      $values["undelete"]=true;
    }

    $d->module=$module;

    $_Backend_Module_List[$project][$module]=Array(
     "enabled" => $d->tableExists(),
     "title" => (isset($values["title"])?$language->t($values["title"]):$module),
     "type" => (isset($values["type"])?$values["type"]:false),
     "extend" => (isset($values["extend"])?$values["extend"]:false),
     "owner" => (isset($values["owner"])?$values["owner"]:false),
     "category" => (isset($values["category"])?$values["category"]:false),
     "undelete" => (isset($values["undelete"])?$values["undelete"]:false),
     "plugin" => (isset($values["plugin"])?$values["plugin"]:false),
     "pluginfo" => (isset($values["plugin"])&&isset($plugins[$values["plugin"]])?$plugins[$values["plugin"]]:false),
    );
   }

   //get properties from extended modules
   foreach($_Backend_Module_List[$project] as $module => $values) //run through all extended modules
    if($values["extend"])
     foreach($values["extend"] as $extend)
      if(isset($_Backend_Module_List[$project][$extend])) //copy (set) properties from extended module
       foreach(Array("owner", "category", "undelete") as $property)
        if($_Backend_Module_List[$project][$extend][$property])
         $_Backend_Module_List[$project][$module][$property]=$_Backend_Module_List[$project][$extend][$property];

   return $_Backend_Module_List[$project];
  }


  function actionsMenu($d=false)
  {
   global $_config;

   $class=strtolower(preg_replace("/Backend_/", "", get_class($this)));

   if($d)
   {
    $this->language->read("module_".$d->module);

    if($cms=$this->cmsOptions($d, "add", false, (request("catnr")&&$d->getRightsReference()?Array($d->getRightsReference() => strip_tags(request("catnr"))):false)))
     $this->menuItem("actions", $cms["content"], $cms["action"], $cms["title"], "add", ($cms["action"]?false:Array("disabled")));

    if($d->getRightsReference())
     $this->menuItem("actions", $this->language->t("Move"), $this->url("move/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Move ".$d->title." items"), "move", Array(($class=="move"?"selected":false)));

    $this->menuItem("actions", $this->language->t("Delete"), $this->url("delete/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Delete ".$d->title." items"), "delete", Array(($class=="delete"?"selected":false)));

    if($d->validAction("undelete"))
     $this->menuItem("actions", $this->language->t("Undelete"), $this->url("undelete/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Undelete ".$d->title." items"), "undelete", Array(($class=="undelete"?"selected":false)));

    if($d->check("properties", "owner"))
     $this->menuItem("actions", $this->language->t("Owners"), $this->url("owner/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Change ".$d->title." owners"), "owner", Array(($class=="owner"?"selected":false)));

    if($d->module=="categories")
     $this->menuItem("actions", $this->language->t("Rights"), $this->url("rights/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Change rights"), "rights", Array(($class=="rights"?"selected":false)));

    $this->menuItem("actions", $this->language->t("Module"), $this->url("modules/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Show ".$d->title." items"), "module", Array(($class=="modules"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Export"), $this->url("export/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Export ".$d->title." items"), "export", Array(($class=="export"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Import"), $this->url("import/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Import ".$d->title." items"), "import", Array(($class=="import"?"selected":false)));

    if($d->getRightsReference())
     $this->menuItem("actions", $this->language->t("Lost & Found"), $this->url("lostandfound/".$d->module."/", (request("catnr")?Array("catnr" => strip_tags(request("catnr"))):false)), $this->language->t("Recover lost and found ".$d->title." items"), "lostandfound", Array(($class=="lostandfound"?"selected":false), (!$_config->userRoot()?"disabled":false)));

    $this->menuItem("actions", $this->language->t("Database"), $this->url("database/".$d->module."/"), $this->language->t("Database"), "database", Array(($class=="database"?"selected":false), (!$_config->userRoot()?"disabled":false)));
   }
   else
   {
    $this->menuItem("actions", $this->language->t("Move"), $this->url("move/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Move items"), "move", Array(($class=="move"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Delete"), $this->url("delete/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Delete items"), "delete", Array(($class=="delete"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Undelete"), $this->url("undelete/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Undelete items"), "undelete", Array(($class=="undelete"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Owners"), $this->url("owner/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Change owners"), "owner", Array(($class=="owner"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Rights"), $this->url("rights/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Change rights"), "rights", Array(($class=="rights"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Modules"), $this->url("modules/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Show modules"), "module", Array(($class=="modules"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Export"), $this->url("export/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Export items"), "export", Array(($class=="export"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Import"), $this->url("import/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Import items"), "import", Array(($class=="import"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Lost & Found"), $this->url("lostandfound/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Show lost and found"), "lostandfound", Array(($class=="lostandfound"?"selected":false), (!$_config->userRoot()?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Database"), $this->url("database/", (request("search")?Array("search" => strip_tags(request("search"))):false)), $this->language->t("Database"), "database", Array(($class=="database"?"selected":false), (!$_config->userRoot()?"disabled":false)));
   }

   return true;
  }
 }
?>
