<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/online.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/plugins.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."jas.php");

 class Backend_Online extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->jasRootUser=false;

   $this->nr=false;
   $this->name=false;
   $this->repository=false;
   $this->repositories=Array();

   $this->upgrade=Array();
  }

  static function updates(&$component)
  {
   global $_session, $_config;

   $_session->frontend();

   if($_session->get("online.time")&&(time()-$_session->get("online.time"))<3600) //cached less than 1 hour ago
   {
    $updates=$_session->get("online.updates"); //use cached updates
    $_session->backend();
    return $updates;
   }

   $updates="";

   if(!($repositories=$_config->get("repositories")))
    $repositories=Array();

   $language=new Language();
   $language->read("jas");

   //get latest release
   $d=Data::cache("onlineReleases", "view", "all", Array("skipCheckTables" => true, "format" => "sql"));
   $d->setShowFields("title", "version", "tag");
   $d->sortOrder("title", "version-", "tag-");

   $local=JAS::versionS("tag");
   $local=JAS::versionS().($local?$local:"_");

   foreach($repositories as $code => $info)
   {
    if(!in_array("Releases", $info["content"]))
     continue;

    $d->storageSwitch($info["apiurl"]);
    $d->clearValues();

    $d->setValue("aggregation_field", "title");

    if(!$d->get()||!$d->next())
     continue;

    if(Backend_Online::_compareVersion($d->v["version"].($d->v["tag"]?$d->v["tag"]:"_"), $local)=="newer")
    {
     $updates.=$component->link($d->v("title"), "online/releases/", $language->t($d->v("title")." is available"));
     break;
    }
   }

   //get latest plugins
   $d=Data::cache("onlinePlugins", "view", "all", Array("skipCheckTables" => true, "format" => "sql"));
   $d->setShowFields("plugin", "title", "version");
   $d->sortOrder("title", "version-");

   $plugins=Array();
   foreach($repositories as $code => $info)
   {
    if(!in_array("Plugins", $info["content"]))
     continue;

    $d->storageSwitch($info["apiurl"]);
    $d->clearValues();

    $d->setValue("aggregation_field", "title");

    $c=0;
    $locals=Array();
    foreach(Backend_Plugins::plugins() as $plugin => $v)
    {
     $locals[$plugin]=$v["version"];
     $d->setValue("plugin", $plugin, $c);
     $d->setOperant("plugin", "OR", $c);
     $c++;
    }

    if(!$d->get())
     continue;

    while($d->next())
     if(Backend_Online::_compareVersion($d->v["version"], $locals[$d->v["plugin"]])=="newer")
      $plugins[$d->v["plugin"]]=$d->v;
   }

   if($plugins)
    $updates.=($updates?" | ":"").$component->link($language->t(sizeof($plugins)." plugins"), "online/plugins/", $language->t(sizeof($plugins)." new plugins are available").":\n- ".implode("\n- ", array_keys($plugins)));

   //store cached updates
   $cache=Array(
    "updates" => $updates,
    "time" => time()
   );
   $_session->set("online", $cache);

   $_session->backend();

   return $updates;
  }

  function main($section=false, $repository=false, $name=false, $download=false)
  {
   global $_config, $_session, $_path;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status

   //section
   if(!parent::main($section))
    return false;

   if(!$this->section)
    $this->section="updates";

   $_session->frontend();

   $this->backend->search("plugins"); //inline search all plugins
   $this->backend->searchURL($this->url("online/plugins/")); //search page

   $this->language->read("module_onlineReleases");
   $this->language->read("module_onlinePlugins");

   //get release repositories
   $this->repositories=Array();
   foreach($_config->get("repositories") as $code => $info)
    $this->repositories[$code]=$info;

   $_session->backend();

   if(!$this->repositories)
   {
    $this->e->add("No release repositories setup", "error");
    return false;
   }

   //repository filter
   if($repository&&$repository!=" "&&$repository!="+")
   {
    if(!isset($this->repositories[$repository]))
    {
     $this->e->add("Unknown repository: \"".$repository."\"", "error");
     return false;
    }

    if($this->section=="releases"||$this->section=="release"||$this->section=="upgrade")
    {
     if(!in_array("Releases", $this->repositories[$repository]["content"]))
     {
      $this->e->add("Repository: \"".$repository."\" doesn't support releases", "error");
      return false;
     }
    }
    else if($this->section=="plugins"||$this->section=="plugin")
    {
     if(!in_array("Plugins", $this->repositories[$repository]["content"]))
     {
      $this->e->add("Repository: \"".$repository."\" doesn't support plugins", "error");
      return false;
     }
    }

    $this->repository=$repository;
   }

   //determine name / nr
   if($name)
   {
    if(preg_match("/^\d+$/", $name))
     $this->nr=(int)$name;
    else
     $this->name=strip_tags($name);
   }

   //download/upgrade release
   if($this->nr&&($this->section=="upgrade"||($this->section=="release"&&$download)))
   {
    if(!$this->jasRootUser)
     $this->e->add("Release can only be ".($this->section=="upgrade"?"upgraded":"downloaded")." by the root user", "error");

    $releaseCache=$_path["project"]["cache"]."releases/";
    if(!FS::existsS($releaseCache)&&!FS::mkdirS($releaseCache, true))
    {
     $this->e->add("Can't create release cache directory: \"".$releaseCache."\"", "error");
     return false;
    }

    //get release info
    $d=Data::cache("onlineReleases", "view", "item", Array("skipCheckTables" => true, "db" => $this->repositories[$this->repository]["apiurl"]));
    $d->setShowFields("file", "fileSize", "fileMD5");

    $d->clearValues();
    $d->setValue("nr", $this->nr);

    if(!$d->get()||!$d->next())
    {
     $this->e->add($d->e);
     return false;
    }

    $filename=$releaseCache.$d->v["file"];
    $filesize=$d->v["fileSize"];
    $filemd5=$d->v["fileMD5"];

    if(FS::existsS($filename))
    {
     if(FS::sizeS($filename)!=$filesize)
      $this->e->add("Release file: \"".$filename."\" doesn't match original size", "warning");
     else if(md5_file($filename)!=$filemd5)
      $this->e->add("Release file: \"".$filename."\" doesn't match MD5", "error");
     else if($download)
      $this->e->add("Release file: \"".$filename."\" already downloaded", "note");

     $download=false;
    }
    else if($this->section=="upgrade") //download file for upgrade
     $download=true;

    //download release if required
    if($download)
    {
     if(!$this->jasRootUser) //must be root to download
      return false;

     $d->setShowFields("fileData");

     $d->clearValues();
     $d->setValue("nr", $this->nr);

     if(!$d->get()||!$d->next())
     {
      $this->e->add($d->e);
      return false;
     }

     if(!FS::writeS($filename, $d->v["fileData"]))
      $this->e->add("Unable to write release to: \"".$filename."\"", "error");
     else if(FS::sizeS($filename)!=$filesize)
      $this->e->add("Release file: \"".$filename."\" doesn't match original size", "error");
     else if(md5_file($filename)!=$filemd5)
      $this->e->add("Release file: \"".$filename."\" doesn't match MD5", "error");
     else
      $this->e->add("Release downloaded to: \"".$filename."\"", "note");
    }

    //upgrade release
    if($this->section=="upgrade")
    {
     $files=false;
     $jas=new JAS();

     if(request("confirm"))
     {
      if(!request("upgrade"))
       $this->e->add("No release items selected to upgrade", "warning");
     }
     else if(request("s")&&request("upgrade")) //fails if not root
     {
      if(!($files=$jas->upgrade($filename, in_array("new", request("upgrade")), in_array("changed", request("upgrade")), in_array("delete", request("upgrade")))))
       $this->e->add($jas->e);
     }

     if(!$files) //files not yet gathered
     {
      if(!($files=$jas->upgrade($filename)))
      {
       $this->e->add($jas->e);
       return false;
      }
     }

     //generate stats
     $this->upgrade["stats"]=Array();
     foreach($files as $file => $info)
     {
      if($info["type"]=="directory")
       continue;

      $status=$info["status"];
      if(!isset($this->upgrade["stats"][$status]))
       $this->upgrade["stats"][$status]=Array(
        "name" => $this->language->t(ucfirst($files[$file]["status"])),
        "size" => 0,
        "files" => 0,
        "list" => Array(),
       );

      $this->upgrade["stats"][$status]["size"]+=$info["size"];
      $this->upgrade["stats"][$status]["files"]++;

      if(($tmp=strrpos($file, "/"))!==false)
      {
       $directory=substr($file, 0, $tmp+1);
       $file=substr($file, $tmp+1);
      }
      else
       $directory="";

      $this->upgrade["stats"][$status]["list"][$directory][$file]=$info["size"];
     }

     //totals
     $this->upgrade["total"]=Array(
      "size" => 0,
      "files" => 0,
     );
     foreach($this->upgrade["stats"] as $info)
     {
      $this->upgrade["total"]["size"]+=$info["size"];
      $this->upgrade["total"]["files"]+=$info["files"];
     }

     //sort results
     ksort($this->upgrade["stats"]);
     foreach(array_keys($this->upgrade["stats"]) as $status)
     {
      ksort($this->upgrade["stats"][$status]["list"], SORT_NATURAL);

      foreach(array_keys($this->upgrade["stats"][$status]["list"]) as $directory)
       ksort($this->upgrade["stats"][$status]["list"][$directory], SORT_NATURAL|SORT_FLAG_CASE);
     }
    }
   }
   else if($this->nr&&($this->section=="import"||($this->section=="plugin"&&$download))) //download/import plugin
   {
    $_session->frontend();

    if(!$_config->userRoot())
     $this->e->add("Plugin can only be ".($this->section=="import"?"imported":"downloaded")." by the root user", "error");

    $pluginCache=$_path["project"]["cache"]."plugins/";
    if(!FS::existsS($pluginCache)&&!FS::mkdirS($pluginCache, true))
    {
     $this->e->add("Can't create plugin cache directory: \"".$pluginCache."\"", "error");
     return false;
    }

    //get plugin info
    $d=Data::cache("onlinePlugins", "view", "item", Array("skipCheckTables" => true, "db" => $this->repositories[$this->repository]["apiurl"]));
    $d->setShowFields("plugin", "file", "fileSize", "fileMD5");

    $d->clearValues();
    $d->setValue("nr", $this->nr);

    if(!$d->get()||!$d->next())
    {
     $this->e->add($d->e);
     $_session->backend();
     return false;
    }

    $plugin=$d->v["plugin"];
    $filename=$pluginCache.$d->v["file"];
    $filesize=$d->v["fileSize"];
    $filemd5=$d->v["fileMD5"];

    if(FS::existsS($filename))
    {
     if(FS::sizeS($filename)!=$filesize)
      $this->e->add("Plugin file: \"".$filename."\" doesn't match original size", "warning");
     else if($filemd5&&md5_file($filename)!=$filemd5)
      $this->e->add("Plugin file: \"".$filename."\" doesn't match MD5", "error");
     else if($download)
      $this->e->add("Plugin file: \"".$filename."\" already downloaded", "note");

     $download=false;
    }
    else if($this->section=="import") //download file for import
     $download=true;

    //download plugin if required
    if($download)
    {
     if(!$_config->userRoot()) //must be root to download
     {
      $_session->backend();
      return false;
     }

     $d->setShowFields("fileData");

     $d->clearValues();
     $d->setValue("nr", $this->nr);

     if(!$d->get()||!$d->next())
     {
      $this->e->add($d->e);
      $_session->backend();
      return false;
     }

     if(!FS::writeS($filename, $d->v["fileData"]))
      $this->e->add("Unable to write plugin to: \"".$filename."\"", "error");
     else if(FS::sizeS($filename)!=$filesize)
      $this->e->add("Plugin file: \"".$filename."\" doesn't match original size", "error");
     else if($filemd5&&md5_file($filename)!=$filemd5)
      $this->e->add("Plugin file: \"".$filename."\" doesn't match MD5", "error");
     else
      $this->e->add("Plugin downloaded to: \"".$filename."\"", "note");
    }

    //import plugin
    if($this->section=="import"&&$_config->userRoot())
    {
     if(request("s"))
     {
      $p=new Plugin(false);
      $p->e->clear();

      if(!$p->import($filename)) //import failed
      {
       if(in_array($plugin, JAS::plugins())&&FS::existsS($_path["project"]["plugins"].".tmp/")) //plugin already exists
       {
        $p->e->clear();
        $p->importOverwrite($plugin); //force overwrite
       }
      }

      $this->e->add($p->e); //show import errors/success
     }
    }

    $_session->backend();
   }

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t("Updates"), Array("online/updates/"))
   );

   $r="<div class=\"twothird\">\n";

   if($this->repositories)
   {
    $r.=$this->_listReleases();
    $r.="<br/>\n";
    $r.=$this->_listPlugins();
   }
   else
    $r.="&nbsp;";

   $r.="</div>\n";

   //actions menu
   $this->menuItem("actions", $this->language->t("Releases"), Array("online/releases/"), $this->language->t("View releases"), "updates");
   $this->menuItem("actions", $this->language->t("Plugins"), Array("online/plugins/"), $this->language->t("View plugins"), "updates");

   //repositories menu
   foreach($this->repositories as $code => $info)
    $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "online", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";


   $_session->backend();

   return $r;
  }


  /*
   * Releases
   */

  function output_releases()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t("Releases"), Array("online/releases/")),
    "JAS v".JAS::versionS()." ".JAS::versionS("tag")
   );

   $r="<div class=\"twothird\">\n";

   if($this->repositories)
    $r.=$this->_listReleases();
   else
    $r.="&nbsp;";

   $r.="</div>\n";

   //actions menu
   $this->menuItem("actions", $this->language->t("Updates"), Array("online/updates/"), $this->language->t("Check for updates"), "updates");
   $this->menuItem("actions", $this->language->t("Plugins"), Array("online/plugins/"), $this->language->t("View plugins"), "updates");

   //repositories menu
   foreach($this->repositories as $code => $info)
    if(in_array("Releases", $info["content"]))
     $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "online", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";


   $_session->backend();

   return $r;
  }

  function output_release()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t("Releases"), Array("online/releases/"))
   );

   $r="<div class=\"twothird\">\n";

   if($this->repository&&$this->nr)
   {
    //get remote release information
    $d=Data::cache("onlineReleases", "view", "item", Array("skipCheckTables" => true, "db" => $this->repositories[$this->repository]["apiurl"]));
    $d->setShowFields("title", "version", "tag", "date", "changelog", "file", "fileSize");
    $d->addMutilation("changelog", "strip", 1000);
    $d->addMutilation("fileSize", "humanReadableFileSize", 1);

    $d->clearValues();
    $d->setValue("nr", $this->nr);

    if(!$d->get()||!$d->next())
    {
     $this->e->add($d->e);
     $r.="&nbsp;";
    }
    else
    {
     $this->backend->setTitle($this->icon("updates_dim"),
      $this->link($this->language->t("Online"), Array("online/")),
      $this->link($this->language->t("Releases"), Array("online/releases/")),
      $this->link($this->repository, Array("online/releases/".$this->repository."/")),
      $this->link($d->v("title"), Array("online/release/".$this->repository."/".$d->v["nr"]."/"))
     );

     //gather local release information
     $local=$this->_getLocalRelease();

     if(!($state=$this->_compareVersion($d->v["version"].($d->v["tag"]?$d->v["tag"]:"_"), $local["version"].$local["tag"])))
      $state="installed";

     if($state=="newer")
      $action="Upgrade";
     else if($state=="older")
      $action="Downgrade";
     else
      $action="Reinstall";

     $this->tableRow("release", true);
     $this->tableColumn("release", $this->icon("updates".($state!="installed"?"_".$state:"_dim")));
     $this->tableColumn("release", $this->language->t("Installed"));
     $this->tableColumn("release", $this->repositories[$this->repository]["name"]);

     foreach(Array("version", "tag", "date") as $field)
     {
      $this->tableRow("release");
      $this->tableColumn("release", $d->l->t($d->field[$field]["title"]), false, false, Array("class" => "key"));

      switch($field)
      {
       case "tag":
       {
        $tmp=($d->v[$field]?$d->v[$field]:"_"); //comparison workaround
        if($local[$field]<$tmp)
         $class="newer";
        else if($local[$field]>$tmp)
         $class="older";
        else
         $class="same";

        $local[$field]=($local[$field]!="_"?$local[$field]:"");

        $this->tableColumn("release", $local[$field], false, false, Array("class" => "value local ".$d->field[$field]["type"]));
        $this->tableColumn("release", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"].($class?" ".$class:"")));
        break;
       }
       case "date":
       {
        $tmp=DT::dateConvertS($d->v[$field], $d->field["date"]["format"][$d->dataformat]["dateformat"], "Y-m-d");
        if($local[$field]<$tmp)
         $class="newer";
        else if($local[$field]>$tmp)
         $class="older";
        else
         $class="same";

        $local[$field]=DT::dateConvertS($local[$field], "Y-m-d", $d->field["date"]["format"][$d->dataformat]["dateformat"]);

        $this->tableColumn("release", $local[$field], false, false, Array("class" => "value local ".$d->field[$field]["type"]));
        $this->tableColumn("release", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"].($class?" ".$class:"")));
        break;
       }
       default:
       {
        if($local[$field]<$d->v[$field])
         $class="newer";
        else if($local[$field]>$d->v[$field])
         $class="older";
        else
         $class="same";

        $this->tableColumn("release", $local[$field], false, false, Array("class" => "value local ".$d->field[$field]["type"]));
        $this->tableColumn("release", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"].($class?" ".$class:"")));
        break;
       }
      }
     }

     $r.=$this->table("release");

     $r.="<h2>".$this->icon("download_dim", "small")." ".$this->language->t("File")."</h2>";
     $r.="<div class=\"file\">";
     $r.=$this->link($d->v["file"], $this->repositories[$this->repository]["url"]."files/onlineReleases/file/".$d->v["nr"]."/", $this->language->t("Download ".$d->v["file"]));
     $r.=" (".$d->v["fileSize"].")";
     $r.="</div>";

     $r.="<h2>".$this->icon("changelog_dim", "small")." ".$this->language->t("Changelog")."</h2>";
     $r.="<div class=\"changelog\">".$d->v["changelog"]."</div>";


     //additional actions
     $this->menuItem("actions", $this->language->t($action), Array("online/upgrade/".$this->repository."/".$d->v["nr"]."/"), $this->language->t($action." ".$d->v["file"]." on server"), "updates".($state!="installed"?"_".$state:""), Array("class" => $state, (!$this->jasRootUser?"disabled":false)));
     $this->menuItem("actions", $this->language->t("Download"), $this->repositories[$this->repository]["url"]."files/onlineReleases/file/".$d->v["nr"]."/", $this->language->t("Download ".$d->v["file"]), "download");
     $this->menuItem("actions", $this->language->t("To server"), Array("online/release/".$this->repository."/".$d->v["nr"]."/download/"), $this->language->t("Download ".$d->v["file"]." to server"), "download", (!$this->jasRootUser?Array("disabled"):false));
    }
   }
   else
   {
    $r.="&nbsp;";
    $this->menuItem("actions", $this->language->t("Updates"), Array("online/updates/"), $this->language->t("Check for updates"), "updates");
   }

   $r.="</div>\n";

   //repositories menu
   foreach($this->repositories as $code => $info)
    if(in_array("Releases", $info["content"]))
     $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "updates", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions", "release");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";


   $_session->backend();

   return $r;
  }

  function output_upgrade()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t("Upgrade"), Array("online/releases/"))
   );

   $r="<div class=\"twothird\">\n";

   if($this->upgrade)
   {
    //gather local release information
    $local=$this->_getLocalRelease();

    //get remote release information
    $d=Data::cache("onlineReleases", "view", "item", Array("skipCheckTables" => true, "db" => $this->repositories[$this->repository]["apiurl"]));
    $d->setShowFields("title", "version", "tag", "date", "changelog", "file", "fileSize");
    $d->addMutilation("changelog", "strip", 1000);
    $d->addMutilation("fileSize", "humanReadableFileSize", 1);

    $d->clearValues();
    $d->setValue("nr", $this->nr);

    if(!$d->get()||!$d->next())
    {
     $this->e->add($d->e);
     $r.="&nbsp;";
    }
    else
    {
     $mutilate=new Mutilate();

     if(!($checked=request("upgrade")))
      $checked=Array();

     if(!($state=$this->_compareVersion($d->v["version"].($d->v["tag"]?$d->v["tag"]:"_"), $local["version"].$local["tag"])))
      $state="installed";

     if($state=="newer")
      $action="Upgrade";
     else if($state=="older")
      $action="Downgrade";
     else
      $action="Reinstall";

     $this->backend->setTitle($this->icon("updates".($state!="installed"?"_".$state:"_dim")),
      $this->link($this->language->t("Online"), Array("online/")),
      $this->link($this->language->t($action), Array("online/releases/")),
      $this->link($this->repository, Array("online/releases/".$this->repository."/")),
      $this->link($d->v("title"), Array("online/upgrade/".$this->repository."/".$d->v["nr"]."/"))
     );

     //render upgrade statistics table
     $this->tableRow("upgrade", true);
     $this->tableColumn("upgrade", "", false, false, Array("class" => "checkall"));
     $this->tableColumn("upgrade", $this->language->t("Status"));
     $this->tableColumn("upgrade", $this->language->t("Files"), false, false, Array("colspan" => 2));
     $this->tableColumn("upgrade", $this->language->t("Size"));
     $this->tableColumn("upgrade", "");

     foreach($this->upgrade["stats"] as $status => $info)
     {
      if($status==request("show")) //show files
       $rows=$info["files"]+1;
      else
       $rows=0;

      $this->tableRow("upgrade");

      if($status=="new"||$status=="changed"||$status=="delete")
       $this->tableColumn("upgrade", $this->form->checkbox((in_array($status, $checked)), false, "upgrade[]", $status), false, false, Array("rowspan" => $rows));
      else
       $this->tableColumn("upgrade", "", false, false, Array("rowspan" => $rows));

      $this->tableColumn("upgrade", $info["name"], false, false, Array("class" => "status ".$status, "rowspan" => $rows));
      $this->tableColumn("upgrade", "", false, false, Array("class" => "directory"));
      $this->tableColumn("upgrade", ($info["files"]?$info["files"]:""), false, false, Array("class" => "nr file"));
      $this->tableColumn("upgrade", ($info["size"]?$mutilate->humanReadableFileSize($info["size"]):""), false, false, Array("class" => "nr size"));

      if($status=="new"||$status=="changed"||$status=="delete")
       $this->tableColumn("upgrade", $this->icon("view"), Array("online/upgrade/".$this->repository."/".$d->v["nr"]."/", Array("show" => $status)), false, Array("class" => "cms"));
      else
       $this->tableColumn("upgrade", "");

      if($rows) //show files
      {
       foreach(array_keys($info["list"]) as $directory)
       {
        $this->tableRow("upgrade");
        $this->tableColumn("upgrade", $directory, false, false, Array("class" => "directory", "rowspan" => sizeof($info["list"][$directory])));

        $first=true;
        foreach($info["list"][$directory] as $file => $size)
        {
         if($first)
          $first=false;
         else
          $this->tableRow("upgrade");

         $this->tableColumn("upgrade", $file, false, false, Array("class" => "file hilite"));
         $this->tableColumn("upgrade", ($size?$mutilate->humanReadableFileSize($size):""), false, false, Array("class" => "nr size"));
        }
       }
      }
     }

     $this->tableRow("upgrade");
     $this->tableColumn("upgrade", $this->language->t("Total"), false, false, Array("colspan" => 2));
     $this->tableColumn("upgrade", $this->upgrade["total"]["files"], false, false, Array("class" => "nr", "colspan" => 2));
     $this->tableColumn("upgrade", ($this->upgrade["total"]["size"]?$mutilate->humanReadableFileSize($this->upgrade["total"]["size"]):""), false, false, Array("class" => "nr size"));
     $this->tableColumn("upgrade", "");

     $r.=$this->table("upgrade");


     //additional actions
     $this->menuItem("actions", $this->language->t("Download"), $this->repositories[$this->repository]["url"]."files/onlineReleases/file/".$d->v["nr"]."/", $this->language->t("Download ".$d->v["file"]), "download");
     $this->menuItem("actions", $this->language->t("To server"), Array("online/release/".$this->repository."/".$d->v["nr"]."/download/"), $this->language->t("Download ".$d->v["file"]." to server"), "download", (!$this->jasRootUser?Array("disabled"):false));
     $this->menuItem("actions", $this->language->t("Release"), Array("online/release/".$this->repository."/".$d->v["nr"]."/"), $this->language->t("View ".$d->v("title")), "updates");
    }
   }
   else
   {
    $r.="&nbsp;";
    $this->menuItem("actions", $this->language->t("Updates"), Array("online/updates/"), $this->language->t("Check for updates"), "updates");
   }

   $r.="</div>\n";

   //repositories menu
   foreach($this->repositories as $code => $info)
    if(in_array("Releases", $info["content"]))
     $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "updates", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";

   //upgrade submit
   if($this->upgrade)
   {
    $r.="<h3".($state!="installed"?" class=\"".$state."\"":"").">".$this->icon("updates".($state!="installed"?"_".$state:"_dim"), "small")." ".$this->language->t($action)."</h3>\n";

    if(request("confirm")&&request("upgrade"))
    {
     if($this->jasRootUser)
      $this->e->add("Are you sure you wish to ".strtolower($action).($action!="Reinstall"?" to":"").": ".$d->v("title")."?", "warning");

     $r.=$this->confirm((!$this->jasRootUser?Array("disabled"):false));
    }
    else
     $r.=$this->field($this->language->t("Items"), $this->form->button("confirm", $this->icon("updates", "small")." ".$this->language->t("Install"), false, false, "submit".($this->jasRootUser?"":" disabled")));
   }

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";

   if($this->upgrade) //make full page part of the form
    $r=$this->form($r);

   $_session->backend();

   return $r;
  }

  function _listReleases()
  {
   $r="";

   //gather local release information
   $local=$this->_getLocalRelease();

   //gather online versions
   $remotes=$this->_getRemoteReleases($this->repository, $this->name, ($this->section!="updates"));

   //render release table header
   $this->tableRow("releases", true);
   $this->tableColumn("releases", $this->language->t("Release"), false, false, Array("class" => "title"));
   $this->tableColumn("releases", $this->language->t("Version"), false, false, Array("colspan" => 2));

   foreach($this->repositories as $code => $info)
    if(in_array("Releases", $info["content"]))
     if(!$this->repository||$code==$this->repository)
      $this->tableColumn("releases", $code, Array("online/releases/".$code."/".($this->name?str_replace(" ", "+", $this->name)."/":"")), $this->language->t("Show releases for ".$info["name"]));

   //render release table body
   $currentFound=false;
   foreach($remotes as $key => $release)
   {
    if(!$key)
     continue;

    $remote=reset($release);
    if($key==$local["key"])
    {
     $currentFound=true;
     $state="installed";
    }
    else if(!($state=$this->_compareVersion($remote["version"].$remote["tag"], $local["version"].$local["tag"])))
     $state="installed";

    if($this->section=="updates"&&$state!="newer")
    {
     foreach(array_keys($remotes[""]) as $code)
      $remotes[""][$code]["max"]--;

     continue;
    }

    if(!$currentFound&&$state=="older"&&$this->section=="releases") //add local releases to the list
    {
     $this->_listCurrentRelease($local);
     $currentFound=true;
    }

    if($state=="newer")
     $action="Upgrade";
    else if($state=="older")
     $action="Downgrade";
    else
     $action="Reinstall";

    $this->tableRow("releases", false, ($state=="installed"?Array("class" => "hilite"):false));
    $this->tableColumn("releases", $remote["title"], Array("online/releases/".($this->repository?$this->repository:"+")."/".str_replace(" ", "+", $remote["title"])."/"), false, Array("class" => "title"));
    $this->tableColumn("releases", $remote["version"], false, false, Array("class" => "version"));
    $this->tableColumn("releases", ($remote["tag"]=="_"?"":$remote["tag"]), false, false, Array("class" => "tag"));

    foreach($this->repositories as $code => $info)
    {
     if(!in_array("Releases", $info["content"]))
      continue;

     if($this->repository&&$code!=$this->repository)
      continue;

     if(!isset($release[$code]))
      $this->tableColumn("releases", "");
     else
     {
      if($state=="newer")
       $tmp="Newer version";
      else if($state=="older")
       $tmp="Older version";
      else
      {
       if($release[$code]["date"]>$local["date"])
        $tmp="Older installed";
       else if($release[$code]["date"]<$local["date"])
        $tmp="Newer installed";
       else
        $tmp="Installed";
      }

      $this->menuItem($key.$code, $this->language->t($tmp), Array("online/release/".$code."/".$release[$code]["nr"]."/"), $this->language->t("View ".$release[$code]["title"]." v".$release[$code]["version"].($release[$code]["tag"]!="_"?" ".$release[$code]["tag"]:"")), "updates".($state!="installed"?"_".$state:""), Array("class" => $state));
      $this->menuItem($key.$code, $this->language->t($action), Array("online/upgrade/".$code."/".$release[$code]["nr"]."/"), $this->language->t($action." ".$release[$code]["file"]." on server"), "updates".($state!="installed"?"_".$state:""), Array("class" => $state, (!$this->jasRootUser?"disabled":false)));
      $this->menuItem($key.$code, $this->language->t("Download"), $info["url"]."files/onlineReleases/file/".$release[$code]["nr"]."/", $this->language->t("Download ".$release[$code]["file"]), "download");
      $this->menuItem($key.$code, $this->language->t("To server"), Array("online/release/".$code."/".$release[$code]["nr"]."/download/"), $this->language->t("Download ".$release[$code]["file"]." to server"), "download", (!$this->jasRootUser?Array("disabled"):false));

      $this->tableColumn("releases", $this->icon("updates".($state!="installed"?"_".$state:"")).$this->menu($key.$code), false, false, Array("class" => "repository popup right"));
     }
    }
   }

   if(!$currentFound&&$this->section=="releases")
    $this->_listCurrentRelease($local);

   if(sizeof($this->table["releases"])==1) //no entries
   {
    $this->tableRow("releases", false, Array("class" => "total"));
    $this->tableColumn("releases", $this->language->t("No updates available"), false, false, Array("colspan" => 3));

    foreach($remotes[""] as $code => $info)
     $this->tableColumn("releases", "", false, false, Array("class" => "repository"));
   }
   else
   {
    $this->tableRow("releases", false, Array("class" => "total"));
    $this->tableColumn("releases", $this->language->t("Total"), false, false, Array("colspan" => 3));

    foreach($remotes[""] as $code => $info)
     $this->tableColumn("releases", $info["max"], false, false, Array("class" => "repository nr"));
   }

    $r.=$this->table("releases");

   return $r;
  }
  function _listCurrentRelease($local)
  {
   $this->tableRow("releases", false, Array("hilite"));
   $this->tableColumn("releases", $local["title"], false, false, Array("class" => "title"));
   $this->tableColumn("releases", $local["version"], false, false, Array("class" => "version"));
   $this->tableColumn("releases", $local["tag"], false, false, Array("class" => "tag"));

   foreach($this->repositories as $code => $info)
    if(in_array("Releases", $info["content"]))
     if(!$this->repository||$code==$this->repository)
      $this->tableColumn("releases", "", false, false, Array("class" => "repository"));

   return true;
  }

  function _getLocalRelease()
  {
   $version=JAS::versionS();
   $tag=JAS::versionS("tag");

   if(substr_count($version, ".")==1) //no maintenance version
    $version.=".0";

   if(!$tag) //tag comparison workaround
    $tag="_";

   $local=Array(
    "key" => "JAS_".$version."_".$tag,
    "title" => "JAS",
    "version" => $version,
    "tag" => $tag,
    "date" => date("Y-m-d", JAS::versionS("updated")),
   );

   return $local;
  }

  function _getRemoteReleases($repository=false, $name=false, $all=false)
  {
   $releases=Array("" => Array());
   $d=Data::cache("onlineReleases", "view", "all", Array("skipCheckTables" => true, "format" => "sql"));
   $d->setShowFields("title", "version", "tag", "file", "size", "date");
   $d->sortOrder("title", "version-", "tag-");

   foreach($this->repositories as $code => $info)
   {
    if(!in_array("Releases", $info["content"]))
     continue;

    if($this->repository&&$code!=$this->repository)
     continue;

    $d->storageSwitch($info["apiurl"]);
    $d->clearValues();

    if(!$all)
     $d->setValue("aggregation_field", "title");

    if($name)
     $d->setValue("title", $name);

    if(!$d->get(0, false))
    {
     $this->e->add($d->e);
     continue;
    }

    $releases[""][$code]=Array("max" => $d->vmax, "count" => $d->getValue("show_count")); //totals

    while($d->next())
    {
     if(!$d->v["tag"])
      $d->v["tag"]="_";

     $key=$d->v["title"]."_".$d->v["version"]."_".$d->v["tag"];

     if(!isset($releases[$key]))
      $releases[$key]=Array();

     $releases[$key][$code]=$d->v;
    }
   }

   return $releases;
  }


  /*
   * Plugins
   */

  function output_plugins()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t("Plugins"), Array("online/plugins/"))
   );

   if($this->name&&in_array($this->name, JAS::plugins()))
   {
    $p=new Plugin($this->name);
    if($p->info())
    {
     $this->backend->setTitle($this->icon("updates_dim"),
      $this->link($this->language->t("Online"), Array("online/")),
      $this->link($this->language->t("Plugins"), Array("online/plugins/")),
      $this->link($p->l->t($p->title)." ".$p->version.$p->status2, Array("plugins/".$p->name."/"))
     );
    }
   }

   $r="<div class=\"twothird\">\n";

   if($this->repositories)
    $r.=$this->_listPlugins();
   else
    $r.="&nbsp;";

   $r.="</div>\n";

   //actions menu
   $this->menuItem("actions", $this->language->t("Updates"), Array("online/updates/"), $this->language->t("Check for updates"), "updates");
   $this->menuItem("actions", $this->language->t("Releases"), Array("online/releases/"), $this->language->t("View releases"), "updates");

   if($this->name&&($local=$this->_getLocalPlugins($this->name)))
    $this->menuItem("actions", $local["title"]." ".$local["version"], Array("plugins/".$this->name."/"), $this->language->t("View")." ".$local["title"]." ".$local["version"], "plugin");

   //repositories menu
   foreach($this->repositories as $code => $info)
    if(in_array("Plugins", $info["content"]))
     $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "online", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";


   $_session->backend();

   return $r;
  }

  function output_plugin($import=false)
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("updates_dim"),
    $this->link($this->language->t("Online"), Array("online/")),
    $this->link($this->language->t(($import?"Import":"Plugins")), Array("online/plugins/"))
   );

   $r="<div class=\"twothird\">\n";

   if($this->repository&&$this->nr)
   {
    //get remote plugin information
    $d=Data::cache("onlinePlugins", "view", "item", Array("skipCheckTables" => true, "db" => $this->repositories[$this->repository]["apiurl"]));
    $d->setShowFields("plugin", "title", "description", "version", "status", "dependencies", "license", "author", "changelog", "file", "fileSize");
    $d->addMutilation("changelog", "strip", 1000);
    $d->addMutilation("fileSize", "humanReadableFileSize", 1);

    $d->clearValues();
    $d->setValue("nr", $this->nr);

    if(!$d->get()||!$d->next())
    {
     $this->e->add($d->e);
     $r.="&nbsp;";
    }
    else
    {
     $this->backend->setTitle($this->icon("updates_dim"),
      $this->link($this->language->t("Online"), Array("online/")),
      $this->link($this->language->t(($import?"Import":"Plugins")), Array("online/plugins/")),
      $this->link($this->repository, Array("online/plugins/".$this->repository."/")),
      $this->link($d->v("title"), Array("online/".($import?"import":"plugin")."/".$this->repository."/".$d->v["nr"]."/"))
     );

     //gather local plugin information
     $local=$this->_getLocalPlugins($d->v["plugin"]);

     if(!$local)
      $state="uninstalled";
     else if(!($state=$this->_compareVersion($d->v["version"], $local["version"])))
      $state="installed";

     if($state=="newer")
      $action="Upgrade";
     else if($state=="older")
      $action="Downgrade";
     else if($state=="uninstalled")
      $action="Install";
     else
      $action="Reinstall";

     $this->tableRow("plugin", true);
     $this->tableColumn("plugin", $this->icon("updates".($state!="installed"&&$state!="uninstalled"?"_".$state:"_dim")));

     $this->tableColumn("plugin", $this->language->t("Installed"));
     $this->tableColumn("plugin", $this->repositories[$this->repository]["name"]);

     foreach(Array("version", "status", "license", "author") as $field)
     {
      $this->tableRow("plugin");
      $this->tableColumn("plugin", $d->l->t($d->field[$field]["title"]), false, false, Array("class" => "key"));

      if($state=="uninstalled")
      {
       $this->tableColumn("plugin", "", false, false, Array("class" => "value local ".$d->field[$field]["type"]));
       $this->tableColumn("plugin", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"]." changed"));
      }
      else
      {
       switch($field)
       {
        case "author":
        case "license":
        {
         if($local[$field]!=$d->v[$field])
          $class="older";
         else
          $class="same";

         $this->tableColumn("plugin", $local[$field], false, false, Array("class" => "value local ".$d->field[$field]["type"]));
         $this->tableColumn("plugin", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"].($class?" ".$class:"")));
         break;
        }
        default:
        {
         if($local[$field]<$d->v[$field])
          $class="newer";
         else if($local[$field]>$d->v[$field])
          $class="older";
         else
          $class="same";

         $this->tableColumn("plugin", $local[$field], false, false, Array("class" => "value local ".$d->field[$field]["type"]));
         $this->tableColumn("plugin", $d->v[$field], false, false, Array("class" => "value remote ".$d->field[$field]["type"].($class?" ".$class:"")));
         break;
        }
       }
      }
     }

     //list dependencies
     if($d->v["dependencies"]&&preg_match_all("/([a-z0-9\_]{1,25})-([\d\.]+)(.*required)?/i", $d->v["dependencies"], $m))
     {
      $this->tableRow("plugin", true);
      $this->tableColumn("plugin", $this->icon("dependency_dim", "small")." ".$d->l->t($d->field["dependencies"]["title"]), false, false, Array("colspan" => 3));

      $l=sizeof($m[0]);
      for($c=0;$c<$l;$c++)
      {
       if(substr_count($m[2][$c], ".")==1) //no maintenance version
        $m[2][$c].=".0";

       if($m[1][$c]=="JAS")
        $localdep=$this->_getLocalRelease();
       else
        $localdep=$this->_getLocalPlugins($m[1][$c]);

       if(!$localdep||$this->_compareVersion($m[2][$c], $localdep["version"])=="newer")
       {
        if($m[3][$c]) //required
         $class="older";
        else
         $class="changed";
       }
       else
        $class="same";

       $this->tableRow("plugin");

       if($m[1][$c]=="JAS")
        $this->tableColumn("plugin", $m[1][$c], Array("online/releases/".$this->repository."/".$m[1][$c]."/"), false, Array("class" => "key"));
       else
        $this->tableColumn("plugin", ($localdep?$localdep["title"]:ucfirst(str_replace("_", " ", $m[1][$c]))), Array("online/plugins/".$this->repository."/".$m[1][$c]."/"), false, Array("class" => "key"));

       $this->tableColumn("plugin", ($localdep?$localdep["version"]:""), false, false, Array("class" => "value local nr"));
       $this->tableColumn("plugin", $m[2][$c], false, false, Array("class" => "value remote nr".($class?" ".$class:"")));
      }
     }

     $r.=$this->table("plugin");

     if($d->v["description"])
      $r.="<div class=\"description\">".$d->v["description"]."</div>";

     $r.="<h2>".$this->icon("download_dim", "small")." ".$this->language->t("File")."</h2>";
     $r.="<div class=\"file\">";
     $r.=$this->link($d->v["file"], $this->repositories[$this->repository]["url"]."files/onlinePlugins/file/".$d->v["nr"]."/", $this->language->t("Download ".$d->v["file"]));
     $r.=" (".$d->v["fileSize"].")";
     $r.="</div>";

     if($d->v["changelog"])
     {
      $r.="<h2>".$this->icon("changelog_dim", "small")." ".$this->language->t("Changelog")."</h2>";
      $r.="<div class=\"changelog\">".$d->v["changelog"]."</div>";
     }


     //additional actions
     if($import)
      $this->menuItem("actions", $this->language->t("Plugin"), Array("online/plugin/".$this->repository."/".$d->v["nr"]."/"), $this->language->t("View ".$d->v("title")), "updates");
     else
      $this->menuItem("actions", $this->language->t($action), Array("online/import/".$this->repository."/".$d->v["nr"]."/", Array("confirm" => 1)), $this->language->t($action." ".$d->v["file"]." on server"), "updates".($state!="installed"&&$state!="uninstalled"?"_".$state:""), Array("class" => $state, (!$_config->userRoot()?"disabled":false)));

     $this->menuItem("actions", $this->language->t("Download"), $this->repositories[$this->repository]["url"]."files/onlinePlugins/file/".$d->v["nr"]."/", $this->language->t("Download ".$d->v["file"]), "download");
     $this->menuItem("actions", $this->language->t("To server"), Array("online/".($import?"import":"plugin")."/".$this->repository."/".$d->v["nr"]."/download/"), $this->language->t("Download ".$d->v["file"]." to server"), "download", (!$_config->userRoot()?Array("disabled"):false));

     if($local)
      $this->menuItem("actions", $local["title"]." ".$local["version"], Array("plugins/".$d->v["plugin"]."/"), $this->language->t("View")." ".$local["title"]." ".$local["version"], "plugin");
    }
   }
   else
   {
    $r.="&nbsp;";
    $this->menuItem("actions", $this->language->t("Updates"), Array("online/updates/"), $this->language->t("Check for updates"), "updates");
   }

   $r.="</div>\n";

   //repositories menu
   foreach($this->repositories as $code => $info)
    if(in_array("Plugins", $info["content"]))
     $this->menuItem("repositories", $info["name"], $info["url"], $info["name"]." website\n".$info["url"], "updates", Array("rel" => "external", "class" => "repository"));

   $this->menuItem("repositories", $this->language->t("Configuration"), Array("configuration/configRepositories/"), $this->language->t("View repositories configuration"), "config");

   $r.="<div class=\"third\">\n";

   //import submit
   if($import&&$_config->userRoot()&&isset($state))
   {
    $r.="<h3".($state!="installed"?" class=\"".$state."\"":"").">".$this->icon("updates".($state!="installed"&&$state!="uninstalled"?"_".$state:"_dim"), "small")." ".$this->language->t($action)."</h3>\n";

    if(request("confirm"))
    {
     $this->e->add("Are you sure you wish to ".strtolower($action).($action!="Reinstall"&&$action!="Install"?" to":"").": ".$d->v("title")."?", "warning");
     $r.=$this->confirm();
    }
    else
     $r.=$this->field($this->language->t("Items"), $this->form->button("confirm", $this->icon("updates", "small")." ".$this->language->t("Install"), false, false, "submit"));
   }

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions", "plugin");
   $r.="<h3>".$this->icon("updates_dim", "small")." ".$this->language->t("Repositories")."</h3>\n";
   $r.=$this->menu("repositories");
   $r.="<h3>".$this->icon("license_dim", "small")." ".$this->language->t("Disclaimer")."</h3>\n";
   $r.="<div class=\"disclaimer\">".$this->language->t("All software and plugins made available here are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the license of the individual repositories, software and plugins for more details.")."</div>";
   $r.="</div>\n";

   if($import&&$_config->userRoot()) //make full page part of the form
    $r=$this->form($r);


   $_session->backend();

   return $r;
  }

  function output_import()
  {
   return $this->output_plugin(true);
  }



  function _listPlugins()
  {
   global $_config;

   $r="";

   //gather local plugins information
   $locals=$this->_getLocalPlugins();

   //gather online versions
   $remotes=$this->_getRemotePlugins($this->repository, $this->name, ($this->name));

   //render plugin table header
   $this->tableRow("plugins", true);
   $this->tableColumn("plugins", $this->language->t("Plugin"), false, false, Array("class" => "title"));
   $this->tableColumn("plugins", $this->language->t("Version"), false, false, Array("colspan" => 2));

   foreach($this->repositories as $code => $info)
    if(in_array("Plugins", $info["content"]))
     if(!$this->repository||$code==$this->repository)
      $this->tableColumn("plugins", $code, Array("online/plugins/".$code."/".($this->name?str_replace(" ", "+", $this->name)."/":"")), $this->language->t("Show plugins for ".$info["name"]));

   //render plugin table body
   foreach(array_keys($remotes) as $plugin)
   {
    if(!$plugin)
     continue;

    $currentFound=false;

    if(isset($locals[$plugin]))
     $local=$locals[$plugin];
    else
     $local=false;

    foreach($remotes[$plugin] as $key => $plugin)
    {
     if(!$key)
      continue;

     $remote=reset($plugin);
     if(!$local)
      $state="uninstalled";
     else if($key==$local["key"])
     {
      $currentFound=true;
      $state="installed";
     }
     else if(!($state=$this->_compareVersion($remote["version"], $local["version"])))
      $state="installed";

     if($this->section=="updates"&&$state!="newer")
     {
      foreach(array_keys($remotes[""]) as $code)
       $remotes[""][$code]["max"]--;

      continue;
     }

     if($local&&!$currentFound&&$state=="older"&&$this->section=="plugins"&&$this->name) //add local plugin to the list
     {
      $this->_listCurrentPlugin($local);
      $currentFound=true;
     }

     if($state=="newer")
      $action="Upgrade";
     else if($state=="older")
      $action="Downgrade";
     else if($state=="uninstalled")
      $action="Install";
     else
      $action="Reinstall";

     $this->tableRow("plugins", false, ($state=="installed"?Array("class" => "hilite"):($state=="uninstalled"?Array("class" => "disabled"):false)));
     $this->tableColumn("plugins", $remote["title"], Array("online/plugins/".($this->repository?$this->repository:"+")."/".str_replace(" ", "+", $remote["plugin"])."/"), false, Array("class" => "title"));
     $this->tableColumn("plugins", $remote["version"], false, false, Array("class" => "version"));
     $this->tableColumn("plugins", $remote["status2"], false, false, Array("class" => "status"));

     foreach($this->repositories as $code => $info)
     {
      if(!in_array("Plugins", $info["content"]))
       continue;

      if($this->repository&&$code!=$this->repository)
       continue;

      if(!isset($plugin[$code]))
       $this->tableColumn("plugins", "");
      else
      {
       if($state=="newer")
        $tmp="Newer version";
       else if($state=="older")
        $tmp="Older version";
       else if($state=="uninstalled")
        $tmp="Not installed";
       else
        $tmp="Installed";

       $this->menuItem($key.$code, $this->language->t($tmp), Array("online/plugin/".$code."/".$plugin[$code]["nr"]."/"), $this->language->t("View ".$plugin[$code]["title"]." v".$plugin[$code]["version"]), "updates".($state!="installed"&&$state!="uninstalled"?"_".$state:""), Array("class" => $state));
       $this->menuItem($key.$code, $this->language->t($action), Array("online/import/".$code."/".$plugin[$code]["nr"]."/"), $this->language->t($action." ".$plugin[$code]["file"]." on server"), "updates".($state!="installed"&&$state!="uninstalled"?"_".$state:""), Array("class" => $state, (!$_config->userRoot()?"disabled":false)));
       $this->menuItem($key.$code, $this->language->t("Download"), $info["url"]."files/onlinePlugins/file/".$plugin[$code]["nr"]."/", $this->language->t("Download ".$plugin[$code]["file"]), "download");
       $this->menuItem($key.$code, $this->language->t("To server"), Array("online/plugin/".$code."/".$plugin[$code]["nr"]."/download/"), $this->language->t("Download ".$plugin[$code]["file"]." to server"), "download", (!$_config->userRoot()?Array("disabled"):false));

       $this->tableColumn("plugins", $this->icon("updates".($state!="installed"&&$state!="uninstalled"?"_".$state:"")).$this->menu($key.$code), false, false, Array("class" => "repository popup right"));
      }
     }
    }

    if($local&&!$currentFound&&$this->section=="plugins"&&$this->name) //add local plugin to the list
     $this->_listCurrentPlugin($local);
   }

   if(sizeof($this->table["plugins"])==1) //no entries
   {
    $this->tableRow("plugins", false, Array("class" => "total"));

    if($this->section=="updates")
     $this->tableColumn("plugins", $this->language->t("No updates available"), false, false, Array("colspan" => 3));
    else
     $this->tableColumn("plugins", $this->language->t("No plugins found"), false, false, Array("colspan" => 3));

    foreach($remotes[""] as $code => $info)
     $this->tableColumn("plugins", "", false, false, Array("class" => "repository"));
   }
   else
   {
    $this->tableRow("plugins", false, Array("class" => "total"));
    $this->tableColumn("plugins", $this->language->t("Total"), false, false, Array("colspan" => 3));

    foreach($remotes[""] as $code => $info)
     $this->tableColumn("plugins", $info["max"], false, false, Array("class" => "repository nr"));
   }

   $r.=$this->table("plugins");

   return $r;
  }
  function _listCurrentPlugin($local)
  {
   $this->tableRow("plugins", false, Array("hilite"));
   $this->tableColumn("plugins", $local["title"], false, false, Array("class" => "title"));
   $this->tableColumn("plugins", $local["version"], false, false, Array("class" => "version"));
   $this->tableColumn("plugins", $local["status2"], false, false, Array("class" => "status"));

   foreach($this->repositories as $code => $info)
    if(in_array("Plugins", $info["content"]))
     if(!$this->repository||$code==$this->repository)
      $this->tableColumn("plugins", "", false, false, Array("class" => "repository"));
  }

  function _getLocalPlugins($name=false)
  {
   $local=Array();
   $statusConvert=Array("alpha" => "&alpha;", "beta" => "&beta;", "stable" => "&gamma;", "deprecated" => "&delta;");

   foreach(Backend_Plugins::plugins() as $plugin => $v)
   {
    if($name&&$name!=$plugin)
     continue;

    $version=$v["version"];
    if(substr_count($version, ".")==1) //no maintenance version
     $version.=".0";

    $local[$plugin]=$v;
    $local[$plugin]["key"]=$plugin."_".$version;
    $local[$plugin]["version"]=$version;
    $local[$plugin]["status2"]=(isset($statusConvert[$v["status"]])?$statusConvert[$v["status"]]:"");
   }

   if($name&&isset($local[$name]))
    return $local[$name];

   return $local;
  }

  function _getRemotePlugins($repository=false, $name=false, $all=false)
  {
   $plugins=Array("" => Array());
   $d=Data::cache("onlinePlugins", "view", "all", Array("skipCheckTables" => true, "format" => "sql"));
   $d->setShowFields("plugin", "title", "version", "status2", "file", "size", "date");
   $d->sortOrder("title", "version-");

   foreach($this->repositories as $code => $info)
   {
    if(!in_array("Plugins", $info["content"]))
     continue;

    if($repository&&$code!=$repository)
     continue;

    $d->storageSwitch($info["apiurl"]);
    $d->clearValues();

    if(!$all)
     $d->setValue("aggregation_field", "plugin");

    if(($search=request("search"))!==NULL)
    {
     if($search) //get all online plugins if no search provided
     {
      $d->setAction("search");
      $d->setExtra("full");
      $d->setValue("search", $search);
     }
    }
    else if($name)
     $d->setValue("plugin", $name);
    else //list all plugins
    {
     $c=0;
     foreach(JAS::plugins() as $plugin)
     {
      $d->setValue("plugin", $plugin, $c);
      $d->setOperant("plugin", "OR", $c);
      $c++;
     }
    }

    if(!$d->get(0, false))
    {
     $this->e->add($d->e);
     continue;
    }

    if($d->vmax==1&&$d->action=="search") //only 1 search result
    {
     //fake "this->name"
     $this->name=$d->getValue("plugin");
     $d->setAction("view");
     $d->setExtra("all");

     $d->clearValues();
     $d->setValue("plugin", $this->name);

     if(!$d->get(0, false))
     {
      $this->e->add($d->e);
      continue;
     }
    }

    $plugins[""][$code]=Array("max" => $d->vmax, "count" => $d->getValue("show_count")); //totals

    while($d->next())
    {
     $key=$d->v["plugin"]."_".$d->v["version"];

     if(!isset($plugins[$d->v["plugin"]]))
      $plugins[$d->v["plugin"]]=Array();
     if(!isset($plugins[$d->v["plugin"]][$key]))
      $plugins[$d->v["plugin"]][$key]=Array();

     $plugins[$d->v["plugin"]][$key][$code]=$d->v;
    }
   }

   return $plugins;
  }


  /*
   * Support
   */

  static function _compareVersion($first, $second)
  {
   if(!preg_match("/^(\d+)\.(\d+)(\.(\d+))?(\s*([A-Z0-9\_]+))?$/i", $first, $m))
    return false;

   $first=Array(
    "major" => $m[1],
    "minor" => $m[2],
    "maintenance" => (isset($m[4])&&$m[4]?$m[4]:0),
    "tag" => (isset($m[6])&&$m[6]?$m[6]:0),
   );

   if(!preg_match("/^(\d+)\.(\d+)(\.(\d+))?(\s*([A-Z0-9\_]+))?$/i", $second, $m))
    return false;

   $second=Array(
    "major" => $m[1],
    "minor" => $m[2],
    "maintenance" => (isset($m[4])&&$m[4]?$m[4]:0),
    "tag" => (isset($m[6])&&$m[6]?$m[6]:0),
   );

   if(
    ($first["major"]>$second["major"])||
    ($first["major"]==$second["major"]&&$first["minor"]>$second["minor"])||
    ($first["major"]==$second["major"]&&$first["minor"]==$second["minor"]&&$first["maintenance"]>$second["maintenance"])||
    ($first["major"]==$second["major"]&&$first["minor"]==$second["minor"]&&$first["maintenance"]==$second["maintenance"]&&$first["tag"]>$second["tag"])
   )
    $state="newer";
   else if(
    ($first["major"]<$second["major"])||
    ($first["major"]==$second["major"]&&$first["minor"]<$second["minor"])||
    ($first["major"]==$second["major"]&&$first["minor"]==$second["minor"]&&$first["maintenance"]<$second["maintenance"])||
    ($first["major"]==$second["major"]&&$first["minor"]==$second["minor"]&&$first["maintenance"]==$second["maintenance"]&&$first["tag"]<$second["tag"])
   )
    $state="older";
   else
    $state="";

   return $state;
  }
 }
?>
