<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/plugins.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."plugin.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."fs.php");

 class Backend_Plugins extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->plugin=false;
   $this->module=false;
   $this->plugins=Array();

   $this->setupStatus=Array();
   $this->setupStatusMapping=Array(
    "Setup correctly" => "done",
    "Not setup" => "done",
    "Skipped" => "done",

    "Setup" => "ok",
    "Forced setup" => "ok",
    "DeSetup" => "ok",

    "File changed" => "warning",
   );
  }

  function main($plugin=false, $section=false, $module=false)
  {
   global $_session, $_config, $_path;

   //section
   if(!parent::main($section))
    return false;

   $_session->frontend();

   //import
   if($this->section=="import")
   {
    if(!$_config->userRoot())
    {
     $this->e->add("Plugin can only be imported by the root user", "error");
     return false;
    }

    //set page as CMS in history
    $hs=new History();
    if(!$hs->cms())
     $this->e->add($hs->e);

    if(request("s")) //confirm overwrite
    {
     $p=new Plugin(false);
     $p->e->clear();

     $p->importOverwrite($plugin);
     $this->e->add($p->e); //show import errors/success

     $this->section=false;
    }
    else if($file=request("file", "FILES"))
    {
     $pluginCache=$_path["project"]["cache"]."plugins/";
     if(!FS::existsS($pluginCache)&&!FS::mkdirS($pluginCache, true))
     {
      $this->e->add("Can't create plugin cache directory: \"".$pluginCache."\"", "error");
      return false;
     }

     if(!$file["name"])
     {
      $this->e->add("No file uploaded for import", "warning");
      $this->section=false;
     }
     else if(!FS::renameS($file["tmp_name"], $pluginCache.$file["name"]))
     {
      $this->e->add("Can't move uploaded file: \"".$file["tmp_name"]."\" to plugin cache directory: \"".$pluginCache."\"", "error");
      $this->section=false;
     }
     else
     {
      $p=new Plugin(false);
      $p->e->clear();

      if($p->import($pluginCache.$file["name"])) //import successful
       $this->section=false; //show plugin

      $this->e->add($p->e); //show import errors/success

      if(!$p->name||!in_array($p->name, JAS::plugins())) //import failed
       $this->section=false; //show plugin list
      else //show plugin
       $plugin=$p->name;
     }
    }
    else //no file uploaded
     $this->section=false;
   }

   //plugin specific actions
   if($plugin&&$plugin!="_")
   {
    if(!in_array($plugin, JAS::plugins()))
    {
     $this->e->add("Plugin: \"".$plugin."\" not found", "error");
     $this->section=false;
    }
    else
    {
     $this->plugin=new Plugin($plugin);
     $this->language->read("plugin_".$this->plugin->name);

     if(!$this->plugin->info())
      $this->e->add($this->plugin->e);

     if(request("enable")!==NULL)
     {
      if(request("enable"))
      {
       if(!$this->plugin->setStatus(true))
        $this->e->add($this->plugin->e);
       else
        $this->e->add("Plugin: \"".$this->plugin->title."\" enabled", "note");
      }
      else
      {
       if(!$this->plugin->setStatus(false))
        $this->e->add($this->plugin->e);
       else
        $this->e->add("Plugin: \"".$this->plugin->title."\" disabled", "note");
      }
     }
     else if($this->section)
     {
      if($this->section=="export")
      {
       if(!$this->plugin->export("zip"))
        $this->e->add($this->plugin->e);
      }
      else if($this->section=="delete")
      {
       //set page as CMS in history
       $hs=new History();
       if(!$hs->cms())
        $this->e->add($hs->e);

       if(!request("s"))
       {
        if($this->plugin->isEnabled())
        {
         $this->e->add("Can't delete active plugin: \"".$this->plugin->title." ".$this->plugin->version."\"", "error");
         $this->section=false;
        }
        else
         $this->e->add("Are you sure you wish to remove: \"".$this->plugin->title." ".$this->plugin->version."\"?", "warning");
       }
       else
       {
        $this->plugin->e->clear(); //clear errors (so only remove errors are returned)

        if(!$this->plugin->remove())
         $this->e->add($this->plugin->e);
        else
        {
         $this->e->add($this->plugin->e);
         $this->section=false;
         $this->plugin=false;
        }
       }
      }
      else if($this->section=="setup")
      {
       if(request("mode")=="DeSetup")
        $this->plugin->initClass("setup", false, true);
       else if(request("mode")=="Force")
        $this->plugin->initClass("setup", true, false);
       else
        $this->plugin->initClass("setup", false, false);

       if(request("s"))
       {
        $this->plugin->call("setup", "init");

        if(!$this->plugin->e->any("fatal"))
         $this->plugin->call("setup", "exec");

        $this->plugin->call("setup", "clear");
       }

       $this->e->add($this->plugin->e);
      }
      else if($this->section=="data")
      {
       if(strpos($module, $this->plugin->name."_")!==0) //plugin module
       {
        $this->e->add("Module: \"".$module."\" isn't a ".$this->plugin->title." plugin module", "error");
        $this->section=false;
       }
       else if(!in_array($module, JAS::modules()))
       {
        $this->e->add("Module: \"".$module."\" not found", "error");
        $this->section=false;
       }
       else
       {
        $this->module=$module;

        $d=Data::cache($this->module);

        //set inline search options
        if($titles=$d->check("properties", "title"))
        {
         $this->backend->search("data", $this->module, "title", $titles[0]);

         //search redirect
         if(request("search"))
         {
          $d->setAction("search");
          if($d->fulltextIndex) //full text index exists
           $d->setExtra("index");
          else //normal search
           $d->setExtra("full");

          $d->setShowFields();
          $d->setValue("show_total", 2);
          $d->setRequestValue("search");

          if($d->get()&&$d->vmax==1) //only 1 entry
           Headers::redirectS($this->url("data/".$d->module."/".$d->getValue($d->getID())."/"));
         }
        }
       }
      }
      else if($this->section=="frontend"||$this->section=="backend")
      {
       if(!$this->plugin->isEnabled())
       {
        $this->e->add("Plugin: \"".$this->plugin->title."\" is disabled", "warning");
        $this->section=false;
       }
       else if($this->section=="frontend")
       {
        if(!$this->plugin->classExists("frontend"))
        {
         $this->e->add("Plugin: \"".$this->plugin->title."\" doesn't have a frontend", "error");
         $_session->backend();
         return false;
        }

        if(($config=request("config")))
         if($this->plugin->config($config))
          $this->plugin->addVar("config", $config);

        if(!$this->backend->rewrite) //vars
         foreach($this->backend->pageVars as $k => $page)
          $this->plugin->addVar("p[".$k."]", $page);

        //call frontend
        if(!$this->plugin->call("frontend", "main"))
         $this->plugin->failed=true; //don't render output
       }
       else if($this->section=="backend")
       {
        if(!$this->plugin->classExists("backend"))
        {
         $this->e->add("Plugin: \"".$this->plugin->title."\" doesn't have a backend", "error");
         $_session->backend();
         return false;
        }

        if(!$this->backend->rewrite) //vars
         foreach($this->backend->pageVars as $k => $page)
          $this->plugin->addVar("p[".$k."]", $page);

        $this->plugin->getClass("backend")->backend=&$this->backend; //hand backend to plugin

        if(!$this->plugin->call("backend", "main"))
         $this->plugin->failed=true; //don't render output
       }
      }
     }
    }
   }

   if(!$this->section) //default section
   {
    $this->backend->search("plugins"); //inline search all plugins

    //search
    if($search=request("search"))
    {
     foreach($this->plugins() as $plugin => $v) //filter plugins list
      if(stristr($plugin, $search)||stristr($v["title"], $search))
       $this->plugins[$plugin]=$v;

     if(sizeof($this->plugins)==1) //only one plugin
      Headers::redirectS($this->url("plugins/".key($this->plugins)."/"));

     $this->plugin=false; //show results
    }
    else
     $this->plugins=$this->plugins();
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   if($this->plugin)
   {
    $this->backend->setTitle($this->icon("plugin_dim"),
     $this->link($this->language->t("Plugins"), Array("plugins/")),
     $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")));

    $r="<div class=\"twothird plugin\">\n";
    $r.="<h2>".$this->language->t("Version")." ".$this->plugin->version.$this->plugin->status2."</h2>\n";
    $r.="<div class=\"description\">".htmlentities($this->plugin->l->t($this->plugin->description), ENT_COMPAT|ENT_XHTML)."</div>\n";

    $r.="<div class=\"info\">\n";
    $r.="<div class=\"half author\">".htmlentities($this->plugin->author, ENT_COMPAT|ENT_XHTML).($this->plugin->email?" (".$this->link($this->plugin->email, "mailto:".$this->plugin->email."?Subject=JAS%20".$this->plugin->name."%20plugin%20v".$this->plugin->version, $this->language->t("Email: ".$this->plugin->author)).")":false)."</div>\n";
    $r.="<div class=\"half license\">".($this->plugin->license?$this->link($this->icon("license_dim", "small")." ".$this->plugin->license.(isset($this->plugin->info->listOptions["license"][$this->plugin->license])?" - ".$this->plugin->info->listOptions["license"][$this->plugin->license]:""), Array("plugins/".$this->plugin->name."/license/"), $this->language->t("Show ".$this->plugin->title." plugin license")):$this->icon("license_dim", "small")." ".$this->plugin->info->listOptions["license"][$this->plugin->license])."</div>\n";
    $r.="</div>\n";

    //plugin info
    $r.="<div class=\"info\">\n";

    //setup
    if($_config->userRoot()&&$this->plugin->call("setup", "init"))
    {
     $this->tableRow("setup", true);
     $this->tableColumn("setup", $this->link($this->language->t("Setup"), $this->url("plugins/".$this->plugin->name."/setup/"), $this->language->t("Setup ".$this->plugin->title), "setup"), false, false, Array("colspan" => 2));

     foreach(Array("modules", "files", "directories") as $type)
     {
      $status="";
      foreach($this->plugin->call("setup", "get", $type) as $v)
       if(!isset($this->setupStatusMapping[$v["status"]])||$this->setupStatusMapping[$v["status"]]!="done"||!$status)
        $status=$v["status"];

      $this->tableRow("setup", false, Array("class" => "status"));
      $this->tableColumn("setup", $this->language->t(ucfirst($type)));
      $this->tableColumn("setup", $status, false, false, Array("class" => ($status?(isset($this->setupStatusMapping[$status])?$this->setupStatusMapping[$status]:"failure"):false)));
     }

     $r.="<div class=\"half\">\n";
     $r.=$this->table("setup");
     $r.="</div>\n";
    }

    //show dependencies
    $this->plugin->dependencies();
    $this->language->read("plugin");
    if($this->plugin->deps)
    {
     $this->tableRow("dependencies", true);
     $this->tableColumn("dependencies", $this->icon("dependency_dim", "small")." ".$this->language->t("Dependencies"), false, false, Array("colspan" => 3));

     while($this->plugin->deps->next())
     {
      //get error for this dependency
      $error="";
      $status="Ok";
      if($this->plugin->deps->v["plugin"])
      {
       for($c=0;$c<$this->plugin->e->getErrors();$c++)
       {
        if(preg_match("/dependency (error|warning), (\"".$this->plugin->deps->v["plugin"]."\" .+)/i", $this->plugin->e->getError($c), $m))
        {
         $error=$m[2];

         if(stristr($error, "not found"))
          $status="Not found";
         else if(stristr($error, "disabled"))
          $status="Disabled";
         else if(stristr($error, "version"))
          $status="Version";
         else
          $status="Unknown";

         break;
        }
       }
      }
      else //JAS dependency
      {
       for($c=0;$c<$this->plugin->e->getErrors();$c++)
       {
        if(preg_match("/dependency (error|warning), (JAS .+)/i", $this->plugin->e->getError($c), $m))
        {
         $error=$m[2];
         $status="Version";

         break;
        }
       }
      }

      $this->tableRow("dependencies", false, Array("class" => "status"));

      if($this->plugin->deps->v["plugin"])
      {
       $plugin=ucfirst(str_replace("_", " ", $this->plugin->deps->v["plugin"]));

       if($status=="Ok"||$status=="Disabled")
        $this->tableColumn("dependencies", $plugin, Array("plugins/".$this->plugin->deps->v["plugin"]."/"), $this->language->t("Show: ".$plugin." plugin"));
       else
        $this->tableColumn("dependencies", $plugin, Array("online/plugins/+/".$this->plugin->deps->v["plugin"]."/"), $this->language->t("Search online for: ".$plugin." plugin"));
      }
      else
      {
       if($error)
        $this->tableColumn("dependencies", "JAS", Array("online/releases/+/JAS/"), $this->language->t("Search online for JAS release"));
       else
        $this->tableColumn("dependencies", "JAS");
      }

      $this->tableColumn("dependencies", $this->plugin->deps->v["version"], false, false, Array("class" => false));

      if($error)
       $this->tableColumn("dependencies", $this->tooltip($this->language->t($status), $this->language->t($error)), false, false, Array("class" => ($this->plugin->deps->v["required"]?"failure":"warning")));
      else
       $this->tableColumn("dependencies", $this->language->t("Ok"), false, false, Array("class" => "done"));
     }

     $r.="<div class=\"half\">\n";
     $r.=$this->table("dependencies");
     $r.="</div>\n";
    }

    $r.="</div>\n";
    $r.="</div>\n";


    $r.="<div class=\"third\">\n";

    //import / delete
    if($this->section=="import") //failed with warning (presumably overwrite)
    {
     $form="<h3>".$this->icon("import_dim", "small")." ".$this->language->t("Import")."</h3>\n";
     $form.=$this->confirm();

     $r.=$this->form($form, $this->url("plugins/".$this->plugin->name."/import/"))."\n";
    }
    else if($this->section=="delete") //delete confirmation
    {
     $form="<h3>".$this->icon("delete", "small")." ".$this->language->t("Delete")."</h3>\n";
     $form.=$this->confirm(Array("history"));

     $r.=$this->form($form, $this->url("plugins/".$this->plugin->name."/delete/"))."\n";
    }

    //actions
    $this->actionsMenu();

    $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
    $r.=$this->menu("actions");

    if($this->modulesMenu())
    {
     $r.="<h3>".$this->icon("module_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
     $r.=$this->table("modules");
    }

    $r.="</div>\n";
   }
   else //plugin list
   {
    $this->backend->setTitle($this->icon("plugin_dim"),
     $this->link($this->language->t("Plugins"), Array("plugins/")));

    //header
    $this->tableRow("plugins", true);
    $this->tableColumn("plugins", $this->language->t("Plugin"));
    $this->tableColumn("plugins", $this->language->t("Version"), false, false, Array("colspan" => 2));

    $enabled=0;
    foreach($this->plugins as $plugin => $v)
    {
     if(!$v["enabled"])
      $this->tableRow("plugins", false, Array("class" => "disabled"));
     else
     {
      $this->tableRow("plugins");
      $enabled++;
     }

     $this->tableColumn("plugins", $v["title"], $this->url("plugins/".$plugin."/"), $plugin." v".$v["version"]." ".$v["status"].($v["description"]?" - ".$v["description"]:""));
     $this->tableColumn("plugins", $v["version"], false, false, Array("class" => "version"));

     $this->tableColumn("plugins", $this->language->t(ucfirst($v["type"])));
    }

    $this->tableRow("plugins", false, Array("class" => "total"));

    if(!$this->plugins)
     $this->tableColumn("plugins", $this->language->t("No plugins"), false, false, Array("colspan" => 4));
    else
    {
     $this->tableColumn("plugins", $this->language->t("Total"));
     $this->tableColumn("plugins", "");
     $this->tableColumn("plugins", $enabled." / ".sizeof($this->plugins), false, false, Array("class" => "nr", "colspan" => 2));
    }

    $r="<div class=\"twothird\">\n";
    $r.=$this->table("plugins");
    $r.="</div>\n";


    $r.="<div class=\"third\">\n";

    //online search
    $form=$this->field($this->language->t("Search"), $this->form->input("text", "search", false, false, false, "placeholder=\"".$this->language->t("Online")."\""));
    $form.=$this->field(false, $this->form->button("", $this->icon("online", "small")." ".$this->language->t("Search"), false, false, "submit"));

    $r.="<h3>".$this->icon("online_dim", "small")." ".$this->language->t("Search online")."</h3>\n";
    $r.=$this->form($form, $this->url("online/plugins/"));

    //import
    if($_config->userRoot())
    {
     $form=$this->field($this->language->t("File"), $this->form->input("file", "file"));
     $form.=$this->field(false, $this->form->button("", $this->icon("import", "small")." ".$this->language->t("Import"), false, false, "submit"));

     $r.="<h3>".$this->icon("import_dim", "small")." ".$this->language->t("Import")."</h3>\n";
     $r.=$this->form($form, $this->url("plugins/_/import/"));
    }

    $r.="</div>\n";
   }

   $_session->backend();

   return $r;
  }

  function output_setup()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("plugin_dim"),
    $this->link($this->language->t("Plugins"), Array("plugins/")),
    $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
    $this->link($this->language->t("Setup"), Array("plugins/".$this->plugin->name."/setup/")));

   if($this->plugin->call("setup", "init"))
   {
    $this->tableRow("setup", true);
    $this->tableColumn("setup", $this->language->t("Modules"), false, false, Array("colspan" => 3));

    $total=0;
    foreach($this->plugin->call("setup", "get", "modules") as $v)
    {
     $total++;
     $this->tableRow("setup", false, Array("class" => "status"));
     $this->tableColumn("setup", $v["module"], false, false, Array("colspan" => 2));
     $this->tableColumn("setup", $v["status"], false, false, Array("class" => (isset($this->setupStatusMapping[$v["status"]])?$this->setupStatusMapping[$v["status"]]:"failure")));
    }

    if(!$total)
    {
     $this->tableRow("setup", false, Array("class" => "total"));
     $this->tableColumn("setup", $this->language->t("No modules to setup"), false, false, Array("colspan" => 3));
    }

    $this->tableRow("setup", true);
    $this->tableColumn("setup", $this->language->t("Files"), false, false, Array("colspan" => 3));

    $total=0;
    foreach($this->plugin->call("setup", "get", "files") as $v)
    {
     $total++;
     $this->tableRow("setup", false, Array("class" => "status"));
     $this->tableColumn("setup", $v["source"]);
     $this->tableColumn("setup", $v["destination"]);
     $this->tableColumn("setup", $v["status"], false, false, Array("class" => (isset($this->setupStatusMapping[$v["status"]])?$this->setupStatusMapping[$v["status"]]:"failure")));
    }

    if(!$total)
    {
     $this->tableRow("setup", false, Array("class" => "total"));
     $this->tableColumn("setup", $this->language->t("No files to setup"), false, false, Array("colspan" => 3));
    }

    $this->tableRow("setup", true);
    $this->tableColumn("setup", $this->language->t("Directories"), false, false, Array("colspan" => 3));

    $total=0;
    foreach($this->plugin->call("setup", "get", "directories") as $v)
    {
     $total++;
     $this->tableRow("setup", false, Array("class" => "status"));
     $this->tableColumn("setup", $v["directory"], false, false, Array("colspan" => 2));
     $this->tableColumn("setup", $v["status"], false, false, Array("class" => (isset($this->setupStatusMapping[$v["status"]])?$this->setupStatusMapping[$v["status"]]:"failure")));
    }

    if(!$total)
    {
     $this->tableRow("setup", false, Array("class" => "total"));
     $this->tableColumn("setup", $this->language->t("No directories to setup"), false, false, Array("colspan" => 3));
    }

    $r="<div class=\"twothird\">\n";
    $r.=$this->table("setup");
    $r.="</div>\n";
   }
   else
   {
    $r="<div class=\"twothird\">\n";
    $r.=$this->plugin->e->show();
    $r.="</div>\n";
   }

   $r.="<div class=\"third\">\n";

   //setup
   if($_config->userRoot())
   {
    $ds=Data::cache("empty", "add", "item");
    $ds->l->read("jas");

    $ds->addField("mode", "enum", "form", "reload", false, false, "Normal", "Mode");
    $ds->enumOptions("mode", Array("Normal", "Force", "DeSetup"));
    $ds->addField("s", "submit", "form", false, false, false, $this->language->t("Setup"), "");

    $ds->setShowFields(Array("mode", "s", "reload_trigger"));
    $ds->setRequestValue("mode");

    $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Setup")."</h3>\n";
    $r.=$this->cms($ds, $this->url("plugins/".$this->plugin->name."/setup/"))."\n";
   }

   //actions
   $this->actionsMenu();

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   if($this->modulesMenu())
   {
    $r.="<h3>".$this->icon("module_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
    $r.=$this->table("modules");
   }

   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_data()
  {
   global $_session;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");

   $this->backend->setTitle($this->icon("plugin_dim"),
    $this->link($this->language->t("Plugins"), Array("plugins/")),
    $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
    $this->link($d->l->t($d->title), Array("plugins/".$this->plugin->name."/data/".$this->module."/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
   {
    $d->setExtra("list");
    $d->setValue($field, $catnr);
   }

   $r="<div class=\"twothird\">\n";
   $r.=$this->viewList($d, Array("get" => 0));
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   if($this->modulesMenu())
   {
    $r.="<h3>".$this->icon("module_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
    $r.=$this->table("modules");
   }

   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_changelog()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("plugin_dim"),
    $this->link($this->language->t("Plugins"), Array("plugins/")),
    $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
    $this->link($this->language->t("Changelog"), Array("plugins/".$this->plugin->name."/changelog/")));

   if(!$this->plugin->path["pluginChangelog"])
    $this->e->add("Plugin: \"".$this->plugin->title."\" doesn't have a changelog", "error");

   $r="<div class=\"twothird\">\n";

   if($this->plugin->path["pluginChangelog"])
    $r.=preg_replace("!<br/>\n\s*<h4>.+</h4>!", "", $this->plugin->changelog(true));
   else
    $r.="&nbsp; ";

   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   if($this->modulesMenu())
   {
    $r.="<h3>".$this->icon("module_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
    $r.=$this->table("modules");
   }

   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_license()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("plugin_dim"),
    $this->link($this->language->t("Plugins"), Array("plugins/")),
    $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
    $this->link($this->language->t("License"), Array("plugins/".$this->plugin->name."/license/")));

   $license="";
   if($this->plugin->path["pluginLicense"])
    $license.=$this->plugin->license(true);

   if($this->plugin->license)
    $license.="<h2>".$this->icon("license_dim", "small")." ".$this->plugin->license.(isset($this->plugin->info->listOptions["license"][$this->plugin->license])?" - ".$this->plugin->info->listOptions["license"][$this->plugin->license]:"")."</h2>\n";

   if($this->plugin->license=="JAS")
   {
    $license.=JAS::licenseS(true)."<br/><br/>";
    $license.=$this->link($this->language->t("View the full license"), Array("about/license/"), "JAS License", "license", Array("class" => "small"));
   }
   else if($this->plugin->path["pluginLicense2"])
    $license.=$this->plugin->license(true, true);

   $r="<div class=\"twothird\">\n";

   if($license)
    $r.=$license;
   else
   {
    $this->e->add("Plugin: \"".$this->plugin->title."\" doesn't have a license", "error");
    $r.="&nbsp; ";
   }

   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   if($this->modulesMenu())
   {
    $r.="<h3>".$this->icon("module_dim", "small")." ".$this->language->t("Modules")."</h3>\n";
    $r.=$this->table("modules");
   }

   $r.="</div>\n";

   $_session->backend();

   return $r;
  }


  function output_frontend()
  {
   global $_session;

   $_session->frontend();

   if($this->plugin)
   {
    $this->backend->setTitle($this->icon("plugin_dim"),
     $this->link($this->language->t("Plugins"), Array("plugins/")),
     $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
     $this->link($this->language->t("Frontend"), Array("plugins/".$this->plugin->name."/frontend/")));

    //config dropdown
    $d=new Data("pluginConfigs", "view", "all");
    $d->setValue("plugin", $this->plugin->name);

    $tmp=Array("");
    if($d->get())
     while($d->next())
      $tmp[]=$d->v["key"];

    $r=$this->field($this->icon("config", "small")." ".$this->language->t("Configuration"),
     $this->form->select(false, "config", $tmp, request("config"), false, false, "onchange=\"this.form.submit();\"").
     $this->form->button("", $this->icon("view", "small"), false, false, "submit"));

    $r="<div class=\"pluginConfig\">".$this->form($r)."</div>";

    if(isset($this->plugin->failed)||($output=$this->plugin->call("frontend", "output"))===false) //main/output failed
     $this->e->add($this->plugin->e); //show plugin errors
    else
     $r.=$output; //show output
   }
   else
    $r="";

   $_session->backend();

   return $r;
  }

  function output_backend()
  {
   global $_session;

   $r="";
   $_session->frontend();

   if($this->plugin)
   {
    $this->backend->setTitle($this->icon("plugin_dim"),
     $this->link($this->language->t("Plugins"), Array("plugins/")),
     $this->link($this->plugin->l->t($this->plugin->title), Array("plugins/".$this->plugin->name."/")),
     $this->link($this->language->t("Backend"), Array("plugins/".$this->plugin->name."/backend/")));

    if(isset($this->plugin->failed)||($output=$this->plugin->call("backend", "output"))===false) //main/output failed
     $this->e->add($this->plugin->e); //show plugin errors
    else
     $r.=$output; //show output
   }

   $_session->backend();

   return $r;
  }

  function output_quick()
  {
   global $_session;

   $this->menuItem("plugins", $this->language->t("Plugins"), $this->url("plugins/"), $this->language->t("Plugins"), "plugin");

   //list plugins
   $_session->frontend();

   if(substr(request("p", 0), 0, 6)=="plugin")
    $currentPlugin=request("p", 1);
   else
    $currentPlugin=false;

   if(preg_match(REGEXP_DATA_PLUGIN_MODULE, request("p", 1)))
    $currentModule=request("p", 1);
   else
    $currentModule=false;

   foreach($this->plugins() as $plugin => $v)
   {
    if($v["enabled"])
    {
     $this->menuItem("plugins", $v["title"], $this->url("plugins/".$plugin."/"), $plugin." v".$v["version"]." ".$v["status"].($v["description"]?" - ".$v["description"]:""), false, Array("menu" => $plugin."_modules", "class" => "popup", ($plugin==$currentPlugin?"selected":false)));

     //render modules sub menu
     $modules=Array();
     foreach(Backend_Modules::modules(true) as $module => $v)
      if(strpos($module, $plugin."_")===0) //plugin module
       $this->menuItem($plugin."_modules", $v["title"], $this->url("plugins/".$plugin."/data/".$module."/"), $v["title"], false, Array(($module==$currentModule?"selected":false)));
    }
   }

   $_session->backend();

   //search online
   $this->menuItem("plugins", $this->form->input("text", "search", false, false, false, "onchange=\"this.form.submit();\" placeholder=\"Online\" autocomplete=\"off\"")."<button type=\"submit\">".$this->icon("view", "small")."</button>");

   $r=$this->form($this->menu("plugins"), $this->url("online/plugins/"));

   return $r;
  }


  /*
   * Support
   */

  static function plugins()
  {
   global $_path, $_session, $_Backend_Plugin_List, $backend;

   $project=$_session->get("project");

   if(isset($_Backend_Plugin_List[$project]))
    return $_Backend_Plugin_List[$project];

   //read plugin status
   $enabled=Array();
   if(FS::existsS($_path["project"]["plugins"]."/status.txt"))
   {
    foreach(FS::readLinesS($_path["project"]["plugins"]."/status.txt") as $line)
    {
     $line=explode("=", $line);
     if(isset($line[1])&&$line[1]) //plugin enabled
      $enabled[]=$line[0];
    }
   }

   $language=new Language();
   $_Backend_Plugin_List[$project]=Array();
   foreach(JAS::plugins() as $plugin)
   {
    $language->sets=Array();
    $language->read("plugin_".$plugin);

    //find plugin info
    $values=Array();
    if(FS::existsS($_path["project"]["plugins"].$plugin."/pluginInfo.xml"))
    {
     $values["type"]="project";
     $file=$_path["project"]["plugins"].$plugin."/pluginInfo.xml";
    }
    else if(FS::existsS($_path["plugins"].$plugin."/pluginInfo.xml"))
    {
     $values["type"]="system";
     $file=$_path["plugins"].$plugin."/pluginInfo.xml";
    }
    else //no plugin config
    {
     $backend->e->add("No plugin info for plugin: \"".$plugin."\"", "plugin", "pluginInfo.xml not found in plugin root");
     continue;
    }

    //get values from plugin config (faster than using plugin->info)
    if($m=FS::grepS("/name=\"([^\"]+)\"\s+value=\"([^\"]+)\"/", $file))
    {
     $l=sizeof($m[0]);
     for($c=0;$c<$l;$c++)
      $values[$m[0][$c]]=html_entity_decode($m[1][$c], ENT_QUOTES|ENT_XHTML);
    }

    $_Backend_Plugin_List[$project][$plugin]=Array(
     "enabled" => in_array($plugin, $enabled),
     "title" => (isset($values["title"])?$language->t($values["title"]):$plugin),
     "type" => (isset($values["type"])?$values["type"]:false),
     "description" => (isset($values["description"])?$language->t($values["description"]):false),
     "version" => (isset($values["version"])?$values["version"]:false),
     "status" => (isset($values["status"])?$values["status"]:false),
     "license" => (isset($values["license"])?$values["license"]:false),
     "author" => (isset($values["author"])?$values["author"]:false),
     "email" => (isset($values["email"])?$values["email"]:false),
    );
   }

   return $_Backend_Plugin_List[$project];
  }

  function actionsMenu()
  {
   global $_config;

   if($this->plugin->isEnabled())
   {
    $this->menuItem("actions", $this->language->t("Frontend"), $this->url("plugins/".$this->plugin->name."/frontend/"), $this->language->t("Show ".$this->plugin->title." frontend"), "view", Array(($this->section=="frontend"?"selected":false), (!$this->plugin->classExists("frontend")?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Backend"), $this->url("plugins/".$this->plugin->name."/backend/"), $this->language->t("Show ".$this->plugin->title." backend"), "view", Array(($this->section=="backend"?"selected":false), (!$this->plugin->classExists("backend")?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Setup"), $this->url("plugins/".$this->plugin->name."/setup/"), $this->language->t("Setup ".$this->plugin->title), "setup", Array(($this->section=="setup"?"selected":false), (!$this->plugin->classExists("setup")||!$_config->userRoot()?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Export"), $this->url("plugins/".$this->plugin->name."/export/"), $this->language->t("Export ".$this->plugin->title." plugin"), "export", Array(($this->section=="backend"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Changelog"), $this->url("plugins/".$this->plugin->name."/changelog/"), $this->language->t("Show ".$this->plugin->title." plugin changelog"), "changelog", Array(($this->section=="changelog"?"selected":false), (!$this->plugin->path["pluginChangelog"]?"disabled":false)));
    $this->menuItem("actions", $this->language->t("License"), $this->url("plugins/".$this->plugin->name."/license/"), $this->language->t("Show ".$this->plugin->title." plugin license"), "license", Array(($this->section=="license"?"selected":false), (!$this->plugin->license&&!$this->plugin->path["pluginLicense"]&&!$this->plugin->path["pluginLicense2"]?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Updates"), $this->url("online/plugins/+/".$this->plugin->name."/"), $this->language->t("Check for updates"), "online");
    $this->menuItem("actions", $this->language->t("Disable"), $this->url("plugins/".$this->plugin->name."/", Array("enable" => 0)), $this->language->t("Disable ".$this->plugin->title. " plugin"), "disable", Array((request("enable")!==NULL?"selected":false), (!$_config->userRoot()?"disabled":false)));
   }
   else
   {
    $this->menuItem("actions", $this->language->t("Enable"), $this->url("plugins/".$this->plugin->name."/", Array("enable" => 1)), $this->language->t("Enable ".$this->plugin->title." plugin"), "enable", Array((request("enable")!==NULL?"selected":false), (!$this->plugin->classExists("setup")||!$_config->userRoot()?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Export"), $this->url("plugins/".$this->plugin->name."/export/"), $this->language->t("Export ".$this->plugin->title." plugin"), "export", Array(($this->section=="backend"?"selected":false)));
    $this->menuItem("actions", $this->language->t("Changelog"), $this->url("plugins/".$this->plugin->name."/changelog/"), $this->language->t("Show ".$this->plugin->title." plugin changelog"), "changelog", Array(($this->section=="changelog"?"selected":false), (!$this->plugin->path["pluginChangelog"]?"disabled":false)));
    $this->menuItem("actions", $this->language->t("License"), $this->url("plugins/".$this->plugin->name."/license/"), $this->language->t("Show ".$this->plugin->title." plugin license"), "license", Array(($this->section=="license"?"selected":false), (!$this->plugin->license&&!$this->plugin->path["pluginLicense"]&&!$this->plugin->path["pluginLicense2"]?"disabled":false)));
    $this->menuItem("actions", $this->language->t("Updates"), $this->url("online/plugins/+/".$this->plugin->name."/"), $this->language->t("Check for updates"), "online");
    $this->menuItem("actions", $this->language->t("Delete"), $this->url("plugins/".$this->plugin->name."/delete/"), $this->language->t("Delete ".$this->plugin->title." plugin"), "delete", Array(($this->section=="delete"?"selected":false), (!$_config->userRoot()?"disabled":false)));
   }

   return true;
  }

  function modulesMenu()
  {
   //get modules
   $modules=Array();
   foreach(Backend_Modules::modules(true) as $module => $v)
    if(strpos($module, $this->plugin->name."_")===0) //plugin module
     $modules[$module]=$v;

   if(!$modules)
    return false;

   $this->tableRow("modules", true);
   $this->tableColumn("modules", "", false, false, Array("class" => "module", "colspan" => 2));

   //modules list
   $enabled=0;
   foreach($modules as $module => $v)
   {
    if(strpos($module, "_pluginInfo")) //skip pluginInfo module
     continue;

    if(!$v["enabled"])
    {
     $this->tableRow("modules", false, Array("class" => "disabled"));
     $this->tableColumn("modules", $v["title"], $this->url("database/".$module."/"), $this->language->t("Create table: ".$v["title"]), Array("colspan" => 2));
    }
    else
    {
     $enabled++;
     $this->tableRow("modules");
     $this->tableColumn("modules", $v["title"], $this->url("plugins/".$this->plugin->name."/data/".$module."/"), $v["title"], Array("colspan" => 2));
    }
   }

   $this->tableRow("modules", false, Array("class" => "total"));

   if(!$modules)
    $this->tableColumn("modules", $this->language->t("No modules"), false, false, Array("colspan" => 2));
   else
   {
    $this->tableColumn("modules", $this->language->t("Total"));
    $this->tableColumn("modules", $enabled." / ".sizeof($modules), false, false, Array("class" => "nr"));
   }

   return true;
  }
 }
?>
