<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/projects.database.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."fs.php");

 class Backend_Projects_Database extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->fs=new FS();

   $this->data=false;
   $this->jasRootUser=false;
  }


  function data()
  {
   global $_session, $_path;

   $this->data=Data::cache("empty", "add", "item", Array("skipCheckTables" => true));

   $_session->frontend();

   $db=false;
   if($this->jasRootUser&&$this->fs->exists($_path["project"]["include"]."db.conf.php")) //only read DB information if jas root user
   {
    require($_path["project"]["include"]."db.conf.php");

    if(isset($db["default"]))
    {
     $storage=new Storage($this->data, false);
     $db=$storage->getConfig();
    }
   }

   $this->data->addField("type", "enum", "all", "reload", false, false, ($db?$db["type"]:"mysql"), "Database type");
   $this->data->listOptions("type", Array("api" => "API", "mysql" => "MariaDB / MySQL", "xml" => "XML"));
   $this->data->group("Type", "type", "reload_trigger");

   //API
   $this->data->addField("url", "url", "all", "", 5, 255, ($db?(isset($db["vars"]["secure"])?"https":"http")."://".$db["host"].($db["path"]?$db["path"]:"/"):"https://jas.junerules.com/"), "URL");
   $this->data->addField("api", "enum", "all", "", false, false, ($db&&isset($db["vars"]["type"])&&$db["vars"]["type"]?$db["vars"]["type"]:"Data"), "API type");
   $this->data->listOptions("api", Array("CSV", "Data", "XML"));
   $this->data->group("API", "url", "api");

   //MYSQL
   $this->data->addField("host", "hostname", "all", "", 5, 255, ($db&&$db["host"]?$db["host"]:"localhost"), "Hostname");
   $this->data->addField("port", "nr", "all", "", false, 65535, ($db&&$db["port"]?$db["port"]:"3306"), "Port");

   $this->data->addField("name", "title", "all", "", 2, 255, ($db&&$db["name"]?$db["name"]:$_session->get("project")), "Database name");
   $this->data->addField("user", "title", "all", "", 2, 255, ($db&&$db["user"]?$db["user"]:$_session->get("project")), "Username");
   $this->data->addField("pass", "password", "all", "password", 11, 255, ($db&&$db["pass"]?$db["pass"]:false), "Password");
   $this->data->field["verifypass"]["title"].="word";

   $this->data->addField("adminuser", "text", "all", "", false, 255, "root", "Database admin username");
   $this->data->addField("adminpass", "text", "all", "", false, 255, false, "Database admin password");

   $this->data->group("Server", "host", "port");
   $this->data->group("Details", "name", "user", "pass", "verifypass");
   $this->data->group("Admin user", "adminuser", "adminpass");

   //XML
   $this->data->addField("path", "text", "all", "", false, 255, ($db?$db["host"].$db["path"]:$_path["project"]["data"]), "Path");
   $this->data->group("XML", "path");

   $this->data->setRequestValues();

   $_session->backend();
  }


  function status(&$text)
  {
   global $_path, $_session;

   $_session->frontend();

   if(!$this->fs->exists($_path["project"]["include"]."db.conf.php"))
   {
    $text="Database configuration: \"".$_path["project"]["include"]."db.conf.php\" doesn't exist";
    $_session->backend();
    return "Failure";
   }

   require($_path["project"]["include"]."db.conf.php");

   if(!isset($db["default"]))
   {
    $text="Database configuration: \"".$_path["project"]["include"]."db.conf.php\" doesn't contain a default configuration";
    $_session->backend();
    return "Failure";
   }

   $d=Data::cache("config", "view", "all", Array("skipCheckTables" => true));
   if(!($storage=$d->storageObject()))
   {
    $this->language->read("storage");
    $text=$d->e->getError(0);
    $_session->backend();
    return "Failure";
   }

   $_session->backend();
   return "Done";
  }

  function main($submit=true)
  {
   global $_session, $_config, $_path;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status
   if(!$this->jasRootUser)
   {
    $this->e->add("Project database can only be setup by the JAS root user", "error");
    return false;
   }

   $this->data();

   if(request("s")&&$submit)
   {
    $success=true;
    $_session->frontend();

    foreach($this->data->check("group", "1") as $field)
     if(!$this->data->checkField($field))
      $success=false;

    switch($this->data->getValue("type"))
    {
     case "api":
     {
      foreach($this->data->check("group", "2") as $field)
       if(!$this->data->checkField($field))
        $success=false;

      $api=$this->data->getValue("url")."/api/".($this->data->getValue("api")?strtolower($this->data->getValue("api")):"data")."/";
      if(!($headers=@get_headers($api, 1)))
      {
       $this->e->add("Unable to connect to API: \"".$api."\"", "error");
       $success=false;
      }
      else if(strstr($headers[0], "404"))
      {
       $this->e->add("API page: \"".$api."\" not found", "error");
       $success=false;
      }
      else if(!isset($headers["X-JAS-API"]))
      {
       $this->e->add("API page: \"".$api."\" not a JAS API", "error");
       $success=false;
      }

      if(preg_match("!^([a-z0-9]+)://(.+)$!i", $this->data->getValue("url"), $m))
      {
       $db="\"api://".$m[2];

       if($m[1]=="https")
        $db.=(strpos($db, "?")?"&":"?")."secure";

       if($this->data->getValue("api")!="Data")
        $db.=(strpos($db, "?")?"&":"?")."type=".$this->data->getValue("api");

       $db.="\"";
      }

      break;
     }
     case "mysql":
     {
      if(!$this->data->getValue("pass")) //no password provided
      {
       $this->data->setDefaultValue("pass");
       $this->data->setDefaultValue("verifypass");
      }

      foreach($this->data->check("group", "3") as $field)
       if(!$this->data->checkField($field))
        $success=false;

      foreach($this->data->check("group", "4") as $field)
       if(!$this->data->checkField($field))
        $success=false;

      $db="mysql://".$this->data->getValue("user").":".$this->data->getValue("pass")."@".$this->data->getValue("host").($this->data->getValue("port")&&$this->data->getValue("port")!=3306?":".$this->data->getValue("port"):"")."/".$this->data->getValue("name");
      $dbroot="mysql://".$this->data->getValue("adminuser").":".$this->data->getValue("adminpass")."@".$this->data->getValue("host").($this->data->getValue("port")&&$this->data->getValue("port")!=3306?":".$this->data->getValue("port"):"")."/mysql";

      if($success) //validate database
      {
       $droot=clone $this->data;
       $droot->db=$dbroot;
       $mysqlroot=$droot->storageObject();
       unset($droot, $dbroot);

       $last=false;
       $this->data->db=$db;
       while($success&&!($mysql=$this->data->storageObject())) //keep trying to connect
       {
        $errno=preg_replace("/^.*\[(\d+)\].*$/", "\\1", $this->data->e->getExtra(0));

        if($last==$errno) //last attempt failed in the same way
        {
         $this->e->add("Unable to fix database setup, error code: ".$errno." persists", "error");
         $success=false;
         break;
        }

        $last=$errno;
        switch($errno)
        {
         case "2002": //unable to connect
         {
          $this->e->add("Please check your hostname and port", "warning");
          $success=false;
          break;
         }
         case "1044": //no database access
         case "1049": //no database
         {
          if(!$mysqlroot)
          {
           $this->e->add("Please provide (correct) root username and password to fix database access", "warning");
           $success=false;
           break;
          }

          //get hostname
          if($this->data->getValue("host")=="localhost")
           $connectFrom="localhost";
          else //get current user's hostname
          {
           if(!$mysqlroot->exec("SELECT USER() as user;", "Get current user")||!$mysqlroot->next())
           {
            $this->e->add($mysqlroot->e);
            $success=false;
            break;
           }

           $connectFrom=preg_replace("/^.+@/", "", $mysqlroot->res["user"]);
          }

          //check if database exists
          if(!$mysqlroot->exec("USE `".$this->data->getValue("name")."`;", "Select database")) //database doesn't exist
          {
           $mysqlroot->e->clear();

           //create database
           if(!$mysqlroot->exec("CREATE DATABASE `".$this->data->getValue("name")."` DEFAULT CHARACTER SET \"".$mysqlroot->charset."\" COLLATE  \"".$mysqlroot->collation."\";", "Create database"))
           {
            $this->e->add($mysqlroot->e);
            $success=false;
            break;
           }

           $this->e->add("Database: \"".$this->data->getValue("name")."\" created", "warning");
          }
          else //database exists, reselect mysql database
           $mysqlroot->exec("USE `mysql`;", "Select database");

          //grant privileges
          if(!$mysqlroot->exec("GRANT SELECT, INSERT, UPDATE, DELETE, CREATE, ALTER, INDEX ON `".$this->data->getValue("name")."`.* TO \"".addslashes($this->data->getValue("user"))."\"@\"".addslashes($connectFrom)."\";", "Grant privileges"))
          {
           $this->e->add($mysqlroot->e);
           $success=false;
           break;
          }

          $this->e->add("Granted user: \"".$this->data->getValue("user")."@".$this->data->getValue("host")."\" access to database: \"".$this->data->getValue("name")."\"", "warning");

          //update privileges
          if(!$mysqlroot->exec("FLUSH PRIVILEGES;", "Flush privileges"))
          {
           $this->e->add($mysqlroot->e);
           $success=false;
           break;
          }

          break;
         }
         case "1045": //access denied
         {
          if(!$mysqlroot)
          {
           $this->e->add("Please provide (correct) root username and password to fix database access", "warning");
           $success=false;
           break;
          }

          //get hostname
          if($this->data->getValue("host")=="localhost")
           $connectFrom="localhost";
          else //get current user's hostname
          {
           if(!$mysqlroot->exec("SELECT USER() as user;", "Get current user")||!$mysqlroot->next())
           {
            $this->e->add($mysqlroot->e);
            $success=false;
            break;
           }

           $connectFrom=preg_replace("/^.+@/", "", $mysqlroot->res["user"]);
          }

          //check if user exists
          if(!$mysqlroot->exec("SELECT User FROM user WHERE `User`=\"".addslashes($this->data->getValue("user"))."\" AND `Host`=\"".addslashes($connectFrom)."\";", "Get user"))
          {
           $this->e->add($mysqlroot->e);
           $success=false;
           break;
          }

          if(!$mysqlroot->next()) //no user
          {
           //create user
           if(!$mysqlroot->exec("CREATE USER \"".addslashes($this->data->getValue("user"))."\"@\"".addslashes($connectFrom)."\" IDENTIFIED BY \"".addslashes($this->data->getValue("pass"))."\";", "Create user"))
           {
            $this->e->add($mysqlroot->e);
            $success=false;
            break;
           }

           $this->e->add("Database user: \"".$this->data->getValue("user")."@".$this->data->getValue("host")."\" created", "warning");
          }
          else //user exists
          {
           //update password
           if(!$mysqlroot->exec("SET PASSWORD FOR \"".addslashes($this->data->getValue("user"))."\"@\"".addslashes($connectFrom)."\" = PASSWORD(\"".addslashes($this->data->getValue("pass"))."\");", "Change password"))
           {
            $this->e->add($mysqlroot->e);
            $success=false;
            break;
           }

           $this->e->add("Database user: \"".$this->data->getValue("user")."@".$this->data->getValue("host")."\" password changed", "warning");
          }

          //update privileges
          if(!$mysqlroot->exec("FLUSH PRIVILEGES;", "Flush privileges"))
          {
           $this->e->add($mysqlroot->e);
           $success=false;
           break;
          }

          break;
         }
         default:
         {
          $this->e->add("Unknown database error code: ".$errno, "warning");
          $success=false;
          break;
         }
        }

        if($success)
         $this->data->e->clear(); //clear errors for next run
       }

       //close database connections
       if($mysqlroot&&$mysqlroot->dbcon)
       {
        mysqli_close($mysqlroot->dbcon);
        $mysqlroot->dbcon=false;
       }
      }

      $db="\"".$db."\"";
      break;
     }
     case "xml":
     {
      foreach($this->data->check("group", "5") as $field)
       if(!$this->data->checkField($field))
        $success=false;

      if(!($directory=$this->data->getValue("path")))
       $directory=$_path["project"]["data"];

      if(!$this->fs->exists($directory))
      {
       $this->e->add("Database path: \"".$directory."\" doesn't exist", "error");
       $success=false;
      }
      else if(!$this->fs->exists($directory, "directory"))
      {
       $this->e->add("Database path: \"".$directory."\" isn't a directory", "error");
       $success=false;
      }
      else
      {
       $db=false;
       foreach($_path["project"] as $k => $v)
       {
        if($directory==$v) //same as path
        {
         $db="\"xml://\".\$_path[\"project\"][\"".$k."\"]"; //use path variable
         break;
        }
       }

       if(!$db) //no match in path variable
        $db="\"xml://".$directory."\""; //use actual path
      }

      break;
     }
     default:
     {
      $this->e->add("Database type: \"".$this->data->getValue("type")."\" unknown", "error");
      $success=false;
     }
    }

    if(!$success)
    {
     $this->e->add($this->data->e);
     $_session->backend();
     return false;
    }

    if(!$this->fs->exists($_path["project"]["include"]."db.conf.php")||!$this->fs->size($_path["project"]["include"]."db.conf.php"))
    {
     $conf='<?
$db["default"]='.$db.';
?>
';

     if(!$this->fs->write($_path["project"]["include"]."db.conf.php", $conf, "overwrite"))
     {
      $this->e->add($this->fs->e);
      $_session->backend();
      return false;
     }
    }
    else //exists
    {
     if(($conf=$this->fs->read($_path["project"]["include"]."db.conf.php"))===false)
     {
      $this->e->add($this->fs->e);
      $_session->backend();
      return false;
     }

     $conf=preg_replace("/(\s*\\\$db\[\"default\"\]=).*/", "\\1".$db.";", $conf);

     if(!$this->fs->write($_path["project"]["include"]."db.conf.php", $conf, "overwrite"))
     {
      $this->e->add($this->fs->e);
      $_session->backend();
      return false;
     }
    }

    if($this->fs->getOtherMode($_path["project"]["include"]."db.conf.php")) //check if other users have access
     if(!$this->fs->setOtherMode($_path["project"]["include"]."db.conf.php", 0)) //remove access for other users
      $this->e->add("Unable to remove rights from: \"".$_path["project"]["include"]."db.conf.php"."\" for other users", "warning");

    $this->e->add("Database for project: \"".$_session->get("project")."\" successfully configured", "note", preg_replace("!^(\"\w+\://).+@!", "\\1", $db));
    $_session->backend();
    return "next";
   }

   $this->e->add($this->data->e);
   return true;
  }


  /*
   * Output
   */

  function output()
  {
   global $_session;

   if(!$this->jasRootUser)
    return "&nbsp; ";

   $_session->frontend();

   $cms=new CMS();
   $cms->l->read("jas");
   $cms->setDataForm($this->data, $this->form);

   $form="";
   foreach($this->data->check("group", "1") as $field)
    $form.=$cms->field($field);

   switch($this->data->getValue("type"))
   {
    case "api":
    {
     foreach($this->data->check("group", "2") as $field)
      $form.=$cms->field($field);

     break;
    }
    case "mysql":
    {
     if(!$this->data->check("valuetype", "request", "pass")) //no (new) password submitted
     {
      $this->data->unsetValue("pass"); //don't show password
      $this->data->unsetValue("verifypass");
     }

     foreach($this->data->check("group", "3") as $field)
      $form.=$cms->field($field);

     $form.="<br/>";

     foreach($this->data->check("group", "4") as $field)
      $form.=$cms->field($field);

     $form.="<br/>";

     $this->data->setAction("view"); //ensure adminpass doesn't show generate option
     $this->data->field["adminpass"]["type"]="password"; //ensure adminpass is masked
     foreach($this->data->check("group", "5") as $field)
      $form.=$cms->field($field);

     break;
    }
    case "xml":
    {
     foreach($this->data->check("group", "6") as $field)
      $form.=$cms->field($field);

     break;
    }
    default:
    {
     $this->e->add("Database type: \"".$this->data->getValue("type")."\" unknown", "error");
     break;
    }
   }

   $form.=$this->confirm(Array("history", (!$this->jasRootUser?"disabled":false)));
   $r=$this->form($form, $this->url("projects/setup/database/"));

   $_session->backend();

   return $r;
  }

 }
?>
