<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/projects.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["include"]."jas.php");

 class Backend_Projects extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->jasRootUser=false;

   $this->projects=Array();

   $this->setup=false;
   $this->subsection=false;
   $this->subsections=Array(
    "directories" => Array("directories", "title" => "Directories", "icon" => "category"),
    "database" => Array("database", "title" => "Database", "icon" => "database"),
    "modules" => Array("modules", "title" => "Modules", "icon" => "module"),
    "config" => Array("configuration", "title" => "Configuration", "icon" => "config"),
    "users" => Array("users", "title" => "Users", "icon" => "user"),
    "plugins" => Array("plugins", "title" => "Plugins", "icon" => "plugin"),
   );
  }

  function main($section=false, $subsection=false)
  {
   global $_session, $_config, $_path;

   $this->jasRootUser=$_config->userRoot(); //get jas project user root status

   //section
   if(!parent::main($section))
    return false;

   if(!$this->section) //default section
   {
    $_session->frontend();
    $this->backend->search("projects"); //inline search all projects
    $_session->backend();

    //search
    if($search=request("search"))
    {
     foreach($this->projects() as $project => $title)
      if(stristr($project, $search)||stristr($title, $search))
       $this->projects[$project]=$title;
    }
    else
     $this->projects=$this->projects();
   }
   else if($this->section=="setup")
   {
    $this->e->clear(); //ignore any project errors

    if($subsection&&$this->jasRootUser)
    {
     if(preg_match("/^\w+$/", $subsection))
     {
      if(!file_exists($_path["project"]["include"]."backend/projects.".$subsection.".php")) //invalid subsection
      {
       $this->e->add("Project setup section: \"".$subsection."\" not found", "error");
       return false;
      }

      //set page as CMS in history
      $hs=new History();
      if(!$hs->cms())
       $this->e->add($hs->e);

      require_once($_path["project"]["include"]."backend/projects.".$subsection.".php");

      $class="Backend_Projects_$subsection";
      $this->setup=new $class($this->backend);

      if(!($r=$this->setup->main()))
      {
       $this->subsection=$subsection;
       return false;
      }

      if($r==="next") //next subsection
      {
       $subsections=array_keys($this->subsections);
       if(($key=array_search($subsection, $subsections))!==false)
       {
        $key++;
        if(!isset($subsections[$key]))
        {
         $this->setup=false;
         $subsection=false;
        }
        else //subsequent subsection
        {
         $subsection=$subsections[$key];

         if(!file_exists($_path["project"]["include"]."backend/projects.".$subsection.".php")) //invalid subsection
         {
          $this->e->add("Project setup section: \"".$subsection."\" not found", "error");
          $this->setup=false;
          return false;
         }

         require_once($_path["project"]["include"]."backend/projects.".$subsection.".php");

         $class="Backend_Projects_$subsection";
         $this->setup=new $class($this->backend);

         if(!($r=$this->setup->main(false))) //don't subit
         {
          $this->subsection=$subsection;
          return false;
         }
        }
       }
      }

      $this->subsection=$subsection;
     }
     else //no (valid) subsection
     {
      $this->e->add("Project setup section: \"".$subsection."\" invalid", "error");
      return false;
     }
    }
   }

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config, $_path;

   $this->backend->setTitle($this->icon("project_dim"),
    $this->link($this->language->t("Projects"), Array("projects/")));

   $current=$_session->get("backendProject");

   //header
   $this->tableRow("projects", true);
   $this->tableColumn("projects", "", false, false, Array("colspan" => 3));

   //data
   foreach($this->projects as $project => $title)
   {
    $this->tableRow("projects", false, ($project==$current?Array("selected"):false));
    $this->tableColumn("projects", $this->icon($_path["projects"].$project."/images/favicon.ico")." ".$title, Array("projects/setup/", Array("backendProject" => $project)), $this->language->t("Setup"));

    //fake set backend project to ensure config (and thus duplicate user modules) isn't loaded
    $_session->setKey($project);
    if(!$_session->get("project")) //project not yet set in session
     $_session->set("project", $project);

    unset($_path["project"]); //reinit project path
    require($_path["include"]."path.php");

    //changelog
    $this->tableColumn("projects", $this->link("", Array("about/changelog/".$project."/", ($project!=$current?Array("backendProject" => $project):false)), $this->language->t("Show changelog for: ".$title), "changelog"), false, false, Array("class" => "cms popup right"));

    //setup menu
    if(!$this->jasRootUser)
     $this->tableColumn("projects", $this->icon("setup"), Array("projects/setup/", Array("backendProject" => $project)), $this->language->t("Setup"), Array("class" => "cms popup right"));
    else
    {
     foreach($this->subsections as $section => $info)
      $this->menuItem($project." setup", $this->language->t($info["title"]), Array("projects/setup/".$section."/", Array("backendProject" => $project)), $this->language->t("Setup ".$info["title"]), $info["icon"]);

     $this->tableColumn("projects", $this->icon("setup").$this->menu($project." setup"), Array("projects/setup/", Array("backendProject" => $project)), $this->language->t("Setup"), Array("class" => "cms popup right"));
    }

    $_session->backend();
   }

   $_session->setBackendProject($current);

   $this->tableRow("projects", false, Array("class" => "total"));

   if(!$this->projects)
    $this->tableColumn("projects", $this->language->t("No projects"), false, false, Array("colspan" => 3));
   else
   {
    $this->tableColumn("projects", $this->language->t("Total"));
    $this->tableColumn("projects", sizeof($this->projects), false, false, Array("class" => "nr", "colspan" => 2));
   }

   $r="<div class=\"twothird\">";
   $r.=$this->table("projects");
   $r.="</div>";

   //new project
   if($this->jasRootUser)
   {
    $r.="<div class=\"third\">\n";
    $r.="<h3>".$this->icon("setup_dim", "small")." ".$this->language->t("New project")."</h3>\n";
    $form=$this->field($this->language->t("Project name"), $this->form->input("text", "backendProject"));
    $form.=$this->field($this->language->t("Project"), $this->form->button("confirm", $this->icon("add", "small")." ".$this->language->t("Create"), false, false, "submit"));
    $r.=$this->form($form, $this->url("projects/setup/directories/"));
    $r.="</div>\n";
   }

   return $r;
  }

  function output_setup()
  {
   global $_session, $_config, $_path;

   $this->backend->setTitle($this->icon("setup_dim"),
    $this->link($this->language->t("Project"), Array("projects/")),
    $this->link($this->language->t("Setup"), Array("projects/setup/")));

   if(!$this->setup)
   {
    $this->tableRow("setup", true);
    $_session->frontend();
    $this->tableColumn("setup", $_config->get("title"), false, false, Array("colspan" => 2));
    $_session->backend();

    foreach($this->subsections as $subsection => $info)
    {
     $this->tableRow("setup", false, Array("class" => "status"));
     $this->tableColumn("setup", $this->icon($info["icon"])." ".$this->language->t($info["title"]), ($this->jasRootUser?Array("projects/setup/".$subsection."/"):false), $this->language->t("Setup ".$info["title"]), Array("class" => "section"));

     if(!file_exists($_path["project"]["include"]."backend/projects.".$subsection.".php")) //unknown subsection
      $this->tableColumn("setup", $this->language->t("Unknown"), false, false, Array("class" => "unknown"));
     else
     {
      require_once($_path["project"]["include"]."backend/projects.".$subsection.".php");

      $text="";
      $status="Backend_Projects_$subsection";
      $status=new $status($this->backend);
      $status=$status->status($text);

      $this->tableColumn("setup", $this->language->t(($text?$text:$status)), false, false, Array("class" => strtolower($status)));
     }
    }

    $r="<div class=\"twothird project setup\">";
    $r.=$this->table("setup");
    $r.="</div>";

				if($this->jasRootUser)
				{
					$r.="<div class=\"third\">\n";
					$r.="<h3>".$this->icon("setup_dim", "small")." ".$this->language->t("New project")."</h3>\n";
					$form=$this->field($this->language->t("Project name"), $this->form->input("text", "backendProject"));
					$form.=$this->field($this->language->t("Project"), $this->form->button("confirm", $this->icon("add", "small")." ".$this->language->t("Create"), false, false, "submit"));
					$r.=$this->form($form, $this->url("projects/setup/directories/"));
					$r.="</div>\n";
				}
   }
   else
   {
    $this->backend->setTitle($this->icon($this->subsections[$this->subsection]["icon"]."_dim"),
     $this->link($this->language->t("Project"), Array("projects/")),
     $this->link($this->language->t("Setup"), Array("projects/setup/")),
     $this->link($this->language->t(ucfirst($this->subsection)), Array("projects/setup/".$this->subsection."/")));

    $r="<div class=\"twothird project setup\">";
    $r.=$this->setup->output();
    $r.="</div>";

				//setup menu
				foreach($this->subsections as $subsection => $info)
					$this->menuItem("setup", $this->language->t($info["title"]), Array("projects/setup/".$subsection."/"), $this->language->t("Setup ".$info["title"]), $info["icon"], ($this->subsection==$subsection?Array("selected"):false));

				$r.="<div class=\"third\">\n";
				$r.="<h3>".$this->icon("setup_dim", "small")." ".$this->language->t("Setup")."</h3>\n";
				$r.=$this->menu("setup");
				$r.="</div>\n";
   }

   return $r;
  }

  function output_quick()
  {
   global $_session, $_path;

   $projects=$this->projects();
   $current=$_session->get("backendProject");

   //determine project link
   if(!$current) //no project selected
    $this->menuItem("projects", $this->language->t("Project"), false, false, "project");
   else //project selected
   {
    //switch to default session
    $key=$_session->getKey();
    $_session->setKey("default");

    if($current==$_session->get("project")) //frontend project (default session project selected)
     $link=$_path["HTML"]["root"];
    else //backend project
     $link=$_path["HTML"]["current"].$_path["projects"];

    $_session->setKey($key); //restore session

    $this->menuItem("projects", $projects[$current], $link, $this->language->t("Open project")." ".$projects[$current], "project", Array("rel" => "external"));
   }


   //list projects
   $this->menuItem("projects", "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; ", Array(false, Array("backendProject" => "")), "", false, (!$current?Array("disabled", "selected"):false));

   foreach($projects as $project => $title)
    $this->menuItem("projects", $title, Array(false, Array("backendProject" => $project)), $title, $_path["projects"].$project."/images/favicon.ico", ($project==$current?Array("disabled", "selected"):false));

   return $this->menu("projects");
  }


  /*
   * Support
   */

  static function projects()
  {
   global $_session, $_Backend_Project_List;

   if($_Backend_Project_List!==NULL)
    return $_Backend_Project_List;

   $_Backend_Project_List=Array();
   foreach(JAS::projects() as $project)
   {
    if($project==$_session->backendKey()) //backend project
     continue; //skip

    if(isset($_SESSION["jses"][$project]["config"]["title"]))
     $_Backend_Project_List[$project]=$_SESSION["jses"][$project]["config"]["title"];
    else
     $_Backend_Project_List[$project]=$project;
   }

   return $_Backend_Project_List;
  }

 }
?>
